package ubiik.utils.view;

import java.util.function.Function;
import javafx.beans.value.ObservableValue;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.input.Clipboard;
import javafx.scene.input.ClipboardContent;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyCodeCombination;
import javafx.scene.input.KeyCombination;
import javafx.scene.input.KeyEvent;

public class TableUtils {
  public static void installCopyPasteHandler(TableView<?> table) {
    table.setOnKeyPressed(new TableKeyEventHandler());
  }
  
  public static class TableKeyEventHandler implements EventHandler<KeyEvent> {
    KeyCodeCombination copyKeyCodeCompination = new KeyCodeCombination(KeyCode.C, new KeyCombination.Modifier[] { KeyCombination.CONTROL_ANY });
    
    public void handle(KeyEvent keyEvent) {
      if (this.copyKeyCodeCompination.match(keyEvent))
        if (keyEvent.getSource() instanceof TableView) {
          TableUtils.copySelectionToClipboard((TableView)keyEvent.getSource());
          keyEvent.consume();
        }  
    }
  }
  
  public static void copySelectionToClipboard(TableView<?> table) {
    StringBuilder clipboardString = new StringBuilder();
    Object obj = table.getSelectionModel().getSelectedItem();
    clipboardString.append(obj.toString());
    ClipboardContent clipboardContent = new ClipboardContent();
    clipboardContent.putString(clipboardString.toString());
    Clipboard.getSystemClipboard().setContent(clipboardContent);
  }
  
  public static <S, T> TableColumn<S, T> column(String title, Function<S, ObservableValue<T>> property, double width) {
    TableColumn<S, T> col = new TableColumn<>(title);
    col.setCellValueFactory(cellData -> (ObservableValue)property.apply(cellData.getValue()));
    col.setPrefWidth(width);
    return col;
  }
}
