package ubiik.wp.ami.controller;

import java.security.InvalidParameterException;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import ubiik.utils.DatatypeConverter;
import ubiik.wp.ami.model.AMIMessage;
import ubiik.wp.ami.model.AmiComparisonResult;
import ubiik.wp.ami.model.AmiMessageFactory;

public class AMITest {
  protected FANTestController fanController;
  
  protected byte[] fan;
  
  protected String name;
  
  protected AMIMessage lastSentMessage;
  
  protected Calendar lastSentMessageTime;
  
  protected List<AMIMessage> messages;
  
  public static final String JSON_KEY_TEST_TYPE = "type";
  
  public static final String JSON_KEY_TEST_TYPE_TIME_NOW = "time";
  
  public static final String JSON_KEY_TEST_TYPE_XAN_LOG = "xanLog";
  
  public static final String JSON_KEY_TEST_TYPE_STRESS_PARALLEL = "stress";
  
  protected int endpoint;
  
  public AMITest(FANTestController fanController, JSONObject json, int endpoint) {
    this.fanController = fanController;
    this.fan = null;
    this.name = "";
    this.messages = new LinkedList<>();
    this.lastSentMessageTime = null;
    this.endpoint = endpoint;
    if (json.containsKey("name"))
      this.name = json.get("name").toString(); 
    loadFromJson(json, this.messages, "messages");
  }
  
  public UUTestResult run() {
    UUTestResult testResult = new UUTestResult(true, "");
    StringBuilder sb = new StringBuilder();
    for (AMIMessage msg : this.messages) {
      if (!testResult.isSuccess())
        break; 
      if (msg.getDirection() == AMIMessage.Direction.out) {
        this.lastSentMessage = msg;
        this.lastSentMessageTime = Calendar.getInstance();
        this.fanController.sendAmiMessage(msg, this.fan, this.endpoint);
        sb.append("Sent ").append(msg.getClass().getSimpleName()).append("\n");
        continue;
      } 
      AMIMessage resp = this.fanController.getAMIMessage(msg.getTimeout(), msg.getMessageId(), msg.getMessageSubType());
      if (resp != null) {
        analyzeResponse(msg, resp, sb, testResult);
        continue;
      } 
      testResult.setSuccess(false);
      byte[] msgType = new byte[1];
      msgType[0] = msg.getMessageId();
      sb.append("No message with type ").append(DatatypeConverter.printHexBinary(msgType))
        .append(" received within ").append(msg.getTimeout()).append("ms\n");
    } 
    testResult.setMessage(sb.toString());
    return testResult;
  }
  
  protected void analyzeResponse(AMIMessage msg, AMIMessage resp, StringBuilder sb, UUTestResult testResult) {
    sb.append("Received ").append(resp.getClass().getSimpleName()).append("\n");
    AmiComparisonResult compRes = msg.amiCompare(resp);
    if (!compRes.equal) {
      sb.append(compRes.message);
      testResult.setSuccess(false);
    } 
  }
  
  public static void loadFromJson(JSONObject json, List<AMIMessage> messages, String jsonMsgKey) {
    JSONArray jArr = (JSONArray)json.get(jsonMsgKey);
    for (int i = 0; i < jArr.size(); i++) {
      JSONObject jMsg = (JSONObject)jArr.get(i);
      AMIMessage amiMsg = AmiMessageFactory.createAmiMessage(jMsg);
      if (amiMsg != null) {
        messages.add(amiMsg);
      } else {
        throw new InvalidParameterException("Invalid Json configuration: " + jMsg.toJSONString());
      } 
    } 
  }
  
  public byte[] getFan() {
    return this.fan;
  }
  
  public void setFan(byte[] uuid) {
    this.fan = uuid;
  }
  
  public String getName() {
    return this.name;
  }
}
