package ubiik.wp.ami.controller;

import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;
import ubiik.wp.ami.model.AMIMessage;

public class AmiParallelStressTest extends AMITest {
  public static final String JSON_KEY_REPEAT_COUNT = "repeat";
  
  public static final String JSON_KEY_STRESS_MESSAGES = "stressMessages";
  
  protected List<AMIMessage> stressMessages;
  
  private Integer repeat;
  
  private AtomicBoolean runParallelThread;
  
  public AmiParallelStressTest(FANTestController fanController, JSONObject json, int endpoint) {
    super(fanController, json, endpoint);
    this.runParallelThread = new AtomicBoolean(false);
    this.repeat = JsonUtils.getIntIfExists(json, "repeat");
    this.stressMessages = new LinkedList<>();
    loadFromJson(json, this.stressMessages, "stressMessages");
  }
  
  public UUTestResult run() {
    Thread stressThread = new Thread(new Runnable() {
          public void run() {
            AmiParallelStressTest.this.runStress();
          }
        });
    stressThread.start();
    UUTestResult res = super.run();
    this.runParallelThread.set(false);
    return res;
  }
  
  public void runStress() {
    int count = 0;
    boolean continueCount = true;
    if (this.repeat != null)
      count = this.repeat.intValue(); 
    this.runParallelThread.set(true);
    while (this.runParallelThread.get()) {
      for (AMIMessage msg : this.stressMessages) {
        if (msg.getDirection() == AMIMessage.Direction.out) {
          this.fanController.sendAmiMessage(msg, this.fan, this.endpoint);
          continue;
        } 
        System.err.println("Stress thread can only send messages");
      } 
      continueCount = true;
      count--;
      if (this.repeat != null && count <= 0)
        continueCount = false; 
      this.runParallelThread.set(this.runParallelThread.get() & continueCount);
    } 
  }
}
