package ubiik.wp.ami.model;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiError extends AMIMessage {
  private Integer errorCode;
  
  public static final int AMI_METER_ERR_SIZE = 2;
  
  public AmiError(byte[] bytes, AMIMessage.Direction dir) {
    super(bytes, dir);
    if (bytes.length < 4)
      throw new IllegalArgumentException("Data is too small for AmiError"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.errorCode = Integer.valueOf(0xFFFF & buf.getShort());
  }
  
  public AmiError(byte[] bytes) {
    this(bytes, AMIMessage.Direction.in);
  }
  
  public AmiError(JSONObject json) {
    super(json);
    this.errorCode = JsonUtils.getIntIfExists(json, "errorCode");
    if (this.errorCode != null) {
      ByteBuffer buffer = ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN);
      buffer.putShort(this.errorCode.shortValue());
      try {
        this.payload.write(buffer.array());
      } catch (IOException e) {
        e.printStackTrace();
      } 
    } 
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiError otherError = (AmiError)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.errorCode, otherError.getErrorCode(), success, message, "errorCode didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getErrorCode() {
    return this.errorCode;
  }
}
