package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiFanParameters extends AMIMessage {
  public static final int AMI_FAN_PARAMS_SIZE = 14;
  
  private Integer opt;
  
  private Integer pollPeriodSec;
  
  private Integer connectPeriodSec;
  
  private Long setTimePeriod;
  
  private Integer timeTolerance;
  
  private Integer initialDelay;
  
  public AmiFanParameters(byte[] bytes) {
    super(bytes);
    if (bytes.length < 16)
      throw new IllegalArgumentException("Data is too small for AmiFanParameters"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.opt = Integer.valueOf(buf.getInt());
    this.pollPeriodSec = Integer.valueOf(0xFFFF & buf.getShort());
    this.connectPeriodSec = Integer.valueOf(0xFFFF & buf.getShort());
    this.setTimePeriod = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    this.timeTolerance = Integer.valueOf(0xFFFF & buf.getShort());
    this.initialDelay = Integer.valueOf(0xFF & buf.get());
  }
  
  public AmiFanParameters(JSONObject json) {
    super(json);
    this.opt = JsonUtils.getIntIfExists(json, "opt");
    this.pollPeriodSec = JsonUtils.getIntIfExists(json, "pollPeriodSec");
    this.connectPeriodSec = JsonUtils.getIntIfExists(json, "connectPeriodSec");
    this.setTimePeriod = JsonUtils.getLongIfExists(json, "setTimePeriod");
    this.timeTolerance = JsonUtils.getIntIfExists(json, "timeTolerance");
    this.initialDelay = JsonUtils.getIntIfExists(json, "initialDelay");
    try {
      ByteBuffer buf = ByteBuffer.allocate(14).order(ByteOrder.LITTLE_ENDIAN);
      buf.putInt(this.opt.intValue());
      buf.putShort(this.pollPeriodSec.shortValue());
      buf.putShort(this.connectPeriodSec.shortValue());
      buf.putInt(this.setTimePeriod.intValue());
      buf.putShort(this.timeTolerance.shortValue());
      buf.put(this.initialDelay.byteValue());
      this.payload.write(buf.array());
    } catch (Exception e) {
      e.printStackTrace();
      throw new IllegalArgumentException("Error writing data");
    } 
  }
  
  public Integer getOpt() {
    return this.opt;
  }
  
  public Long getSetTimePeriod() {
    return this.setTimePeriod;
  }
  
  public Integer getTimeTolerance() {
    return this.timeTolerance;
  }
  
  public Integer getPollPeriodSec() {
    return this.pollPeriodSec;
  }
  
  public Integer getConnectPeriodSec() {
    return this.connectPeriodSec;
  }
  
  public Integer getInitialDelay() {
    return this.initialDelay;
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiFanParameters params = (AmiFanParameters)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.opt, params.getOpt(), success, message, "opt didn't match\n");
    success = compareIfNotNull(this.pollPeriodSec, params.getPollPeriodSec(), success, message, "pollPeriodSec didn't match\n");
    success = compareIfNotNull(this.connectPeriodSec, params.getConnectPeriodSec(), success, message, "connectPeriodSec didn't match\n");
    success = compareIfNotNull(this.setTimePeriod, params.getSetTimePeriod(), success, message, "setTimePeriod didn't match\n");
    success = compareIfNotNull(this.timeTolerance, params.getTimeTolerance(), success, message, "timeTolerance didn't match\n");
    success = compareIfNotNull(this.initialDelay, params.getInitialDelay(), success, message, "initialDelay didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
}
