package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiHanParameters extends AMIMessage {
  public static final int AMI_HAN_PARAMS_REQ_SIZE = 4;
  
  public static final int AMI_HAN_PARAMS_SIZE = 10;
  
  private Integer qos;
  
  private short[] request;
  
  public AmiHanParameters(byte[] bytes) {
    super(bytes);
    if (bytes.length < 12)
      throw new IllegalArgumentException("Data is too small for AmiHanParameters"); 
    this.request = new short[4];
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.qos = Integer.valueOf(0xFFFF & buf.getShort());
    for (int i = 0; i < 4; i++)
      this.request[i] = buf.getShort(); 
  }
  
  public AmiHanParameters(JSONObject json) {
    super(json);
    this.qos = JsonUtils.getIntIfExists(json, "qos");
    byte[] reqArr = JsonUtils.getByteArr(json, "request");
    if (reqArr != null) {
      ByteBuffer buf = ByteBuffer.wrap(reqArr).order(ByteOrder.LITTLE_ENDIAN);
      this.request = new short[4];
      for (int i = 0; i < 4; i++)
        this.request[i] = buf.getShort(); 
    } else {
      this.request = null;
    } 
    try {
      ByteBuffer buf = ByteBuffer.allocate(10).order(ByteOrder.LITTLE_ENDIAN);
      buf.putShort(this.qos.shortValue());
      buf.put(reqArr);
      this.payload.write(buf.array());
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }
  
  public Integer getQos() {
    return this.qos;
  }
  
  public short[] getRequest() {
    return this.request;
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiHanParameters params = (AmiHanParameters)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.qos, params.getQos(), success, message, "qos didn't match\n");
    short[] otherReqs = params.getRequest();
    for (int i = 0; i < 4; i++)
      success = compareIfNotNull(Short.valueOf(this.request[i]), Short.valueOf(otherReqs[i]), success, message, "request " + i + " didn't match\n"); 
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
}
