package ubiik.wp.ami.model;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiLoadProfile extends AMIMessage {
  public static final int AMI_LOAD_PROFILE_MIN_SIZE = 7;
  
  private Integer recordNumber;
  
  private Integer amiTime;
  
  private Integer status;
  
  private byte[] values;
  
  public AmiLoadProfile(JSONObject json) {
    super(json);
    this.recordNumber = JsonUtils.getIntIfExists(json, "recordNumber");
    this.amiTime = JsonUtils.getIntIfExists(json, "time");
    this.status = JsonUtils.getIntIfExists(json, "status");
    this.values = JsonUtils.getByteArr(json, "values");
    try {
      ByteBuffer buf = ByteBuffer.allocate(7).order(ByteOrder.LITTLE_ENDIAN);
      buf.putShort(this.recordNumber.shortValue());
      buf.putInt(this.amiTime.intValue());
      buf.put(this.status.byteValue());
      this.payload.write(buf.array());
      this.payload.write(this.values);
      if (this.values != null)
        this.payload.write(this.values); 
    } catch (IOException e) {
      e.printStackTrace();
    } 
  }
  
  public AmiLoadProfile(byte[] bytes) {
    super(bytes);
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.recordNumber = Integer.valueOf(0xFFFF & buf.getShort());
    this.amiTime = Integer.valueOf(buf.getInt());
    this.status = Integer.valueOf(0xFF & buf.get());
    this.values = new byte[bytes.length - buf.position()];
    buf.get(this.values);
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiLoadProfile lp = (AmiLoadProfile)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.recordNumber, lp.getRecordNumber(), success, message, "recordNumber didn't match\n");
    success = compareIfNotNull(this.amiTime, lp.getAmiTime(), success, message, "time didn't match\n");
    success = compareIfNotNull(this.status, lp.getStatus(), success, message, "status didn't match\n");
    success = compareIfNotNull(this.values, lp.getValues(), success, message, "values didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getRecordNumber() {
    return this.recordNumber;
  }
  
  public Integer getAmiTime() {
    return this.amiTime;
  }
  
  public Integer getStatus() {
    return this.status;
  }
  
  public byte[] getValues() {
    return this.values;
  }
}
