package ubiik.wp.ami.model;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;
import org.json.simple.JSONObject;
import ubiik.utils.DatatypeConverter;
import ubiik.wp.model.messages.PSUplink;

public class AmiMessageFactory {
  private static Map<Byte, Class<?>> amiMsgsClasses = new HashMap<>();
  
  static {
    amiMsgsClasses.put(Byte.valueOf((byte)0), AmiMeterKeys.class);
    amiMsgsClasses.put(Byte.valueOf((byte)1), AmiMeterKeys.class);
    amiMsgsClasses.put(Byte.valueOf((byte)2), AmiGet.class);
    amiMsgsClasses.put(Byte.valueOf((byte)3), AmiSet.class);
    amiMsgsClasses.put(Byte.valueOf((byte)4), AmiSet.class);
    amiMsgsClasses.put(Byte.valueOf((byte)5), AmiGet.class);
    amiMsgsClasses.put(Byte.valueOf((byte)6), AmiSet.class);
    amiMsgsClasses.put(Byte.valueOf((byte)7), AmiSet.class);
    amiMsgsClasses.put(Byte.valueOf((byte)16), AmiRawDlms.class);
    amiMsgsClasses.put(Byte.valueOf((byte)17), AmiRawHdlc.class);
    amiMsgsClasses.put(Byte.valueOf((byte)32), AmiGetMeterDetails.class);
    amiMsgsClasses.put(Byte.valueOf((byte)33), AmiGetXanLog.class);
    amiMsgsClasses.put(Byte.valueOf((byte)48), AmiGetFanParameters.class);
    amiMsgsClasses.put(Byte.valueOf((byte)49), AmiFanParameters.class);
    amiMsgsClasses.put(Byte.valueOf((byte)50), AmiGetHanParameters.class);
    amiMsgsClasses.put(Byte.valueOf((byte)51), AmiHanParameters.class);
    amiMsgsClasses.put(Byte.valueOf((byte)65), AmiSetMeterTime.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-128), AmiLoadProfile.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-127), AmiRawAxdr.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-126), AmiRawAxdr.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-125), AmiEvent.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-124), AmiError.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-118), AmiResponse.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-117), AmiResponse.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-116), AmiResponse.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-112), AmiResponse.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-111), AmiRawHdlc.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-96), AmiMeterDiscovery.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-95), AmiNotification.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-94), AmiMeterDetails.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-93), AmiXanLog.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-92), AmiFanParameters.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-91), AmiHanParameters.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-2), AmiATCommand.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-3), AmiCustom.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-4), AmiSleep.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-5), AmiPSCommand.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-6), AmiBSConnect.class);
    amiMsgsClasses.put(Byte.valueOf((byte)-7), AmiRandomPayload.class);
  }
  
  public static AMIMessage createAmiMessage(PSUplink uplik) {
    AMIMessage amiMsg = createAmiMessage(uplik.getApplicationData());
    if (amiMsg != null)
      amiMsg.setUplink(uplik); 
    return amiMsg;
  }
  
  public static AMIMessage createAmiMessage(byte[] data) {
    AMIMessage amiMsg = null;
    if (data.length > 2) {
      byte applicationId = data[0];
      if (applicationId == 20) {
        byte messageId = data[1];
        try {
          Class<?> cls = amiMsgsClasses.get(Byte.valueOf(messageId));
          if (cls != null) {
            Constructor<?> cons = cls.getConstructor(new Class[] { byte[].class });
            amiMsg = (AMIMessage)cons.newInstance(new Object[] { data });
          } else {
            System.out.println("Not recognized:");
            System.out.println(DatatypeConverter.printHexBinary(data));
          } 
        } catch (Exception e) {
          System.out.println(DatatypeConverter.printHexBinary(data));
          e.printStackTrace();
        } 
      } 
    } 
    return amiMsg;
  }
  
  public static AMIMessage createAmiMessage(JSONObject json) {
    AMIMessage amiMsg = null;
    try {
      byte messageId = AMIMessage.getMessaIdForKey((String)json.get("type"));
      Class<?> cls = amiMsgsClasses.get(Byte.valueOf(messageId));
      if (cls != null) {
        Constructor<?> cons = cls.getConstructor(new Class[] { JSONObject.class });
        amiMsg = (AMIMessage)cons.newInstance(new Object[] { json });
      } else {
        System.out.println("Not recognized:");
        System.out.println(json.toJSONString());
      } 
    } catch (Exception e) {
      System.out.println(json.toJSONString());
      e.printStackTrace();
    } 
    return amiMsg;
  }
}
