package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiMeterDetails extends AMIMessage {
  public static final int DEVICE_NAME_SIZE = 16;
  
  public static final int DISTRICT_ID_SIZE = 8;
  
  public static final int CUSTOMER_ID_SIZE = 8;
  
  public static final int METER_TYPE_CODE_SIZE = 6;
  
  public static final int MIN_DETAILS_SIZE = 48;
  
  private byte[] deviceName;
  
  private byte[] districtId;
  
  private byte[] customerId;
  
  private Integer x;
  
  private Integer y;
  
  private byte[] meterTypeCode;
  
  private Byte fanType;
  
  private Byte hanType;
  
  String firmwareInfo;
  
  public AmiMeterDetails(byte[] bytes) {
    super(bytes);
    if (bytes.length < 50)
      throw new IllegalArgumentException("Data is too small for AmiMeterDetails"); 
    this.deviceName = new byte[16];
    this.districtId = new byte[8];
    this.customerId = new byte[8];
    this.meterTypeCode = new byte[6];
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    buf.get(this.deviceName, 0, 16);
    buf.get(this.districtId, 0, 8);
    buf.get(this.customerId, 0, 8);
    this.x = Integer.valueOf(buf.getInt());
    this.y = Integer.valueOf(buf.getInt());
    buf.get(this.meterTypeCode, 0, 6);
    this.fanType = Byte.valueOf(buf.get());
    this.hanType = Byte.valueOf(buf.get());
    byte[] fwInfoChars = new byte[bytes.length - buf.position()];
    buf.get(fwInfoChars);
    this.firmwareInfo = new String(fwInfoChars);
  }
  
  public AmiMeterDetails(JSONObject json) {
    super(json);
    this.deviceName = JsonUtils.getByteArr(json, "deviceName");
    this.districtId = JsonUtils.getByteArr(json, "districtId");
    this.customerId = JsonUtils.getByteArr(json, "customerId");
    this.meterTypeCode = JsonUtils.getByteArr(json, "meterTypeCode");
    this.x = JsonUtils.getIntIfExists(json, "x");
    this.y = JsonUtils.getIntIfExists(json, "y");
    this.fanType = JsonUtils.getByteIfExists(json, "fanType");
    this.hanType = JsonUtils.getByteIfExists(json, "hanType");
    this.firmwareInfo = (String)JsonUtils.getIfExists(json, "firmwareVersion");
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiMeterDetails details = (AmiMeterDetails)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.deviceName, details.getDeviceName(), success, message, "deviceName didn't match\n");
    success = compareIfNotNull(this.districtId, details.getDistrictId(), success, message, "districtId didn't match\n");
    success = compareIfNotNull(this.customerId, details.getCustomerId(), success, message, "customerId didn't match\n");
    success = compareIfNotNull(this.x, Integer.valueOf(details.getX()), success, message, "x didn't match\n");
    success = compareIfNotNull(this.y, Integer.valueOf(details.getY()), success, message, "y didn't match\n");
    success = compareIfNotNull(this.meterTypeCode, details.getMeterTypeCode(), success, message, "meterTypeCode didn't match\n");
    success = compareIfNotNull(this.fanType, Byte.valueOf(details.getFanType()), success, message, "fanType didn't match\n");
    success = compareIfNotNull(this.hanType, Byte.valueOf(details.getHanType()), success, message, "hanType didn't match\n");
    success = compareIfNotNull(this.firmwareInfo, details.getFirmwareInfo(), success, message, "firmwareInfo didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public byte[] getDeviceName() {
    return this.deviceName;
  }
  
  public byte[] getDistrictId() {
    return this.districtId;
  }
  
  public byte[] getCustomerId() {
    return this.customerId;
  }
  
  public int getX() {
    return this.x.intValue();
  }
  
  public int getY() {
    return this.y.intValue();
  }
  
  public byte[] getMeterTypeCode() {
    return this.meterTypeCode;
  }
  
  public byte getFanType() {
    return this.fanType.byteValue();
  }
  
  public byte getHanType() {
    return this.hanType.byteValue();
  }
  
  public String getFirmwareInfo() {
    return this.firmwareInfo;
  }
}
