package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiRawAxdr extends AMIMessage {
  public static final int MIN_SIZE_AMI_RAW_AXDR = 1;
  
  private Integer reg;
  
  private byte[] axdr;
  
  public AmiRawAxdr(byte[] bytes) {
    super(bytes);
    if (bytes.length < 3)
      throw new IllegalArgumentException("Data is too small for AmiRawAxdr"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.reg = Integer.valueOf(0xFF & buf.get());
    this.messageSubType = Byte.valueOf(this.reg.byteValue());
    this.axdr = new byte[bytes.length - buf.position()];
    buf.get(this.axdr);
  }
  
  public AmiRawAxdr(JSONObject json) {
    super(json);
    this.reg = JsonUtils.getIntIfExists(json, "reg");
    this.axdr = JsonUtils.getByteArr(json, "axdr");
    if (this.reg != null)
      this.messageSubType = Byte.valueOf(this.reg.byteValue()); 
    try {
      this.payload.write(this.reg.byteValue());
      if (this.axdr != null)
        this.payload.write(this.axdr); 
    } catch (Exception e) {
      System.err.println("Message" + getClass().getSimpleName() + "is incomplete to create payload");
    } 
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiRawAxdr resp = (AmiRawAxdr)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.reg, resp.getReg(), success, message, "reg didn't match\n");
    success = compareIfNotNull(this.axdr, resp.getAxdr(), success, message, "axdr didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getReg() {
    return this.reg;
  }
  
  public byte[] getAxdr() {
    return this.axdr;
  }
}
