package ubiik.wp.ami.model;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiRawHdlc extends AMIMessage {
  private Integer requestId;
  
  private byte[] hdlcPdu;
  
  public AmiRawHdlc(JSONObject json) {
    super(json);
    this.requestId = JsonUtils.getIntIfExists(json, "requestId");
    this.hdlcPdu = JsonUtils.getByteArr(json, "hdlcPdu");
    try {
      this.payload.write(this.requestId.byteValue());
      if (this.hdlcPdu != null)
        this.payload.write(this.hdlcPdu); 
    } catch (IOException e) {
      e.printStackTrace();
    } 
  }
  
  public AmiRawHdlc(byte[] bytes) {
    super(bytes);
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.requestId = Integer.valueOf(0xFF & buf.get());
    this.hdlcPdu = new byte[bytes.length - buf.position()];
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiRawHdlc resp = (AmiRawHdlc)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.requestId, resp.getRequestId(), success, message, "requestId didn't match\n");
    success = compareIfNotNull(this.hdlcPdu, resp.getHdlcPdu(), success, message, "hdlcPdu didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getRequestId() {
    return this.requestId;
  }
  
  public byte[] getHdlcPdu() {
    return this.hdlcPdu;
  }
}
