package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiXanLog extends AMIMessage {
  public static final int XAN_LOG_SIZE = 25;
  
  private Long to;
  
  private Long from;
  
  private Long toBytes;
  
  private Long fromBytes;
  
  private Long blockedFrames;
  
  private Long hdlcErrors;
  
  private Byte xanType;
  
  public AmiXanLog(byte[] bytes) {
    super(bytes);
    if (bytes.length < 27)
      throw new IllegalArgumentException("Data is too small for AmiXanLog"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.to = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    this.from = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    this.toBytes = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    this.fromBytes = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    this.blockedFrames = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    this.hdlcErrors = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    this.xanType = Byte.valueOf(buf.get());
  }
  
  public AmiXanLog(JSONObject json) {
    super(json);
    this.to = JsonUtils.getLongIfExists(json, "to");
    this.from = JsonUtils.getLongIfExists(json, "from");
    this.toBytes = JsonUtils.getLongIfExists(json, "toBytes");
    this.fromBytes = JsonUtils.getLongIfExists(json, "fromBytes");
    this.blockedFrames = JsonUtils.getLongIfExists(json, "blockedFrames");
    this.hdlcErrors = JsonUtils.getLongIfExists(json, "hdlcErrors");
    this.xanType = JsonUtils.getByteIfExists(json, "xanType");
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiXanLog xanLog = (AmiXanLog)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    System.out.println("XanLog To bytes: " + this.toBytes + " From bytes: " + this.fromBytes);
    boolean toBytesLarger = (this.toBytes.longValue() > xanLog.toBytes.longValue());
    if (!toBytesLarger)
      message.append("TO: Expected more than ").append(xanLog.toBytes)
        .append(" bytes but received ").append(this.toBytes).append("\n"); 
    success = Boolean.valueOf(success.booleanValue() & toBytesLarger);
    boolean fromBytesLarger = (this.fromBytes.longValue() > xanLog.fromBytes.longValue());
    if (!fromBytesLarger)
      message.append("FROM: Expected more than ").append(xanLog.fromBytes)
        .append(" bytes but received ").append(this.fromBytes).append("\n"); 
    success = Boolean.valueOf(success.booleanValue() & fromBytesLarger);
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Long getBlockedFrames() {
    return this.blockedFrames;
  }
  
  public void setBlockedFrames(Long blockedFrames) {
    this.blockedFrames = blockedFrames;
  }
  
  public Long getTo() {
    return this.to;
  }
  
  public Long getFrom() {
    return this.from;
  }
  
  public Long getToBytes() {
    return this.toBytes;
  }
  
  public Long getFromBytes() {
    return this.fromBytes;
  }
  
  public Long getHdlcErrors() {
    return this.hdlcErrors;
  }
  
  public Byte getXanType() {
    return this.xanType;
  }
}
