package ubiik.wp.controller;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import ubiik.wp.model.devices.UUEndDevice;
import ubiik.wp.model.messages.PSPong;
import ubiik.wp.model.messages.pscommands.PSCommandPingEndDevice;

public class PingManager {
  private static final int DEFAULT_MAX_STORED_PINGS = 1024;
  
  private Map<UUEndDevice, LinkedHashMap<Integer, PSCommandPingEndDevice>> pings;
  
  private Map<UUEndDevice, LinkedHashMap<Integer, PSPong>> pongs;
  
  private long pingCount;
  
  private int maxPingPerDevice;
  
  private DevicesController devicesController;
  
  public PingManager(DevicesController devicesController, int maxStoredPingsPerDevice) {
    this.maxPingPerDevice = maxStoredPingsPerDevice;
    this.pings = new HashMap<>();
    this.pongs = new HashMap<>();
    this.devicesController = devicesController;
  }
  
  public PingManager(DevicesController devicesController) {
    this(devicesController, 1024);
  }
  
  private final LinkedHashMap<Integer, PSPong> getOrCreatePongsForEd(UUEndDevice ed) {
    LinkedHashMap<Integer, PSPong> pongsForEd = this.pongs.get(ed);
    if (pongsForEd == null) {
      pongsForEd = new LinkedHashMap<>();
      this.pongs.put(ed, pongsForEd);
    } 
    return pongsForEd;
  }
  
  private final LinkedHashMap<Integer, PSCommandPingEndDevice> getOrCreatePingsForEd(UUEndDevice ed) {
    LinkedHashMap<Integer, PSCommandPingEndDevice> pingsForEd = this.pings.get(ed);
    if (pingsForEd == null) {
      pingsForEd = new LinkedHashMap<>();
      this.pings.put(ed, pingsForEd);
    } 
    return pingsForEd;
  }
  
  public synchronized void addPong(PSPong pong) {
    UUEndDevice ed = this.devicesController.getDeviceForId(pong.getUueid());
    if (ed != null) {
      LinkedHashMap<Integer, PSPong> pongsForEd = getOrCreatePongsForEd(ed);
      pongsForEd.put(Integer.valueOf(pong.getPingId()), pong);
      PSCommandPingEndDevice ping = getOrCreatePingsForEd(ed).get(Integer.valueOf(pong.getPingId()));
      if (ping != null) {
        ping.setPong(pong);
        pong.setPing(ping);
      } 
      cleanPongs(pongsForEd);
      this.devicesController.setLastPongForDevice(pong);
    } 
  }
  
  public synchronized void addPing(PSCommandPingEndDevice ping) {
    UUEndDevice ed = this.devicesController.getDeviceForId(ping.getTarget());
    if (ed != null) {
      LinkedHashMap<Integer, PSCommandPingEndDevice> pingsForEd = getOrCreatePingsForEd(ed);
      pingsForEd.put(Integer.valueOf(ping.getPingId()), ping);
      PSPong pong = getOrCreatePongsForEd(ed).get(Integer.valueOf(ping.getPingId()));
      if (pong != null) {
        ping.setPong(pong);
        pong.setPing(ping);
      } 
      cleanPings(pingsForEd);
    } 
  }
  
  private void cleanPings(LinkedHashMap<Integer, PSCommandPingEndDevice> pingsForEd) {
    if (pingsForEd.size() > this.maxPingPerDevice) {
      Map.Entry<Integer, PSCommandPingEndDevice> entry = pingsForEd.entrySet().iterator().next();
      if (entry != null)
        pingsForEd.remove(entry.getKey()); 
    } 
  }
  
  private void cleanPongs(LinkedHashMap<Integer, PSPong> pongsForEd) {
    if (pongsForEd.size() > this.maxPingPerDevice) {
      Map.Entry<Integer, PSPong> entry = pongsForEd.entrySet().iterator().next();
      if (entry != null)
        pongsForEd.remove(entry.getKey()); 
    } 
  }
  
  public synchronized long getNextPingId() {
    this.pingCount++;
    return this.pingCount;
  }
}
