package ubiik.wp.demo.controller;

import java.util.Date;
import java.util.Map;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.chart.Axis;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.NumberAxis;
import javafx.scene.chart.XYChart;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.layout.Pane;
import ubiik.utils.view.DateAxis;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.view.UplinkSamplesViewController;

public abstract class UplinksDemoViewController extends UplinkSamplesViewController {
  protected static final int MAX_SAMPLES_IN_LIST = 400;
  
  @FXML
  protected Pane chartsContainer;
  
  @FXML
  protected Label labelTitle;
  
  @FXML
  protected CheckBox cbUseOneChart;
  
  public UplinksDemoViewController(DevicesController devicesController, MessageController messageController) {
    super(messageController, devicesController);
  }
  
  protected LineChart<Date, Number> createDateAxisChart(ObservableList<XYChart.Series<Date, Number>> series) {
    NumberAxis numberAxis = new NumberAxis();
    DateAxis dateAxis = new DateAxis();
    LineChart<Date, Number> lineChart = new LineChart<>((Axis<Date>)dateAxis, numberAxis);
    lineChart.setData(series);
    return lineChart;
  }
  
  public void setTitle(String title) {
    Platform.runLater(() -> this.labelTitle.setText(title));
  }
  
  protected boolean addSample(Date sampleTime, int sampleValue, String uuid, Map<String, ObservableList<XYChart.Data<Date, Number>>> seriesDataMap, ObservableList<XYChart.Series<Date, Number>> series) {
    boolean createdSeries = false;
    if (seriesDataMap.get(uuid) == null) {
      ObservableList<XYChart.Data<Date, Number>> observableList = FXCollections.observableArrayList();
      seriesDataMap.put(uuid, observableList);
      series.add(new XYChart.Series<>(uuid, observableList));
      createdSeries = true;
    } 
    ObservableList<XYChart.Data<Date, Number>> seriesData = seriesDataMap.get(uuid);
    seriesData.add(new XYChart.Data<>(sampleTime, Integer.valueOf(sampleValue)));
    if (seriesData.size() > 400)
      seriesData.remove(0); 
    return createdSeries;
  }
  
  protected LineChart<Date, Number> setUpLineChart(LineChart<Date, Number> lineChart, ObservableList<XYChart.Series<Date, Number>> series, String title) {
    if (lineChart != null)
      this.chartsContainer.getChildren().remove(lineChart); 
    lineChart = createDateAxisChart(series);
    lineChart.setAnimated(true);
    lineChart.setTitle(title);
    this.chartsContainer.getChildren().add(lineChart);
    return lineChart;
  }
}
