package ubiik.wp.model.capacity;

import java.util.concurrent.atomic.AtomicBoolean;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.devices.UUEndDevice;

public abstract class MessageCapacityTest implements Runnable {
  public static final int MIN_INTERVAL_MILLIS = 1000;
  
  public static final int MAX_INTERVAL_MILLIS = 2147483647;
  
  public static final int MAX_NUMBER_MSGS = 2147483647;
  
  public static final int DEFAULT_MESSAGES_TO_SEND = 0;
  
  public static final String[] CHANNEL_NAMES = new String[] { "Data ACK", "Data UNACK", "Control" };
  
  public static final int CHANNEL_DATA_ACK = 0;
  
  public static final int CHANNEL_DATA_UNACK = 1;
  
  public static final int CHANNEL_CONTROL = 2;
  
  private int interval;
  
  private int dataSize;
  
  private UUEndDevice endDevice;
  
  protected AtomicBoolean running;
  
  private MessageController messageController;
  
  private long counter;
  
  private long messagesToSend;
  
  public MessageCapacityTest(int frequency, int dataSize, UUEndDevice endDevice, MessageController messageController, long messagesToSend) {
    this.interval = frequency;
    this.dataSize = dataSize;
    this.endDevice = endDevice;
    this.messageController = messageController;
    this.counter = 0L;
    this.messagesToSend = messagesToSend;
    this.running = new AtomicBoolean(false);
  }
  
  public MessageCapacityTest(int frequency, int dataSize, UUEndDevice endDevice, MessageController messageController) {
    this(frequency, dataSize, endDevice, messageController, 0L);
  }
  
  protected abstract void sendMessage();
  
  public abstract String getChannelName();
  
  public abstract int getChannelCode();
  
  public void run() {
    this.running.set(true);
    this.counter = 0L;
    while (this.running.get()) {
      sendMessage();
      this.counter++;
      if (this.messagesToSend > 0L && this.counter >= this.messagesToSend)
        stop(); 
      try {
        Thread.sleep(this.interval);
      } catch (InterruptedException e) {
        e.printStackTrace();
      } 
    } 
  }
  
  public void stop() {
    this.running.set(false);
  }
  
  public boolean isRunning() {
    return this.running.get();
  }
  
  public int getInterval() {
    return this.interval;
  }
  
  public int getDataSize() {
    return this.dataSize;
  }
  
  public UUEndDevice getEndDevice() {
    return this.endDevice;
  }
  
  public MessageController getMessageController() {
    return this.messageController;
  }
  
  public long getCounter() {
    return this.counter;
  }
  
  public long getMessagesToSend() {
    return this.messagesToSend;
  }
}
