package ubiik.wp.model.capacity;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.security.SecureRandom;
import java.util.Arrays;
import java.util.Date;
import ubiik.utils.CRC;
import ubiik.wp.model.messages.PSUplink;

public class UplinkCapacitySample {
  private static final int BYTES_FOR_APP_ID = 2;
  
  private static final int BYTES_FOR_SEQ_NUMBER = 1;
  
  private static final int BYTES_FOR_CRC = 2;
  
  public static final int NON_PAYLOAD_BYTES = 5;
  
  public static final int MAX_DATA_SIZE = 65535;
  
  private static final int MIN_DATA_SIZE = 0;
  
  public static final int MAX_SEQ_NUMBER = 256;
  
  public static final int MIN_SEQ_NUMBER = 0;
  
  private static final byte APP_ID = 20;
  
  private static final byte SUB_APP_ID = 0;
  
  private PSUplink uplink;
  
  private int seqNumber;
  
  private byte[] data;
  
  private boolean isValid;
  
  public UplinkCapacitySample(PSUplink uplink, int seqNumber, byte[] data, boolean isValid) {
    this.uplink = uplink;
    this.seqNumber = seqNumber;
    this.data = data;
    this.isValid = isValid;
  }
  
  public static UplinkCapacitySample createUplinkCapacitySampleFromUplink(PSUplink uplink) {
    UplinkCapacitySample ulCapacitySample = null;
    byte[] ulData = uplink.getApplicationData();
    try {
      ByteBuffer buff = ByteBuffer.wrap(ulData).order(ByteOrder.LITTLE_ENDIAN);
      byte appId = buff.get();
      byte subAppId = buff.get();
      if (appId == 20 && subAppId == 0) {
        int dataSize = ulData.length - 5;
        int seqNumber = buff.get() & 0xFF;
        boolean validPayload = false;
        byte[] data = new byte[dataSize];
        buff.get(data);
        short crc = buff.getShort();
        short dataCRC = CRC.crc16(Arrays.copyOf(ulData, ulData.length - 2));
        validPayload = (dataCRC == crc);
        ulCapacitySample = new UplinkCapacitySample(uplink, seqNumber, data, validPayload);
      } 
    } catch (Exception exception) {}
    return ulCapacitySample;
  }
  
  public static byte[] createPayload(int dataSize, int seqNumber, boolean isSizeTotal) {
    if (isSizeTotal)
      dataSize -= 5; 
    if (dataSize > 65535 || dataSize < 0 || seqNumber >= 256 || seqNumber < 0)
      throw new IllegalArgumentException("invalid parameters"); 
    byte[] data = new byte[dataSize];
    SecureRandom r = new SecureRandom();
    r.nextBytes(data);
    ByteBuffer buffCRC = ByteBuffer.allocate(5 + dataSize - 2).order(ByteOrder.LITTLE_ENDIAN);
    buffCRC.put((byte)20);
    buffCRC.put((byte)0);
    buffCRC.put((byte)seqNumber);
    buffCRC.put(data);
    short crc = CRC.crc16(buffCRC.array());
    ByteBuffer buff = ByteBuffer.allocate(5 + dataSize).order(ByteOrder.LITTLE_ENDIAN);
    buff.put(buffCRC.array());
    buff.putShort(crc);
    return buff.array();
  }
  
  public static byte[] createPayload(int dataSize, int seqNumber) {
    return createPayload(dataSize, seqNumber, true);
  }
  
  public static boolean isPayloadValid(byte[] payload) {
    boolean valid = false;
    return valid;
  }
  
  public PSUplink getUplink() {
    return this.uplink;
  }
  
  public int getSeqNumber() {
    return this.seqNumber;
  }
  
  public byte[] getData() {
    return this.data;
  }
  
  public boolean isValid() {
    return this.isValid;
  }
  
  public String getDeviceName() {
    return this.uplink.getEndDevice().getName();
  }
  
  public String getApplicationDataString() {
    return this.uplink.getApplicationDataString();
  }
  
  public Integer getRssi() {
    return this.uplink.getRssi();
  }
  
  public String getEndpointStr() {
    return this.uplink.getEndpointStr();
  }
  
  public Date getTimestamp() {
    return this.uplink.getTimestamp();
  }
  
  public int getApplicationDataSize() {
    return this.uplink.getApplicationDataSize();
  }
}
