package ubiik.wp.model.devices;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

public class MulticastGroup {
  public static final byte MCG_STATUS_UNKNOWN = 0;
  
  public static final byte MCG_STATUS_JOINING = 1;
  
  public static final byte MCG_STATUS_JOINED = 2;
  
  public static final byte MCG_STATUS_LEAVING = 3;
  
  public static final byte MCG_STATUS_LEFT = 4;
  
  private static final Map<Byte, String> STATUS_NAMES = new HashMap<>();
  
  private LinkedHashMap<UUEndDevice, Byte> devices;
  
  private int groupId;
  
  static {
    STATUS_NAMES.put(Byte.valueOf((byte)0), "unknown");
    STATUS_NAMES.put(Byte.valueOf((byte)1), "joining");
    STATUS_NAMES.put(Byte.valueOf((byte)2), "joined");
    STATUS_NAMES.put(Byte.valueOf((byte)3), "leaving");
    STATUS_NAMES.put(Byte.valueOf((byte)4), "left");
  }
  
  public MulticastGroup(int groupId) {
    this.groupId = groupId;
    this.devices = new LinkedHashMap<>();
  }
  
  public synchronized boolean setStatusForED(UUEndDevice ed, byte status) {
    boolean exists = deviceExists(ed);
    if (exists)
      if (status == 4) {
        removeED(ed);
      } else {
        this.devices.put(ed, Byte.valueOf(status));
      }  
    return exists;
  }
  
  public synchronized void addOrUpdateED(UUEndDevice ed, byte status) {
    boolean exists = setStatusForED(ed, status);
    if (!exists && status != 4)
      this.devices.put(ed, Byte.valueOf(status)); 
  }
  
  public synchronized void addED(UUEndDevice ed) {
    addOrUpdateED(ed, (byte)0);
  }
  
  public synchronized void removeED(UUEndDevice ed) {
    this.devices.remove(ed);
  }
  
  public synchronized Byte getStatusForEd(UUEndDevice ed) {
    return this.devices.get(ed);
  }
  
  public synchronized boolean deviceExists(UUEndDevice ed) {
    return this.devices.containsKey(ed);
  }
  
  public synchronized boolean deviceExists(byte[] ed) {
    return false;
  }
  
  public synchronized LinkedHashMap<UUEndDevice, Byte> getDevices() {
    return new LinkedHashMap<>(this.devices);
  }
  
  public int getGroupId() {
    return this.groupId;
  }
  
  public int getTotalDevices() {
    return this.devices.size();
  }
  
  public static final String getNameForStatus(Byte status) {
    String statusName = STATUS_NAMES.get(status);
    if (statusName == null)
      statusName = ""; 
    return statusName;
  }
  
  public long getStatusCount(byte status) {
    return this.devices.values().stream().filter(value -> (value.byteValue() == status)).count();
  }
}
