package ubiik.wp.model.devices;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class UUSIB0 {
  public static final int DEFAULT_VALUE = -1;
  
  public static final int BYTES_FOR_UUSIB0 = 5;
  
  public static final int FRAME_DURATION_MIN = 0;
  
  public static final int FRAME_DURATION_MAX = 3;
  
  private short frame_duration;
  
  public static final int NS_MIN = 0;
  
  public static final int NS_MAX = 3;
  
  private short ns;
  
  public static final Integer[] EVEN_RA_MCS_VALID_VALUES = new Integer[] { Integer.valueOf(0), Integer.valueOf(2), Integer.valueOf(6), Integer.valueOf(8) };
  
  private short even_ra_mcs;
  
  public static final Integer[] ODD_RA_MCS_VALID_VALUES = new Integer[] { Integer.valueOf(0), Integer.valueOf(2), Integer.valueOf(6), Integer.valueOf(8) };
  
  private short odd_ra_mcs;
  
  private UUSIB1 sib1 = null;
  
  public UUSIB0(short frame_duration, short ns, short even_ra_mcs, short odd_ra_mcs, UUSIB1 sib1) {
    this.frame_duration = frame_duration;
    this.ns = ns;
    this.even_ra_mcs = even_ra_mcs;
    this.odd_ra_mcs = odd_ra_mcs;
    this.sib1 = sib1;
  }
  
  public synchronized short getFrame_duration() {
    return this.frame_duration;
  }
  
  public synchronized boolean setFrame_duration(short frame_duration) {
    if (frame_duration >= 0 && frame_duration <= 3) {
      this.frame_duration = frame_duration;
      return true;
    } 
    return false;
  }
  
  public synchronized short getNs() {
    return this.ns;
  }
  
  public synchronized boolean setNs(short ns) {
    if (ns >= 0 && ns <= 3) {
      this.ns = ns;
      return true;
    } 
    return false;
  }
  
  public synchronized short getEven_ra_mcs() {
    return this.even_ra_mcs;
  }
  
  public synchronized boolean setEven_ra_mcs(short even_ra_mcs) {
    int idx = Arrays.<Integer>asList(EVEN_RA_MCS_VALID_VALUES).indexOf(Short.valueOf(even_ra_mcs));
    if (idx >= 0) {
      this.even_ra_mcs = even_ra_mcs;
      return true;
    } 
    return false;
  }
  
  public synchronized short getOdd_ra_mcs() {
    return this.odd_ra_mcs;
  }
  
  public synchronized boolean setOdd_ra_mcs(short odd_ra_mcs) {
    int idx = Arrays.<Integer>asList(ODD_RA_MCS_VALID_VALUES).indexOf(Short.valueOf(odd_ra_mcs));
    if (idx >= 0) {
      this.odd_ra_mcs = odd_ra_mcs;
      return true;
    } 
    return false;
  }
  
  public synchronized boolean isSib1_present() {
    return (this.sib1 != null);
  }
  
  public synchronized UUSIB1 getSib1() {
    return this.sib1;
  }
  
  public synchronized void setSib1(UUSIB1 sib1) {
    this.sib1 = sib1;
  }
  
  public int getSize() {
    if (this.sib1 != null)
      return 5 + this.sib1.getSize(); 
    return 5;
  }
  
  public byte[] getData() {
    ByteBuffer buffer = ByteBuffer.allocate(getSize()).order(ByteOrder.LITTLE_ENDIAN);
    buffer.put((byte)(0xFF & this.frame_duration));
    buffer.put((byte)(0xFF & this.ns));
    buffer.put((byte)(0xFF & this.even_ra_mcs));
    buffer.put((byte)(0xFF & this.odd_ra_mcs));
    if (isSib1_present()) {
      buffer.put((byte)1);
      buffer.put(this.sib1.getData());
    } else {
      buffer.put((byte)0);
    } 
    return buffer.array();
  }
}
