package ubiik.wp.model.devices.mcs;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class MCSSetting {
  public static final short NO_MCS_FLAGS = -1;
  
  public static final byte NO_MCS_VALUE = -1;
  
  private static final int MAX_FLAGS_UL = 8;
  
  private static final int MAX_FLAGS_DL = 4;
  
  public static final int UL_NB_MCS_NAMES_START = 4;
  
  public static final ArrayList<String> UL_MCS_NAMES = new ArrayList<>();
  
  static {
    UL_MCS_NAMES.add("GMSK, 100Kbps, FEC OFF");
    UL_MCS_NAMES.add("PSK, 12.5Kbps, FEC OFF");
    UL_MCS_NAMES.add("GMSK, 50Kbps, FEC ON");
    UL_MCS_NAMES.add("PSK, 6.25Kbps, FEC ON");
    UL_MCS_NAMES.add("GMSK, 10Kbps, FEC OFF");
    UL_MCS_NAMES.add("PSK, 1.25Kbps, FEC OFF");
    UL_MCS_NAMES.add("GMSK, 5Kbps, FEC ON");
    UL_MCS_NAMES.add("PSK, 0.625Kbps, FEC ON");
  }
  
  public static final ArrayList<String> UL_MCS_NAMES_NB = new ArrayList<>(UL_MCS_NAMES.subList(4, UL_MCS_NAMES.size()));
  
  public static final ArrayList<String> UL_MCS_NAMES_SHORT = new ArrayList<>();
  
  static {
    UL_MCS_NAMES_SHORT.add("100");
    UL_MCS_NAMES_SHORT.add("12.5");
    UL_MCS_NAMES_SHORT.add("50");
    UL_MCS_NAMES_SHORT.add("6.25");
    UL_MCS_NAMES_SHORT.add("10");
    UL_MCS_NAMES_SHORT.add("1.25");
    UL_MCS_NAMES_SHORT.add("5");
    UL_MCS_NAMES_SHORT.add("0.625");
  }
  
  public static final ArrayList<String> UL_MCS_NAMES_SHORT_NB = new ArrayList<>(UL_MCS_NAMES_SHORT.subList(4, UL_MCS_NAMES_SHORT.size()));
  
  public static final ArrayList<String> DL_MCS_NAMES = new ArrayList<>();
  
  static {
    DL_MCS_NAMES.add("GMSK, 100Kbps, FEC OFF");
    DL_MCS_NAMES.add("PSK, 12.5Kbps, FEC OFF");
    DL_MCS_NAMES.add("GMSK, 50Kbps, FEC ON");
    DL_MCS_NAMES.add("PSK, 6.25Kbps, FEC ON");
  }
  
  public static final ArrayList<String> DL_MCS_NAMES_SHORT = new ArrayList<>();
  
  private short downlinkMcsFlags;
  
  private short uplinkMcsFlags;
  
  private byte currentDownlinkMcsValue;
  
  private byte currentuplinkMcsValue;
  
  private byte currentuplinkMcsValueNB;
  
  static {
    DL_MCS_NAMES_SHORT.add("100");
    DL_MCS_NAMES_SHORT.add("12.5");
    DL_MCS_NAMES_SHORT.add("50");
    DL_MCS_NAMES_SHORT.add("6.25");
  }
  
  public MCSSetting(short downlinkMcsFlags, short uplinkMcsFlags, byte currentDownlinkMcsValue, byte currentuplinkMcsValue, byte currentuplinkMcsValueNB) {
    this.downlinkMcsFlags = downlinkMcsFlags;
    this.uplinkMcsFlags = uplinkMcsFlags;
    this.currentDownlinkMcsValue = currentDownlinkMcsValue;
    this.currentuplinkMcsValue = currentuplinkMcsValue;
    this.currentuplinkMcsValueNB = currentuplinkMcsValueNB;
  }
  
  public MCSSetting() {
    this((short)-1, (short)-1, (byte)-1, (byte)-1, (byte)-1);
  }
  
  public void reset() {
    this.downlinkMcsFlags = -1;
    this.uplinkMcsFlags = -1;
    this.currentDownlinkMcsValue = -1;
    this.currentuplinkMcsValue = -1;
    this.currentuplinkMcsValueNB = -1;
  }
  
  public synchronized boolean isCurrentuplinkMcsValueSet() {
    return (this.currentuplinkMcsValue != -1);
  }
  
  public synchronized boolean isCurrentuplinkMcsValueNBSet() {
    return (this.currentuplinkMcsValueNB != -1);
  }
  
  public synchronized boolean isCurrentDownlinkMcsValueSet() {
    return (this.currentDownlinkMcsValue != -1);
  }
  
  public synchronized boolean isDownlinkMcsFlagsSet() {
    return (this.downlinkMcsFlags != -1);
  }
  
  public synchronized boolean isUplinkMcsFlagsSet() {
    return (this.uplinkMcsFlags != -1);
  }
  
  public synchronized short getDownlinkMcsFlags() {
    return this.downlinkMcsFlags;
  }
  
  public synchronized void setDownlinkMcsFlags(short downlinkMcsFlags) {
    this.downlinkMcsFlags = downlinkMcsFlags;
  }
  
  public synchronized short getUplinkMcsFlags() {
    return this.uplinkMcsFlags;
  }
  
  public synchronized void setUplinkMcsFlags(short uplinkMcsFlags) {
    this.uplinkMcsFlags = uplinkMcsFlags;
  }
  
  public synchronized short getCurrentDownlinkMcsValue() {
    return (short)this.currentDownlinkMcsValue;
  }
  
  public synchronized void setCurrentDownlinkMcsValue(byte currentDownlinkMcsValue) {
    this.currentDownlinkMcsValue = currentDownlinkMcsValue;
  }
  
  public synchronized short getCurrentuplinkMcsValue() {
    return (short)this.currentuplinkMcsValue;
  }
  
  public synchronized void setCurrentuplinkMcsValue(byte currentuplinkMcsValue) {
    this.currentuplinkMcsValue = currentuplinkMcsValue;
  }
  
  public byte getCurrentuplinkMcsValueNB() {
    return this.currentuplinkMcsValueNB;
  }
  
  public synchronized void setCurrentuplinkMcsValueNB(byte currentuplinkMcsValueNB) {
    this.currentuplinkMcsValueNB = currentuplinkMcsValueNB;
  }
  
  public int getFlagIndex(int flag, int flagSize) {
    int index = -1;
    for (int i = 0; i < flagSize; i++) {
      if ((0x1 & flag) != 0) {
        index = i;
        break;
      } 
      flag >>= 1;
    } 
    return index;
  }
  
  public List<Integer> getIndexesDLMCSAvailable() {
    return getFlagIndexes(this.downlinkMcsFlags, 4);
  }
  
  public List<Integer> getIndexesULMCSAvailable() {
    return getFlagIndexes(this.uplinkMcsFlags, 8);
  }
  
  private List<Integer> getFlagIndexes(int flag, int flagSize) {
    List<Integer> indexes = new LinkedList<>();
    if (flag != -1)
      for (int i = 0; i < flagSize; i++) {
        if ((0x1 & flag) != 0)
          indexes.add(Integer.valueOf(i)); 
        flag >>= 1;
      }  
    return indexes;
  }
}
