package ubiik.wp.model.firmwareupgrade;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class BSFirmwareFileManager {
  private static final int BYTES_FOR_HEADER_SIZE_VERSION = 8;
  
  private static final int HEADER_MAX_SIZE = 65536;
  
  private static final int BYTES_FOR_PACKAGE_VERSION = 4;
  
  public static final BSFirmwareFileValidationResult validateFirmwareFile(File firmwareFile, byte[] currentVersion) {
    boolean validFile = false;
    BSFirmwareFileValidationResult result = new BSFirmwareFileValidationResult();
    result.setCurrentVersion(currentVersion);
    int firmwareComp = 0;
    if (currentVersion != null && 
      currentVersion.length == 4)
      try {
        FileInputStream fis = new FileInputStream(firmwareFile);
        byte[] headerBytes = new byte[8];
        int total_read = 0;
        int tmpRead = 0;
        while (total_read < headerBytes.length && tmpRead >= 0) {
          tmpRead = fis.read(headerBytes, total_read, headerBytes.length - total_read);
          if (tmpRead > 0)
            total_read += tmpRead; 
        } 
        if (total_read == headerBytes.length) {
          ByteBuffer buffer = ByteBuffer.wrap(headerBytes).order(ByteOrder.LITTLE_ENDIAN);
          int headerSize = buffer.getInt();
          int headerVersion = buffer.getInt();
          if (headerSize > 0 && headerSize < 65536 && headerSize > 4 && headerVersion > 0) {
            byte[] packageVersionBytes = new byte[4];
            total_read = 0;
            tmpRead = 0;
            while (total_read < packageVersionBytes.length && tmpRead >= 0) {
              tmpRead = fis.read(packageVersionBytes, total_read, packageVersionBytes.length - total_read);
              if (tmpRead > 0)
                total_read += tmpRead; 
            } 
            result.setFileVersion(packageVersionBytes);
            if (total_read == packageVersionBytes.length) {
              validFile = true;
              firmwareComp = compareVersions(packageVersionBytes, currentVersion);
            } 
          } 
        } 
        fis.close();
      } catch (FileNotFoundException e) {
        e.printStackTrace();
      } catch (IOException e) {
        e.printStackTrace();
      }  
    result.setValid(validFile);
    result.setFileCompareResult(firmwareComp);
    return result;
  }
  
  public static int compareVersions(byte[] packageVersionBytes, byte[] currentVersion) {
    int firmwareComp = 0;
    for (int i = 0; i < 4; i++) {
      if (packageVersionBytes[i] < currentVersion[i]) {
        firmwareComp = -1;
        break;
      } 
      if (packageVersionBytes[i] > currentVersion[i]) {
        firmwareComp = 1;
        break;
      } 
    } 
    return firmwareComp;
  }
}
