package ubiik.wp.model.frequencyscanner;

import java.util.Date;
import java.util.LinkedHashMap;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import ubiik.wp.model.messages.JSONMessageDelegate;
import ubiik.wp.util.ApplicationLogger;

public class FrequencyScanManager implements JSONMessageDelegate {
  private LinkedHashMap<Short, FrequencyScanSample> samples = new LinkedHashMap<>();
  
  private static final String KEY_WARFCN = "warfcn";
  
  private static final String KEY_RSSI_MIN = "rssiMin";
  
  private static final String KEY_RSSI_MAX = "rssiMax";
  
  private static final String KEY_RSSI_AVG = "rssiAvg";
  
  private static final String KEY_TOTAL_SAMPLES = "total";
  
  private static final String KEY_CURRENT_SAMPLE = "current";
  
  private static final String KEY_ERROR = "error";
  
  private Date lastUpdate = null;
  
  private Integer expectedSamples = null;
  
  private Integer receivedSamples = null;
  
  private String error = null;
  
  public synchronized void parseJSON(JSONObject json) {
    clear();
    try {
      JSONObject samples = (JSONObject)json.get("samples");
      Object[] warfcnArr = ((JSONArray)samples.get("warfcn")).toArray();
      Object[] minRssiArr = ((JSONArray)samples.get("rssiMin")).toArray();
      Object[] maxRssiArr = ((JSONArray)samples.get("rssiMax")).toArray();
      Object[] avgRssiArr = ((JSONArray)samples.get("rssiAvg")).toArray();
      if (warfcnArr.length == minRssiArr.length && maxRssiArr.length == avgRssiArr.length && warfcnArr.length == maxRssiArr.length) {
        for (int i = 0; i < warfcnArr.length; i++) {
          FrequencyScanSample sample = new FrequencyScanSample(((Long)warfcnArr[i]).shortValue(), ((Long)minRssiArr[i]).shortValue(), ((Long)maxRssiArr[i]).shortValue(), ((Long)avgRssiArr[i]).shortValue());
          this.samples.put(Short.valueOf(sample.getWarfcn()), sample);
        } 
      } else {
        ApplicationLogger.debug("[Frequency Scan] array sizes don't match" + json.toString());
      } 
    } catch (Exception e) {
      e.printStackTrace();
      ApplicationLogger.debug("Invalid Frequency Samples json" + json.toString());
    } 
    try {
      if (json.containsKey("total") && json.containsKey("current")) {
        this.expectedSamples = Integer.valueOf(((Long)json.get("total")).intValue());
        this.receivedSamples = Integer.valueOf(((Long)json.get("current")).intValue());
      } 
    } catch (Exception e) {
      e.printStackTrace();
      ApplicationLogger.debug("Invalid Current and Total samples" + json.toString());
    } 
    try {
      if (json.containsKey("error")) {
        this.error = (String)json.get("error");
      } else {
        this.error = null;
      } 
    } catch (Exception e) {
      e.printStackTrace();
      ApplicationLogger.debug("Invalid error string in JSON" + json.toString());
      this.error = null;
    } 
    this.lastUpdate = new Date();
  }
  
  public synchronized void clear() {
    this.samples.clear();
    this.lastUpdate = new Date();
    this.expectedSamples = null;
    this.receivedSamples = null;
    this.error = null;
  }
  
  public synchronized LinkedHashMap<Short, FrequencyScanSample> getSamples() {
    return new LinkedHashMap<>(this.samples);
  }
  
  public synchronized Date getLastUpdate() {
    return this.lastUpdate;
  }
  
  public synchronized Integer getExpectedSamples() {
    return this.expectedSamples;
  }
  
  public synchronized Integer getReceivedSamples() {
    return this.receivedSamples;
  }
  
  public synchronized String getError() {
    return this.error;
  }
  
  public synchronized boolean hasError() {
    return (this.error != null);
  }
}
