package ubiik.wp.model.messages;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class PSDownlink extends PSLink {
  private static int downlinkIdCounter = 0;
  
  private static Object downlinkCounterLock = new Object();
  
  private int bytesForTarget;
  
  private Boolean targetExists;
  
  private DownlinkAcknowledgement downlinkAcknowledgement;
  
  public PSDownlink(int applicationID, byte[] messageData, byte[] target, int endpoint) {
    super(applicationID, (byte)2, messageData, target, endpoint);
    this.bytesForTarget = 16;
    this.downlinkAcknowledgement = null;
  }
  
  public PSDownlink(int applicationID, byte[] messageData, byte[] target) {
    super(applicationID, (byte)2, messageData, target);
    this.bytesForTarget = 16;
    this.downlinkAcknowledgement = null;
  }
  
  public PSDownlink(byte[] messageData, byte[] target, int endpoint) {
    this(getNextDownlinkId(), messageData, target, endpoint);
  }
  
  public PSDownlink(byte[] messageData, byte[] target) {
    this(getNextDownlinkId(), messageData, target);
  }
  
  protected byte[] getMessageDataBytes() throws MessageFormatException {
    if (this.target.length != this.bytesForTarget)
      throw new MessageFormatException(); 
    short downlinkId = (short)(int)(this.applicationId & 0xFFFFL);
    byte endPoint = (byte)(this.endpoint & 0xFF);
    ByteBuffer buffer = ByteBuffer.allocate(3 + this.target.length + this.data.length);
    buffer.order(ByteOrder.LITTLE_ENDIAN);
    buffer.putShort(downlinkId);
    buffer.put(endPoint);
    buffer.put(this.target);
    buffer.put(this.data);
    return buffer.array();
  }
  
  private static int getNextDownlinkId() {
    int nextMid = -1;
    synchronized (downlinkCounterLock) {
      nextMid = downlinkIdCounter;
      downlinkIdCounter++;
    } 
    return nextMid;
  }
  
  public synchronized int getBytesForTarget() {
    return this.bytesForTarget;
  }
  
  public synchronized void setBytesForTarget(int bytesForTarget) {
    this.bytesForTarget = bytesForTarget;
  }
  
  public synchronized Boolean getTargetExists() {
    return this.targetExists;
  }
  
  public void setTargetExists(Boolean targetExists) {
    this.targetExists = targetExists;
  }
  
  public String toString() {
    return Long.toString(this.applicationId) + "\t" + Long.toString(this.applicationId) + "\t" + getTargetString() + "\t" + 
      getApplicationDataString() + "\t" + getApplicationDataSize() + "\t" + 
      getEndpointStr() + "\t" + getAckStatusString();
  }
  
  public synchronized DownlinkAcknowledgement getDowlinkAcknowledgement() {
    return this.downlinkAcknowledgement;
  }
  
  public synchronized void setDowlinkAcknowledgement(DownlinkAcknowledgement dlAck) {
    if (dlAck.getStatus() != 64)
      this.downlinkAcknowledgement = dlAck; 
  }
  
  public synchronized Integer getAckStatus() {
    if (this.downlinkAcknowledgement == null)
      return null; 
    return Integer.valueOf(this.downlinkAcknowledgement.getStatus());
  }
  
  public synchronized String getAckStatusString() {
    if (this.downlinkAcknowledgement == null)
      return "Pending"; 
    String intHex = Integer.toHexString(this.downlinkAcknowledgement.getStatus());
    return "0x" + ((intHex.length() % 2 == 0) ? intHex : ("0" + intHex));
  }
}
