package ubiik.wp.model.messages;

import ubiik.utils.DatatypeConverter;
import ubiik.wp.model.devices.UUEndDevice;

public abstract class PSLink extends PSMessage {
  protected byte[] target;
  
  protected UUEndDevice endDevice;
  
  protected int endpoint;
  
  public static final int ENDPOINT_UNACKNOWLEDGED = 0;
  
  public static final int ENDPOINT_ACKNOWLEDGED = 1;
  
  public static final int ENDPOINT_ACKNOWLEDGED_INTERRUPT = 2;
  
  public static final int ENDPOINT_FIRMWARE = 255;
  
  public PSLink() {
    this.target = new byte[0];
    this.endpoint = 0;
    this.endDevice = null;
  }
  
  public PSLink(int applicationID, byte typeCode, byte[] messageData, byte[] target, int endpoint) {
    super(applicationID, messageData, typeCode);
    this.target = target;
    this.endpoint = endpoint;
    this.endDevice = null;
  }
  
  public PSLink(int applicationID, byte typeCode, byte[] messageData, byte[] target) {
    this(applicationID, typeCode, messageData, target, 0);
  }
  
  public String getTargetString() {
    return DatatypeConverter.printHexBinary(this.target);
  }
  
  public byte[] getTarget() {
    return this.target;
  }
  
  public int getEndpoint() {
    return this.endpoint;
  }
  
  public String getEndpointStr() {
    String endpointStr = "";
    switch (this.endpoint) {
      case 0:
        endpointStr = "UNACKNOWLEDGED";
        return endpointStr;
      case 1:
        endpointStr = "ACKNOWLEDGED";
        return endpointStr;
      case 2:
        endpointStr = "ACK INTERRUPT";
        return endpointStr;
      case 255:
        endpointStr = "FIRMWARE";
        return endpointStr;
    } 
    endpointStr = "";
    return endpointStr;
  }
  
  public String getTargetName() {
    if (this.endDevice != null)
      return this.endDevice.getName(); 
    return getTargetString();
  }
  
  public synchronized UUEndDevice getEndDevice() {
    return this.endDevice;
  }
  
  public synchronized void setEndDevice(UUEndDevice endDevice) {
    this.endDevice = endDevice;
  }
}
