package ubiik.wp.model.messages;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponse;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseFactory;
import ubiik.wp.util.ApplicationLogger;

public class PSMessageFactory {
  public static PSMessage createMessageFromBytes(byte[] bytes) throws IllegalArgumentException {
    PSCommandResponse pSCommandResponse;
    PSMessageAuthenticated pSMessageAuthenticated;
    PSLogMessage pSLogMessage;
    PSMessageJson pSMessageJson;
    DownlinkAcknowledgement downlinkAcknowledgement;
    PSMulticastNotification pSMulticastNotification;
    PSPong pSPong;
    PSCommandResponseFactory responseFactory;
    PSMessage message = null;
    ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    short messageId = buffer.getShort();
    byte messageType = buffer.get();
    byte[] data = new byte[buffer.capacity() - buffer.position()];
    buffer.get(data);
    ApplicationLogger.debug("Trying to create message with type: " + messageType + " with ID: " + messageId);
    switch (messageType) {
      case 1:
      case 10:
        message = new PSUplink(data, messageId, messageType);
        return message;
      case 124:
        message = new PSMessageNewDiscovery(messageId, data);
        return message;
      case 127:
        message = new PSMessageDiscovery(messageId, data);
        return message;
      case 5:
        message = new PSMessageDeviceStatus(messageId, data);
        return message;
      case 6:
        message = new PSAckMessage(data, messageId);
        return message;
      case -124:
        responseFactory = new PSCommandResponseFactory();
        return (PSMessage)responseFactory.createCommandResponseFromData(messageId, messageType, data);
      case 0:
        return new PSMessageAuthenticated(messageId, data);
      case 7:
        return new PSLogMessage(messageId, data);
      case 9:
        return new PSMessageJson(messageId, data);
      case 8:
        return new DownlinkAcknowledgement(messageId, data);
      case 12:
        return new PSMulticastNotification(messageId, data);
      case 13:
        return new PSPong(messageId, data);
      case 14:
        return new PSHardwareInfo(messageId, data);
    } 
    return new PSGenericMessage(messageId, -1, data, messageType);
  }
}
