package ubiik.wp.model.messages;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.List;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseReadMulticastGroups;

public class PSMulticastNotification extends PSMessage {
  public static final byte NOTIFICATION_TYPE_FULL_UPDATE = 0;
  
  public static final byte NOTIFICATION_TYPE_PARTIAL_UPDATE = 1;
  
  private byte notificationType;
  
  private HashMap<Integer, List<byte[]>> multicastGroups;
  
  private HashMap<Integer, List<Byte>> devicesStatus;
  
  public PSMulticastNotification(int messageId, byte[] messageData) {
    super(messageId, 0, messageData, (byte)12);
    try {
      this.multicastGroups = new HashMap<>();
      this.devicesStatus = new HashMap<>();
      ByteBuffer buffer = ByteBuffer.wrap(this.data).order(ByteOrder.LITTLE_ENDIAN);
      int headerSize = buffer.getInt();
      int payloadSize = buffer.getInt();
      this.protocolVersion = buffer.getInt();
      int groupsCount = buffer.getShort() & 0xFF;
      this.notificationType = buffer.get();
      PSCommandResponseReadMulticastGroups.loadMulticastGroupsFromData(buffer, groupsCount, this.multicastGroups, this.devicesStatus);
    } catch (Exception e) {
      e.printStackTrace();
      throw new IllegalArgumentException();
    } 
  }
  
  protected byte[] getMessageDataBytes() throws MessageFormatException {
    return this.data;
  }
  
  public HashMap<Integer, List<byte[]>> getMulticastGroups() {
    return this.multicastGroups;
  }
  
  public HashMap<Integer, List<Byte>> getDevicesStatusInMulticastGroups() {
    return this.devicesStatus;
  }
  
  public byte getNotificationType() {
    return this.notificationType;
  }
}
