package ubiik.wp.model.messages.pscommands;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.utils.files.FileUtils;
import ubiik.wp.model.messages.MessageFormatException;

public class PSCommandBSFirmwareUpgrade extends PSCommand {
  InputStream fileIs;
  
  private byte[] fileBytes;
  
  private static final int MAX_BS_FW_FILE_BYTES = 68157440;
  
  private static final int BYTES_FOR_FILE_LENGTH = 4;
  
  public PSCommandBSFirmwareUpgrade(File file, PSCommandDelegate delegate) {
    super((byte[])null, (short)25, delegate);
    this.fileBytes = null;
    try {
      this.fileIs = new FileInputStream(file);
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } 
    if (file.length() > 68157440L)
      throw new IllegalArgumentException("File larger than max allowed"); 
  }
  
  public PSCommandBSFirmwareUpgrade(InputStream inputStream, PSCommandDelegate delegate) {
    super((byte[])null, (short)25, delegate);
    this.fileBytes = null;
    this.fileIs = inputStream;
  }
  
  protected byte[] getMessageDataBytes() throws MessageFormatException {
    setUpData();
    return super.getMessageDataBytes();
  }
  
  private void setUpData() {
    if (this.fileBytes == null)
      try {
        this.fileBytes = FileUtils.toByteArray(this.fileIs);
        ByteBuffer buf = ByteBuffer.allocate(4 + this.fileBytes.length).order(ByteOrder.LITTLE_ENDIAN);
        buf.putInt(this.fileBytes.length);
        buf.put(this.fileBytes);
        this.data = buf.array();
      } catch (IOException e) {
        e.printStackTrace();
      }  
  }
  
  public String getTypeString() {
    return "BS Firmware upgrade";
  }
  
  public String getContentString() {
    return "";
  }
}
