package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.utils.DatatypeConverter;

public class PSCommandEDRASlots extends PSCommandWithOperations {
  public static final int BYTES_FOR_VAL = 2;
  
  public static final short RA_MIN_VALUE = 1;
  
  public static final short RA_MAX_VALUE = 15;
  
  public static final short RA_NO_VALUE = 0;
  
  public static final short RA_TAKE_ALL_SLOTS = -1;
  
  protected byte[] uueid;
  
  private short numRaSlots;
  
  private short numRaSlotsSlowMcs;
  
  public PSCommandEDRASlots(PSCommandDelegate delegate, byte operation, byte[] uueid, short raSlots, short raSlotsSlowMcs) {
    super((byte[])null, (short)36, delegate, operation);
    this.numRaSlots = raSlots;
    this.numRaSlotsSlowMcs = raSlotsSlowMcs;
    this.uueid = uueid;
    setUpData();
  }
  
  public PSCommandEDRASlots(PSCommandDelegate delegate, byte operation, byte[] uueid) {
    this(delegate, operation, uueid, (short)0, (short)0);
  }
  
  private void setUpData() {
    int buffSize = 17;
    if (isSet())
      buffSize += 4; 
    ByteBuffer buffer = ByteBuffer.allocate(buffSize).order(ByteOrder.LITTLE_ENDIAN);
    buffer.put(this.operation)
      .put(this.uueid);
    if (isSet())
      buffer.putShort(this.numRaSlots)
        .putShort(this.numRaSlotsSlowMcs); 
    this.data = buffer.array();
  }
  
  public String getContentString() {
    StringBuilder sb = new StringBuilder();
    if (isSet()) {
      sb.append("Set");
    } else {
      sb.append("Get");
    } 
    sb.append(" RA slots for ED: ");
    sb.append(DatatypeConverter.printHexBinary(this.uueid));
    return sb.toString();
  }
  
  public short getNumRaSlots() {
    return this.numRaSlots;
  }
  
  public short getNumRaSlotsSlowMcs() {
    return this.numRaSlotsSlowMcs;
  }
}
