package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.utils.DatatypeConverter;

public class PSCommandMCSSetting extends PSCommandWithOperations {
  public static final int BYTES_FOR_VAL = 2;
  
  protected short uplinkVal;
  
  protected short downlinkVal;
  
  protected byte[] uueid;
  
  public PSCommandMCSSetting(PSCommandDelegate delegate, byte operation, byte[] uueid, short uplinkVal, short downlinkVal) {
    super((byte[])null, (short)34, delegate, operation);
    this.uplinkVal = uplinkVal;
    this.downlinkVal = downlinkVal;
    this.uueid = uueid;
    setUpData();
  }
  
  public PSCommandMCSSetting(PSCommandDelegate delegate, byte operation, byte[] uueid) {
    this(delegate, operation, uueid, (short)-1, (short)-1);
  }
  
  public String getContentString() {
    StringBuilder sb = new StringBuilder();
    if (isSet()) {
      sb.append("Set");
    } else {
      sb.append("Get");
    } 
    sb.append(" MCS settings for ED: ");
    sb.append(DatatypeConverter.printHexBinary(this.uueid));
    return sb.toString();
  }
  
  private void setUpData() {
    int buffSize = 17;
    if (isSet())
      buffSize += 4; 
    ByteBuffer buffer = ByteBuffer.allocate(buffSize).order(ByteOrder.LITTLE_ENDIAN);
    buffer.put(this.operation)
      .put(this.uueid);
    if (isSet())
      buffer.putShort(this.downlinkVal)
        .putShort(this.uplinkVal); 
    this.data = buffer.array();
  }
  
  public short getUplinkVal() {
    return this.uplinkVal;
  }
  
  public short getDownlinkVal() {
    return this.downlinkVal;
  }
  
  public byte[] getUueid() {
    return this.uueid;
  }
}
