package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.utils.DatatypeConverter;

public class PSCommandMCSValue extends PSCommandWithOperations {
  public static final int BYTES_FOR_VAL = 1;
  
  private byte downlinkMCSValue;
  
  private byte uplinkMCSValue;
  
  private byte uplinkMCSValueNB;
  
  protected byte[] uueid;
  
  public PSCommandMCSValue(PSCommandDelegate delegate, byte operation, byte[] uueid, byte downlinkMCSValue, byte uplinkMCSValue, byte uplinkMCSValueNB) {
    super((byte[])null, (short)35, delegate, operation);
    this.downlinkMCSValue = downlinkMCSValue;
    this.uplinkMCSValue = uplinkMCSValue;
    this.uplinkMCSValueNB = uplinkMCSValueNB;
    this.uueid = uueid;
    setUpData();
  }
  
  public PSCommandMCSValue(PSCommandDelegate delegate, byte operation, byte[] uueid) {
    this(delegate, operation, uueid, (byte)-1, (byte)-1, (byte)-1);
  }
  
  public String getContentString() {
    StringBuilder sb = new StringBuilder();
    if (isSet()) {
      sb.append("Set");
    } else {
      sb.append("Get");
    } 
    sb.append(" MCS values for ED: ");
    sb.append(DatatypeConverter.printHexBinary(this.uueid));
    return sb.toString();
  }
  
  private void setUpData() {
    int buffSize = 17;
    if (isSet())
      buffSize += 3; 
    ByteBuffer buffer = ByteBuffer.allocate(buffSize).order(ByteOrder.LITTLE_ENDIAN);
    buffer.put(this.operation)
      .put(this.uueid);
    if (isSet())
      buffer.put(this.downlinkMCSValue)
        .put(this.uplinkMCSValue)
        .put(this.uplinkMCSValueNB); 
    this.data = buffer.array();
  }
  
  public byte getDownlinkMCSValue() {
    return this.downlinkMCSValue;
  }
  
  public byte getUplinkMCSValue() {
    return this.uplinkMCSValue;
  }
  
  public byte getUplinkMCSValueNB() {
    return this.uplinkMCSValueNB;
  }
  
  public byte[] getUueid() {
    return this.uueid;
  }
}
