package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.List;
import java.util.Map;

public abstract class PSCommandManageDevicesToMulticastGroup extends PSCommand {
  private Map<Integer, List<byte[]>> multicastGroups;
  
  public PSCommandManageDevicesToMulticastGroup(short commandType, Map<Integer, List<byte[]>> multicastGroups, PSCommandDelegate delegate) {
    super((byte[])null, commandType, delegate);
    this.multicastGroups = multicastGroups;
    setUpData();
  }
  
  private void setUpData() {
    int totalGroups = this.multicastGroups.size();
    int totalDevices = this.multicastGroups.values().stream().mapToInt(List::size).sum();
    int totalBytes = 2 + totalGroups * 4 + totalDevices * 16;
    ByteBuffer buffer = ByteBuffer.allocate(totalBytes).order(ByteOrder.LITTLE_ENDIAN);
    buffer.putShort((short)this.multicastGroups.size());
    for (Integer group : this.multicastGroups.keySet()) {
      List<byte[]> devicesIds = this.multicastGroups.get(group);
      buffer.putShort(group.shortValue());
      buffer.putShort((short)devicesIds.size());
      for (byte[] bs : devicesIds) {
        if (bs.length == 16) {
          buffer.put(bs);
          continue;
        } 
        throw new IllegalArgumentException();
      } 
    } 
    this.data = buffer.array();
  }
  
  public String getContentString() {
    String content = "Groups: ";
    for (Integer groupId : this.multicastGroups.keySet())
      content = content + " " + content; 
    return content;
  }
  
  public Map<Integer, List<byte[]>> getMulticastGroups() {
    return this.multicastGroups;
  }
}
