package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class PSCommandRunMode extends PSCommandWithOperations {
  public static final int RUN_MODE_STOP = 0;
  
  public static final int RUN_MODE_START = 1;
  
  public static final int RUN_MODE_UNKNOWN = 65535;
  
  public static final int DEFAULT_SUB_MODE = 0;
  
  private static final Map<Integer, String> RUN_MODES;
  
  public static final int BYTES_FOR_MODE = 2;
  
  public static final int BYTES_FOR_SUB_MODE = 2;
  
  private Integer mode;
  
  private Integer subMode;
  
  static {
    Map<Integer, String> runModeMap = new HashMap<>();
    runModeMap.put(Integer.valueOf(0), "STOP");
    runModeMap.put(Integer.valueOf(1), "START");
    runModeMap.put(Integer.valueOf(65535), "-");
    RUN_MODES = Collections.unmodifiableMap(runModeMap);
  }
  
  public PSCommandRunMode(PSCommandDelegate delegate, byte operation, int mode, int subMode) {
    super((byte[])null, (short)20, delegate, operation);
    this.mode = Integer.valueOf(mode);
    this.subMode = Integer.valueOf(subMode);
    setUpData();
  }
  
  public PSCommandRunMode(PSCommandDelegate delegate, byte operation, int mode) {
    this(delegate, operation, mode, 0);
  }
  
  public String getTypeString() {
    String str = super.getTypeString();
    str = str.concat(" Run Mode ");
    if (isSet()) {
      String runModeStr = RUN_MODES.get(this.mode);
      if (runModeStr != null)
        str = str.concat(runModeStr); 
    } 
    return str;
  }
  
  public String getContentString() {
    return "";
  }
  
  private void setUpData() {
    ByteBuffer buffer = ByteBuffer.allocate(5).order(ByteOrder.LITTLE_ENDIAN);
    buffer.put(this.operation)
      .putShort(this.mode.shortValue())
      .putShort(this.subMode.shortValue());
    this.data = buffer.array();
  }
  
  public Integer getMode() {
    return this.mode;
  }
  
  public String getRunModeStr() {
    return getRunModeStrForCode(this.mode.intValue());
  }
  
  public static final String getRunModeStrForCode(int code) {
    String runModeStr = RUN_MODES.get(Integer.valueOf(code));
    if (runModeStr == null)
      runModeStr = ""; 
    return runModeStr;
  }
}
