package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import ubiik.wp.model.sdr.SDRParameter;

public class PSCommandSDRSet extends PSCommand {
  private static final int BYTES_FOR_SDR_PARAM_ID = 2;
  
  private static final int BYTES_FOR_SDR_PARAM_VALUE = 4;
  
  private SDRParameter sdrParameter;
  
  public PSCommandSDRSet(PSCommandDelegate delegate, SDRParameter parameter) {
    super((byte[])null, (short)15, delegate);
    this.sdrParameter = parameter;
    setUpData();
  }
  
  private void setUpData() {
    byte[] valueData = this.sdrParameter.getValueInBytes();
    ByteBuffer buffer = ByteBuffer.allocate(6).order(ByteOrder.LITTLE_ENDIAN);
    buffer.putShort((short)this.sdrParameter.getId());
    if (valueData.length <= 4) {
      buffer.put(valueData);
    } else {
      buffer.put(Arrays.copyOf(valueData, 4));
    } 
    this.data = buffer.array();
  }
  
  public String getTypeString() {
    return "Set SDR Parameter " + this.sdrParameter.getName();
  }
  
  public String getContentString() {
    return "";
  }
  
  public SDRParameter getSdrParameter() {
    return this.sdrParameter;
  }
}
