package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.security.InvalidParameterException;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class PSCommandSetBaseChannel extends PSCommand {
  public static final int MAX_BASE_FREQUENCY = 9297;
  
  public static final int MIN_BASE_FREQUENCY = 1382;
  
  private short channel;
  
  public PSCommandSetBaseChannel(short channel, PSCommandDelegate delegate) {
    super((byte[])null, (short)2, delegate);
    this.channel = channel;
    setUpData(channel);
  }
  
  public PSCommandSetBaseChannel(JSONObject json) {
    super((byte[])null, (short)2, (PSCommandDelegate)null);
    Integer ch = JsonUtils.getIntIfExists(json, "channel");
    if (ch == null)
      throw new InvalidParameterException("channel must be defined in the json file."); 
    this.channel = ch.shortValue();
    setUpData(this.channel);
  }
  
  public void setChannel(short channel) {
    this.channel = channel;
    setUpData(channel);
  }
  
  public int getChannel() {
    return this.channel;
  }
  
  private void setUpData(short channel) {
    this
      
      .data = ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN).putShort(channel).array();
  }
  
  public String getTypeString() {
    return "Set Base Frequency";
  }
  
  public String getContentString() {
    return Integer.toString(this.channel);
  }
}
