package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class PSCommandSetScheduleTransfer extends PSCommand {
  public static final byte FLAG_RELEASE = 0;
  
  public static final byte FLAG_SET = 1;
  
  private byte[] deviceUuid;
  
  private byte flag;
  
  private int interval;
  
  private int window;
  
  public PSCommandSetScheduleTransfer(PSCommandDelegate delegate, byte[] deviceUuid, byte flag, int interval, int window) {
    super(new byte[0], (short)18, delegate);
    this.deviceUuid = deviceUuid;
    this.flag = flag;
    this.interval = interval;
    this.window = window;
    setUpData();
  }
  
  private void setUpData() {
    int buffLen = 17;
    if (isSet())
      buffLen += 4; 
    ByteBuffer buffer = ByteBuffer.allocate(buffLen).order(ByteOrder.LITTLE_ENDIAN);
    buffer.put(this.deviceUuid).put(this.flag);
    if (isSet()) {
      buffer.putShort((short)this.interval);
      buffer.putShort((short)this.window);
    } 
    this.data = buffer.array();
  }
  
  private boolean isSet() {
    return (this.flag == 1);
  }
  
  public String getTypeString() {
    return "Set Schedule Transfer";
  }
  
  public String getContentString() {
    if (isSet()) {
      StringBuilder str = new StringBuilder("Set interval = ");
      str.append(this.interval).append(", window = ").append(this.window);
      return str.toString();
    } 
    return "release";
  }
  
  public byte[] getDeviceUuid() {
    return this.deviceUuid;
  }
  
  public byte getFlag() {
    return this.flag;
  }
  
  public int getInterval() {
    return this.interval;
  }
  
  public int getWindow() {
    return this.window;
  }
}
