package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.utils.DatatypeConverter;
import ubiik.wp.util.ApplicationLogger;

public class PSCommandResponseFactory {
  public PSCommandResponse createCommandResponseFromData(short messageId, byte messageType, byte[] messageData) {
    PSCommandResponse response = null;
    ApplicationLogger.debug("Received Command Response. messageType: " + messageType + " messageId (seq): " + messageId + "message data: " + 
        
        DatatypeConverter.printHexBinary(messageData));
    try {
      ByteBuffer buffer = ByteBuffer.wrap(messageData).order(ByteOrder.LITTLE_ENDIAN);
      int commandId = buffer.getShort() & 0xFFFF;
      short commandType = buffer.getShort();
      byte[] commandData = new byte[buffer.capacity() - buffer.position()];
      buffer.get(commandData);
      switch (commandType) {
        /*case 1:
          response = new PSCommandResponseGetBaseChannel(messageId, commandId, commandData);
          return response;*/
          case 57:
          response = new PSCommandResponseGetBaseChannel(messageId, commandId, commandData);
          return response;
          case 58:
          response = new PSCommandResponseGetULBaseChannel(messageId, commandId, commandData);
          return response;
        case 12:
          response = new PSCommandResponseFPGAReadRegister(messageId, commandId, commandData, commandType);
          return response;
        case 11:
          response = new PSCommandResponseFPGASetRegister(messageId, commandId, commandData, commandType);
          return response;
        case 9:
          response = new PSCommandResponseReadBlacklistedChannels(messageId, commandId, commandData, commandType);
          return response;
        case 14:
          response = new PSCommandResponseReadSIB(messageId, commandId, commandData);
          return response;
        case 3:
          response = new PSCommandResponseReadMulticastGroups(messageId, commandId, commandData);
          return response;
        case 17:
          response = new PSCommandResponseATCommand(messageId, commandId, commandData);
          return response;
        case 19:
          response = new PSCommandResponseGetScheduleTransfer(messageId, commandId, commandData);
          return response;
        case 16:
          response = new PSCommandResponseBSParameter(messageId, commandId, commandData);
          return response;
        case 20:
          response = new PSCommandResponseRunMode(messageId, commandId, commandData);
          return response;
        case 21:
          response = new PSCommandResponseStringParameter(messageId, commandId, commandData);
          return response;
        case 25:
          response = new PSCommandResponseBSFirmwareUpgrade(messageId, commandId, commandData);
          return response;
        case 34:
          response = new PSCommandResponseMCSSetting(messageId, commandId, commandData);
          return response;
        case 35:
          response = new PSCommandResponseMCSValue(messageId, commandId, commandData);
          return response;
        case 36:
          response = new PSCommandResponseEDRASlots(messageId, commandId, commandData);
          return response;
        case 46:
          response = new PSCommandResponseLogFile(messageId, commandId, commandData);
          return response;
      } 
      response = new PSCommandResponseGeneric(messageId, commandId, commandData, commandType);
    } catch (Exception e) {
      ApplicationLogger.debug("Command response with wrong format");
      e.printStackTrace();
    } 
    return response;
  }
}
