package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import ubiik.wp.model.devices.UUSIB0;
import ubiik.wp.model.devices.UUSIB1;

public class PSCommandResponseReadSIB extends PSCommandResponse {
  private UUSIB0 sib0;
  
  public PSCommandResponseReadSIB(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)14);
    if (getErrorCode() == 0) {
      try {
        UUSIB1 sib1 = null;
        ByteBuffer buffer = ByteBuffer.wrap(commandData, 1, commandData.length - 1).order(ByteOrder.LITTLE_ENDIAN);
        short frame_duration = (short)(0xFF & buffer.get());
        short ns = (short)(0xFF & buffer.get());
        short even_ra_mcs = (short)(0xFF & buffer.get());
        short odd_ra_mcs = (short)(0xFF & buffer.get());
        byte isSib1 = buffer.get();
        if (isSib1 != 0) {
          short hop_first_ch = buffer.getShort();
          short hop_last_ch = (short)(0xFF & buffer.get());
          short hop_ch_nb = (short)(0xFF & buffer.get());
          short nb_ncell_hop_ch = (short)(0xFF & buffer.get());
          short[] ncell_ch = new short[nb_ncell_hop_ch];
          for (int i = 0; i < nb_ncell_hop_ch; i++)
            ncell_ch[i] = buffer.getShort(); 
          sib1 = new UUSIB1(hop_first_ch, hop_last_ch, hop_ch_nb, nb_ncell_hop_ch, ncell_ch);
        } 
        this.sib0 = new UUSIB0(frame_duration, ns, even_ra_mcs, odd_ra_mcs, sib1);
      } catch (Exception e) {
        this.sib0 = null;
        e.printStackTrace();
        throw new IllegalArgumentException();
      } 
    } else {
      this.sib0 = null;
    } 
  }
  
  public String getTypeString() {
    return "Response to Read SIB";
  }
  
  public String getContentString() {
    if (this.sib0.isSib1_present())
      return "SIB0 and SIB1"; 
    return "SIB0";
  }
  
  public synchronized UUSIB0 getSib0() {
    return this.sib0;
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseReadSIB) {
      UUSIB0 otherSib = ((PSCommandResponseReadSIB)other).getSib0();
      boolean equalSib0 = (this.sib0.getFrame_duration() == otherSib.getFrame_duration() && this.sib0.getNs() == otherSib.getNs() && this.sib0.getEven_ra_mcs() == otherSib.getEven_ra_mcs() && this.sib0.getOdd_ra_mcs() == otherSib.getOdd_ra_mcs());
      boolean equalSib1 = false;
      if (this.sib0.getSib1() != null && otherSib.getSib1() != null) {
        equalSib1 = (this.sib0.getSib1().getHop_first_ch() == otherSib.getSib1().getHop_first_ch() && this.sib0.getSib1().getHop_ch_spacing() == otherSib.getSib1().getHop_ch_spacing() && this.sib0.getSib1().getHop_ch_nb() == otherSib.getSib1().getHop_ch_nb() && this.sib0.getSib1().getNb_ncell_hop_ch() == otherSib.getSib1().getNb_ncell_hop_ch() && Arrays.equals(this.sib0.getSib1().getNcell_ch(), otherSib.getSib1().getNcell_ch()));
      } else if (this.sib0.getSib1() == null && otherSib.getSib1() == null) {
        equalSib1 = true;
      } 
      return (equalSib0 && equalSib1);
    } 
    return false;
  }
}
