package ubiik.wp.model.sdr;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class SDRParameterInt extends SDRParameter {
  public static final String SDR_PARAM_VALUE_TYPE = "int";
  
  protected int value;
  
  private int defaultValue;
  
  private int max;
  
  private int min;
  
  private int step;
  
  public SDRParameterInt(int id, String name, String description, String category, int value, int defaultValue, int max, int min, int step) {
    super(id, name, description, category);
    this.value = value;
    this.defaultValue = defaultValue;
    this.max = max;
    this.min = min;
    this.step = step;
  }
  
  public SDRParameterInt(int id, String name, String description, String category, int defaultValue) {
    this(id, name, description, category, defaultValue, defaultValue, 2147483647, -2147483648, 0);
  }
  
  public synchronized boolean validateValue(String val) {
    boolean valid = true;
    try {
      int intVal = Integer.parseInt(val);
      if (intVal > this.max || intVal < this.min)
        valid = false; 
      if (this.step > 0 && 
        intVal % this.step != 0)
        valid = false; 
    } catch (Exception e) {
      valid = false;
    } 
    return valid;
  }
  
  public synchronized boolean setValuFromString(String str) {
    boolean valid = validateValue(str);
    if (valid)
      this.value = Integer.parseInt(str); 
    return valid;
  }
  
  public synchronized byte[] getValueInBytes() {
    return ByteBuffer.allocate(4)
      .order(ByteOrder.LITTLE_ENDIAN)
      .putInt(this.value)
      .array();
  }
  
  public byte getDataTypeCode() {
    return 1;
  }
  
  public String getMinStr() {
    return Integer.toString(this.min);
  }
  
  public String getMaxStr() {
    return Integer.toString(this.max);
  }
  
  public String getStepStr() {
    return Integer.toString(this.step);
  }
  
  public String getValueStr() {
    return Integer.toString(this.value);
  }
  
  public String getDefaultStr() {
    return Integer.toString(this.defaultValue);
  }
}
