package ubiik.wp.persistence;

import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import ubiik.utils.DatatypeConverter;
import ubiik.wp.model.devices.MulticastGroup;
import ubiik.wp.model.devices.UUEndDevice;

public class MulticastGroupJSONParser {
  private static final String KEY_GROUPS = "groups";
  
  private static final String KEY_GROUP_ID = "id";
  
  private static final String KEY_GROUP_DEVICES = "devices";
  
  public static final Map<Integer, List<byte[]>> parseJSON(InputStreamReader in) throws IllegalArgumentException {
    Map<Integer, List<byte[]>> groups = new HashMap<>();
    JSONParser jsonParser = new JSONParser();
    try {
      JSONObject root = (JSONObject)jsonParser.parse(in);
      JSONArray groupsArray = (JSONArray)root.get("groups");
      for (Object object : groupsArray) {
        JSONObject jObject = (JSONObject)object;
        Integer groupId = Integer.valueOf(((Long)jObject.get("id")).intValue());
        List<byte[]> uueids = (List)new LinkedList<>();
        groups.put(groupId, uueids);
        JSONArray devices = (JSONArray)jObject.get("devices");
        for (Object device : devices) {
          byte[] uueid = DatatypeConverter.parseHexBinary((String)device);
          if (uueid.length == 16) {
            uueids.add(uueid);
            continue;
          } 
          throw new IllegalArgumentException("Invalid uueid");
        } 
      } 
    } catch (Exception e) {
      e.printStackTrace();
      throw new IllegalArgumentException();
    } 
    return groups;
  }
  
  public static final JSONObject multicastGroupsToJson(Collection<MulticastGroup> multicastGroups) {
    JSONObject root = new JSONObject();
    JSONArray groupsArray = new JSONArray();
    for (MulticastGroup multicastGroup : multicastGroups)
      groupsArray.add(multicastGroupToJson(multicastGroup)); 
    root.put("groups", groupsArray);
    return root;
  }
  
  public static final JSONObject multicastGroupToJson(MulticastGroup multicastGroup) {
    JSONObject groupObj = new JSONObject();
    groupObj.put("id", Integer.valueOf(multicastGroup.getGroupId()));
    JSONArray devicesArray = new JSONArray();
    for (UUEndDevice device : multicastGroup.getDevices().keySet())
      devicesArray.add(device.getUUIDString()); 
    groupObj.put("devices", devicesArray);
    return groupObj;
  }
  
  public static final void multicastGroupsToFile(Collection<MulticastGroup> multicastGroups, String path) throws IOException {
    JSONObject groups = multicastGroupsToJson(multicastGroups);
    FileWriter file = new FileWriter(path);
    Throwable throwable = null;
    try {
      file.write(groups.toJSONString());
    } catch (Throwable throwable1) {
      throwable = throwable1 = null;
      throw new AssertionError(throwable1);
    } finally {
      if (throwable != null) {
        try {
          file.close();
        } catch (Throwable throwable1) {
          throwable.addSuppressed(throwable1);
        } 
      } else {
        file.close();
      } 
    } 
  }
}
