package ubiik.wp.psconnection;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class PSInConnection extends PSSocketConnection implements Runnable {
  private static final int MAX_SIZE_TO_READ = 4096;

  private PSConnectionDelegate delegate;

  private byte[] tempData;

  private ByteBuffer inBuffer;

  private DataInputStream dataStream;

  public PSInConnection(Socket socket, PSConnectionDelegate delegate) {
    super(socket);
    this.delegate = delegate;
    this.tempData = new byte[MAX_SIZE_TO_READ];
    this.inBuffer = ByteBuffer.allocate(0).order(ByteOrder.LITTLE_ENDIAN);
  }

  private void extendBuffer(byte[] bytes, int length) {
    if (0 == length)
      return;
    ByteBuffer newBuffer = ByteBuffer.allocate(this.inBuffer.capacity() + length).order(ByteOrder.LITTLE_ENDIAN);
    newBuffer.put(this.inBuffer.array());
    newBuffer.put(bytes, 0, length);
    this.inBuffer = newBuffer;
  }

  private void removeBytesFromBuffer(int n) {
    int newCapacity = this.inBuffer.capacity() - n;
    ByteBuffer newBuffer = ByteBuffer.allocate(newCapacity).order(ByteOrder.LITTLE_ENDIAN);
    newBuffer.put(this.inBuffer.array(), n, newCapacity);
    this.inBuffer = newBuffer;
  }

  private boolean processInput(int messageLength) throws PSConnectionException {
    byte[] nextMessage = new byte[messageLength - 2];
    this.inBuffer.position(0);
    this.inBuffer.get(nextMessage, 0, messageLength - 2);
    this.inBuffer.position(messageLength - 2);
    short messageCrc = this.inBuffer.getShort();

    short calculatedCRC = crc16(nextMessage);
    if (messageCrc != calculatedCRC) {
    	throw new PSConnectionException();
    }
    removeBytesFromBuffer(messageLength);
    this.delegate.receivedBytes(nextMessage);
    return (this.inBuffer.capacity() > 0);
  }

  public void run() {
    try {
      InputStream inStream = this.socket.getInputStream();
      this.dataStream = new DataInputStream(inStream);
    } catch (IOException e) {
      e.printStackTrace();
      this.delegate.connectionFailed(PSConnectionErrorCode.UNKNOWN);
      return;
    }

    boolean connected = true;
    int tempDataLen = 0;
    int nextMesgLen = 0;
    boolean needToRead = true;
    while (connected) {
      if (Thread.interrupted())
        return;
      try {
        if (needToRead) {
          tempDataLen = this.dataStream.read(this.tempData);
          if (tempDataLen > 0) {
            needToRead = false;
            extendBuffer(this.tempData, tempDataLen);
            continue;
          }
          connected = false;
          this.delegate.connectionFailed(PSConnectionErrorCode.CONNECTION_ENDED);
          continue;
        }
        boolean needToProcess = false;
        if (nextMesgLen == 0) {
          if (this.inBuffer.capacity() >= 4) {
            this.inBuffer.position(0);
            nextMesgLen = this.inBuffer.getInt() + 2;
            removeBytesFromBuffer(4);
            needToProcess = true;
          } else {
            needToRead = true;
          }
        } else {
          needToProcess = true;
        }
        if (needToProcess) {
          if (this.inBuffer.capacity() >= nextMesgLen) {
            needToRead = !processInput(nextMesgLen);
            nextMesgLen = 0;
            continue;
          }
          needToRead = true;
        }
      } catch (IOException e) {
        connected = false;
        this.delegate.connectionFailed(PSConnectionErrorCode.CONNECTION_ENDED);
      } catch (PSConnectionException e) {
        this.delegate.connectionFailed(PSConnectionErrorCode.CRC_ERROR);
      }
    }
    
  }
}
