package ubiik.wp.psconnection;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.nio.ByteBuffer;

public class PSOutConnection extends PSSocketConnection implements Runnable {
  private PSConnectionDelegate delegate;
  
  private DataOutputStream dataStream;
  
  private Object lockBuffer = new Object();
  
  private byte[] bytesToSend;
  
  public PSOutConnection(Socket socket, PSConnectionDelegate delegate) {
    super(socket);
    this.delegate = delegate;
    this.dataStream = null;
    this.bytesToSend = null;
  }
  
  public void run() {
    try {
      OutputStream out = this.socket.getOutputStream();
      this.dataStream = new DataOutputStream(out);
    } catch (IOException e) {
      e.printStackTrace();
    } 
    boolean connected = true;
    while (connected) {
      if (Thread.interrupted())
        return; 
      synchronized (this.lockBuffer) {
        try {
          if (this.bytesToSend != null && this.bytesToSend.length > 0) {
            this.dataStream.write(this.bytesToSend);
            this.dataStream.flush();
            this.bytesToSend = null;
          } 
        } catch (IOException e) {
          e.printStackTrace();
          System.out.println("PSConnectionErrorCode.CONNECTION_ENDED");
          this.delegate.connectionFailed(PSConnectionErrorCode.CONNECTION_ENDED);
          connected = false;
        } 
      } 
    } 
  }
  
  public void sendBytes(byte[] bytes) {
    synchronized (this.lockBuffer) {
      if (this.bytesToSend != null && this.bytesToSend.length > 0) {
        byte[] packedBytes = buildPkt(bytes);
        ByteBuffer allBytes = ByteBuffer.allocate(this.bytesToSend.length + packedBytes.length);
        allBytes.put(this.bytesToSend);
        allBytes.put(packedBytes);
        this.bytesToSend = allBytes.array();
      } else {
        this.bytesToSend = buildPkt(bytes);
      } 
    } 
  }
}
