package ubiik.wp.psconnection;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import org.java_websocket.WebSocket;
import org.java_websocket.AbstractWebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;

public class PSWSConnection extends WebSocketServer {
  private WebSocket currentConn;
  
  private WSConnectionManager connectionManager;
  
  public PSWSConnection(InetSocketAddress address, WSConnectionManager connectionManager) {
    super(address);
    this.connectionManager = connectionManager;
    this.currentConn = null;
    setTcpNoDelay(true);
  }
  
  public void onOpen(WebSocket conn, ClientHandshake handshake) {
    System.out.println("Web Socket: onOpen");
    if (this.currentConn == null) {
      this.currentConn = conn;
    } else if (!this.currentConn.equals(conn)) {
      System.out.println("Web Socket: onOpen --> new connection. close");
      conn.close();
    } 
  }
  
  public void onClose(WebSocket conn, int code, String reason, boolean remote) {
    System.out.println("Web Socket: onClose");
    if (this.currentConn != null)
      if (!this.currentConn.equals(conn)) {
        this.currentConn = null;
        System.out.println("Close current connection");
      } else {
        System.out.println("Close other connection");
      }  
  }
  
  public void onMessage(WebSocket conn, String message) {
    receiveBytes(conn, message.getBytes());
  }
  
  public void onMessage(WebSocket conn, ByteBuffer message) {
    receiveBytes(conn, message.array());
  }
  
  public void receiveBytes(WebSocket conn, byte[] message) {
    if (this.currentConn != null) {
      if (this.currentConn.equals(conn)) {
        this.connectionManager.receivedBytes(message);
      } else {
        conn.close();
      } 
    } else {
      System.out.println("Received bytes but connection is null");
    } 
  }
  
  public void onError(WebSocket conn, Exception ex) {
    System.out.println("Web Socket: onError");
  }
  
  public void onStart() {
    System.out.println("Web Socket: onStart");
  }
  
  public void stop() throws IOException, InterruptedException {
    System.out.println("Web Socket: stop");
    if (this.currentConn != null)
      this.currentConn = null; 
    stop(1000);
  }
  
  public boolean sendMessage(byte[] msg) {
    boolean sent = false;
    try {
      if (this.currentConn != null) {
        if (this.currentConn.isOpen()) {
          this.currentConn.send(msg);
          sent = true;
        } else {
          System.out.println("Trying to sendMessage but this.currentConn is not open");
        } 
      } else {
        System.out.println("Trying to sendMessage but this.currentConn is null");
      } 
    } catch (Exception e) {
      e.printStackTrace();
    } 
    return sent;
  }
}
