package ubiik.wp.psconnection;

import java.net.InetSocketAddress;
import java.util.concurrent.BlockingQueue;
import ubiik.wp.util.ApplicationLogger;

public class WSConnectionManager extends PSConnectionManager {
  private PSWSConnection pswsConnection;
  
  public WSConnectionManager(String host, int port, PSConnectionManagerDelegate delegate, BlockingQueue<byte[]> queue) {
    super(host, port, delegate, queue);
    System.out.println("Create WS server on " + host + " port " + port);
    this.pswsConnection = new PSWSConnection(new InetSocketAddress(host, port), this);
  }
  
  public void connect() throws PSConnectionException {
    Thread wsServerThread = new Thread(() -> this.pswsConnection.run());
    wsServerThread.start();
  }
  
  public void disconnect() {
    try {
      this.pswsConnection.stop();
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }
  
  public void sendBytes(byte[] bytes) {
    if (this.authenticated.get())
      this.pswsConnection.sendMessage(bytes); 
  }
  
  public void receivedBytes(byte[] bytes) {
    if (!this.receivedBaseStationId.get()) {
      ApplicationLogger.debug("WS: Received Base Station ID");
      try {
        this.receivedBaseStationId.set(true);
        this.inMessages.put(bytes);
        this.authenticated.set(true);
        this.delegate.authenticationFinished(true);
      } catch (InterruptedException e) {
        e.printStackTrace();
      } 
    } else {
      try {
        this.inMessages.put(bytes);
      } catch (InterruptedException e) {
        ApplicationLogger.debug("WSConnectionManager: received interrupted");
      } 
    } 
  }
}
