package ubiik.wp.qa.model;

import java.util.Calendar;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import ubiik.utils.communication.ProgressDelegate;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.messages.PSMessage;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponse;

public class CommandTest extends BSTest implements PSCommandDelegate {
  protected MessageController messageController;
  
  protected List<PSCommand> commands;
  
  protected List<PSCommandResponse> expectedResponses;
  
  protected int currentCommand;
  
  protected StringBuilder sbResult;
  
  protected AtomicInteger totalResponses;
  
  protected long timeRunning;
  
  protected Long millisRuntime;
  
  public CommandTest(ProgressDelegate progressDelegate, MessageController messageController, List<PSCommand> commands, List<PSCommandResponse> expectedResponses, Long millisRuntime) {
    super(progressDelegate);
    this.messageController = messageController;
    this.commands = commands;
    this.expectedResponses = expectedResponses;
    this.currentCommand = 0;
    this.sbResult = new StringBuilder();
    this.totalResponses = new AtomicInteger(0);
    this.timeRunning = 0L;
    this.millisRuntime = millisRuntime;
    this.testResult.setSuccess(true);
    this.testResult.setMessage("");
  }
  
  public void commandandReceivedResponse(PSCommand command) {
    System.out.println("commandandReceivedResponse " + command.getResponse().toString());
    if (!((PSCommandResponse)this.expectedResponses.get(this.currentCommand)).compareResponseData(command.getResponse())) {
      this.testResult.setSuccess(false);
      this.sbResult.append("Command \"").append(command.toString())
        .append("\" expected response \"").append(((PSCommandResponse)this.expectedResponses.get(this.currentCommand)).toString())
        .append("\" but received \"").append(command.getResponse().toString()).append("\"\n");
    } else {
      this.sbResult.append("Command \"").append(command.toString())
        .append("\" got correct response \"").append(((PSCommandResponse)this.expectedResponses.get(this.currentCommand)).toString())
        .append("\"\n");
    } 
    this.currentCommand++;
    sendCurrentCommand();
    this.totalResponses.set(this.totalResponses.get() + 1);
  }
  
  public void run() {
    this.doRun.set(true);
    sendCurrentCommand();
    this.timeRunning = 0L;
    long timeStart = Calendar.getInstance().getTime().getTime();
    boolean finished = false;
    while (this.doRun.get() && this.totalResponses
      .get() != this.expectedResponses.size() && !finished) {
      try {
        Thread.sleep(100L);
      } catch (InterruptedException e) {
        e.printStackTrace();
      } 
      this.timeRunning = Calendar.getInstance().getTime().getTime() - timeStart;
      if (this.millisRuntime != null && this.timeRunning > this.millisRuntime.longValue())
        finished = true; 
    } 
    if (this.totalResponses.get() != this.expectedResponses.size()) {
      this.testResult.setSuccess(false);
      this.sbResult.append("Received less responses than expected.\n");
    } 
    this.testResult.setMessage(this.sbResult.toString());
    this.progressDelegate.progressFinished(null);
  }
  
  protected void sendCurrentCommand() {
    if (this.currentCommand >= this.commands.size()) {
      this.doRun.set(false);
    } else {
      System.out.println("Send command " + ((PSCommand)this.commands.get(this.currentCommand)).toString());
      this.messageController.sendMessage((PSMessage)this.commands.get(this.currentCommand));
    } 
  }
}
