package ubiik.wp.qa.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Calendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import ubiik.utils.communication.ProgressDelegate;
import ubiik.wp.model.messages.PSUplink;

public class CompletenessBSTest extends BSTest {
  private Integer totalMessages;
  
  private int receivedMessages;
  
  private Long runTime;
  
  private Long timeRunning;
  
  private List<String> uuids;
  
  private Map<String, List<ULSample>> uplinks;
  
  public CompletenessBSTest(Integer totalMessages, Long runTime, List<String> uuids, ProgressDelegate progressDelegate) {
    super(progressDelegate);
    if (totalMessages == null && runTime == null)
      throw new IllegalArgumentException("One of totalMessages or runTime must be not null"); 
    this.totalMessages = totalMessages;
    this.runTime = runTime;
    this.uplinks = new HashMap<>();
    this.receivedMessages = 0;
    this.timeRunning = Long.valueOf(0L);
    this.uuids = uuids;
    if (uuids != null) {
      for (String uuid : uuids)
        this.uplinks.put(uuid, new LinkedList<>()); 
    } else {
      this.uuids = new LinkedList<>();
    } 
  }
  
  public void run() {
    this.doRun.set(true);
    this.receivedMessages = 0;
    this.timeRunning = Long.valueOf(0L);
    long now = Calendar.getInstance().getTime().getTime();
    for (String uuid : this.uuids)
      this.uplinks.put(uuid, new LinkedList<>()); 
    while ((this.totalMessages == null || this.receivedMessages < this.totalMessages.intValue()) && (this.runTime == null || this.timeRunning
      .longValue() < this.runTime.longValue()) && this.doRun
      .get()) {
      try {
        Thread.sleep(100L);
      } catch (InterruptedException e) {
        this.doRun.set(false);
        e.printStackTrace();
      } 
      if (this.runTime != null)
        this.timeRunning = Long.valueOf(Calendar.getInstance().getTime().getTime() - now); 
    } 
    this.doRun.set(false);
    this.testResult.setSuccess(true);
    this.testResult.setMessage("");
    StringBuilder sb = new StringBuilder();
    for (Map.Entry<String, List<ULSample>> entry : this.uplinks.entrySet()) {
      List<ULSample> samples = entry.getValue();
      samples.sort(new ULSample.SNComparator());
      long first = 0L;
      long last = 0L;
      long total = 0L;
      if (samples.size() > 0) {
        first = ((ULSample)samples.get(0)).snSample.longValue();
        last = ((ULSample)samples.get(samples.size() - 1)).snSample.longValue();
        total = 1L + last - first;
      } 
      sb.append(entry.getKey()).append(": ").append(samples.size()).append(" messages received. Expected: ")
        .append(this.totalMessages).append("\n");
      boolean valid = true;
      for (int i = 1; i < samples.size(); i++) {
        boolean seqValid = (((ULSample)samples.get(i)).snSample.longValue() - ((ULSample)samples.get(i - 1)).snSample.longValue() == 1L);
        if (!seqValid)
          sb.append("Missing or repeated samples between ").append(((ULSample)samples.get(i - 1)).snSample).append(" and ")
            .append(((ULSample)samples.get(i)).snSample).append("\n"); 
        valid &= seqValid;
      } 
      this.testResult.setSuccess((this.testResult.isSuccess() && total == samples.size() && valid));
    } 
    this.testResult.setMessage(sb.toString());
    this.progressDelegate.progressFinished(null);
  }
  
  public void processUplink(PSUplink uplink) {
    if (this.doRun.get()) {
      ByteBuffer buf = ByteBuffer.wrap(uplink.getApplicationData()).order(ByteOrder.LITTLE_ENDIAN);
      if (buf.getShort() == 20) {
        ULSample sample = new ULSample(Long.valueOf(uplink.getTimestamp().getTime()), Long.valueOf(0xFFFFFFFFL & buf.get()));
        if (this.uplinks.containsKey(uplink.getTargetString())) {
          ((List<ULSample>)this.uplinks.get(uplink.getTargetString())).add(sample);
          this.receivedMessages++;
        } 
      } 
    } 
  }
  
  public List<String> getUuids() {
    return this.uuids;
  }
  
  public Map<String, List<ULSample>> getUplinks() {
    return this.uplinks;
  }
  
  public Integer getTotalMessages() {
    return this.totalMessages;
  }
  
  public int getReceivedMessages() {
    return this.receivedMessages;
  }
  
  public Long getRunTime() {
    return this.runTime;
  }
  
  public Long getTimeRunning() {
    return this.timeRunning;
  }
}
