package ubiik.wp.qa.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import ubiik.utils.communication.ProgressDelegate;
import ubiik.utils.model.ByteArrayUtils;
import ubiik.wp.ami.controller.UUTestResult;
import ubiik.wp.controller.MessageController;

public class InvalidMessagesTest extends PSMessageStressTest {
  private int payloadSize;
  
  private long millisInterval;
  
  private Integer totalIterations;
  
  private Long millisRuntime;
  
  private MessageController messageController;
  
  private static ArrayList<Byte> VALID_MSG_IDS = new ArrayList<>(Arrays.asList(new Byte[] { 
          Byte.valueOf((byte)0), 
          Byte.valueOf((byte)1), 
          Byte.valueOf((byte)2), 
          Byte.valueOf((byte)3), 
          Byte.valueOf((byte)4), 
          Byte.valueOf((byte)5), 
          Byte.valueOf((byte)6), 
          Byte.valueOf((byte)7), 
          Byte.valueOf((byte)8), 
          Byte.valueOf((byte)9), 
          Byte.valueOf((byte)10), 
          Byte.valueOf((byte)11), 
          Byte.valueOf((byte)12), 
          Byte.valueOf((byte)13), 
          Byte.valueOf((byte)127), 
          Byte.valueOf((byte)-126), 
          Byte.valueOf((byte)-125), 
          Byte.valueOf((byte)-124), 
          Byte.valueOf((byte)14), 
          Byte.valueOf((byte)-1) }));
  
  public InvalidMessagesTest(ProgressDelegate progressDelegate, int payloadSize, long millisInterval, Integer totalIterations, Long millisRuntime, MessageController messageController) {
    super(progressDelegate, millisInterval, totalIterations, millisRuntime, messageController);
    this.payloadSize = payloadSize;
    this.millisInterval = millisInterval;
    this.totalIterations = totalIterations;
    this.millisRuntime = millisRuntime;
    this.messageController = messageController;
  }
  
  protected boolean runIteration(UUTestResult testResult, StringBuilder sb, long timeStart, Long millisRuntime, long millisInterval) {
    boolean success = true;
    boolean finished = false;
    byte i;
    for (i = Byte.MIN_VALUE; i != Byte.MAX_VALUE && !finished && this.doRun.get(); i = (byte)(i + 1)) {
      if (VALID_MSG_IDS.indexOf(Byte.valueOf(i)) < 0) {
        UnsafePSMessage uf = new UnsafePSMessage(ByteArrayUtils.getRandomApplicationData(this.payloadSize, null), i);
        this.messageController.sendMessage(uf);
        this.timeRunning = Calendar.getInstance().getTime().getTime() - timeStart;
        boolean connected = this.messageController.isConnected().get();
        success = (success && connected);
        if (!connected)
          sb.append("Lost connection in iteration ").append(this.totalIts)
            .append(" message count/type ").append(i)
            .append(" After ").append(this.timeRunning).append("ms\n"); 
        if (this.millisRuntime != null && this.timeRunning > this.millisRuntime.longValue())
          finished = true; 
        try {
          Thread.sleep(this.millisInterval);
        } catch (InterruptedException e) {
          e.printStackTrace();
        } 
      } 
    } 
    this.testResult.setSuccess((this.testResult.isSuccess() && success));
    return finished;
  }
  
  protected void setTestResult(StringBuilder sb) {
    this.testResult.setMessage("Ran " + this.totalIts + " iterations in " + this.timeRunning + "ms. Payload size: " + this.payloadSize + " interval: " + this.millisInterval + " ms");
  }
}
