package ubiik.wp.qa.view;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextArea;
import javafx.stage.Stage;
import ubiik.utils.communication.ProgressDelegate;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.messages.PSUplink;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandGetBaseChannel;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponse;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseGetBaseChannel;
import ubiik.wp.psconnection.PSConnectionManager;
import ubiik.wp.psconnection.PSConnectionManagerDelegate;
import ubiik.wp.psconnection.WSConnectionManager;
import ubiik.wp.qa.model.BSTest;
import ubiik.wp.qa.model.CommandTest;
import ubiik.wp.qa.model.WARFCNTest;
import ubiik.wp.view.UplinkSamplesViewController;

public class BSTestViewController extends UplinkSamplesViewController implements ProgressDelegate {
  private BSTest currentBsTest;
  
  private List<BSTest> bsTests;
  
  private Stage mainStage;
  
  private Thread testThread;
  
  private List<BSTest> disturbances;
  
  private List<Thread> threadDisturbances;
  
  @FXML
  protected TextArea textArea;
  
  @FXML
  protected CheckBox cbAck;
  
  @FXML
  protected Button btnStart;
  
  @FXML
  protected Button btnStop;
  
  public BSTestViewController(MessageController messageController, DevicesController devicesController, Stage mainStage) {
    super(messageController, devicesController);
    this.mainStage = mainStage;
    List<String> uuids = new ArrayList<>();
    this.bsTests = new LinkedList<>();
    uuids.add("02B0D8A0F3D8C4E910677040747946BC");
    List<PSCommand> commands = new LinkedList<>();
    List<PSCommandResponse> expectedResponses = new LinkedList<>();
    CommandTest cmdTest = new CommandTest(this, this.messageController, commands, expectedResponses, Long.valueOf(30000L));
    commands.add(new PSCommandGetBaseChannel((PSCommandDelegate)cmdTest));
    expectedResponses.add(new PSCommandResponseGetBaseChannel(0, 0, (short)8406));
    List<Integer> warfcns = new LinkedList<>();
    for (int i = 0; i <= 10000; i++) {
      warfcns.add(Integer.valueOf(8410));
      warfcns.add(Integer.valueOf(8405));
    } 
    WARFCNTest warfcnTest = new WARFCNTest(this, this.messageController, this.devicesController, 30000L, warfcns);
    this.bsTests.add(warfcnTest);
    this.currentBsTest = null;
    this.testThread = null;
    this.disturbances = new LinkedList<>();
    this.threadDisturbances = new LinkedList<>();
  }
  
  public PSConnectionManager getNewPSConnectionManager(String host, int port) {
    return (PSConnectionManager)new WSConnectionManager(host, port, (PSConnectionManagerDelegate)this.messageController, new LinkedBlockingQueue());
  }
  
  protected void processUplink(PSUplink uplink) {
    if (this.currentBsTest != null)
      this.currentBsTest.processUplink(uplink); 
  }
  
  public void setStateRunning(boolean running) {
    this.btnStart.setDisable(running);
    this.btnStop.setDisable(!running);
  }
  
  public void runTest() {
    if ((this.currentBsTest == null || (this.currentBsTest != null && !this.currentBsTest.running())) && this.bsTests
      .size() > 0) {
      setStateRunning(true);
      this.currentBsTest = this.bsTests.get(0);
      this.bsTests.remove(0);
      this.testThread = new Thread((Runnable)this.currentBsTest);
      this.testThread.start();
      for (BSTest disturbance : this.disturbances) {
        Thread t = new Thread((Runnable)disturbance);
        t.start();
        this.threadDisturbances.add(t);
      } 
    } else {
      setStateRunning(false);
    } 
  }
  
  public void btnStopPressed() {
    if (this.currentBsTest.running()) {
      this.currentBsTest.stop();
      try {
        this.testThread.join();
      } catch (InterruptedException e) {
        e.printStackTrace();
      } 
      this.testThread = null;
    } 
  }
  
  public void btnClearTerminalPressed() {
    this.textArea.clear();
  }
  
  public synchronized void btnLoadTestsPressed() {}
  
  public synchronized void btnStartPressed() {
    runTest();
  }
  
  public void updateProgress(int progress, int total) {}
  
  public void progressFinished(Exception error) {
    setStateRunning(false);
    if (error != null) {
      this.textArea.appendText(error.toString());
      this.textArea.appendText(error.getMessage());
    } 
    this.textArea.appendText(this.currentBsTest.getResult().isSuccess() ? "Test Passed:\n" : "Test Failed:\n");
    this.textArea.appendText(this.currentBsTest.getResult().getMessage() + "\n\n");
    for (BSTest disturbance : this.disturbances)
      disturbance.stop(); 
    for (Thread t : this.threadDisturbances) {
      try {
        t.join();
      } catch (InterruptedException e) {
        e.printStackTrace();
      } 
    } 
    this.threadDisturbances.clear();
    runTest();
  }
  
  public boolean getBSInfo() {return false;}
}
