package ubiik.wp.test;

import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import ubiik.wp.model.fpga.FPGARegister;
import ubiik.wp.model.fpga.FPGARegisterField;

public class FPGARegisterFieldTest extends TestCase {
  private FPGARegister register;
  
  @Before
  public void setUp() throws Exception {
    this.register = new FPGARegister(0);
  }
  
  @After
  public void tearDown() throws Exception {}
  
  @Test
  public void testValidateValue() {
    FPGARegisterField field = new FPGARegisterField(this.register, "rx_start_time", (short)31, (short)28, 0, 0, "lorem ipsum", true);
    assertTrue(field.validateValue(15));
    assertTrue(field.validateValue(0));
    assertFalse(field.validateValue(16));
    assertFalse(field.validateValue(-1));
    field.setMsb((short)31);
    field.setLsb((short)0);
    assertTrue(field.validateValue(-1));
    assertTrue(field.validateValue(0));
  }
  
  @Test
  public void testSetValueSame() {
    FPGARegisterField field = new FPGARegisterField(this.register, "rx_start_time", (short)31, (short)0, 0, 0, "lorem ipsum", true);
    int value = -1;
    field.setFrom32Bit(value);
    assertEquals(value, field.getAs32Bit());
    value = 0;
    field.setFrom32Bit(value);
    assertEquals(value, field.getAs32Bit());
    value = -11231;
    field.setFrom32Bit(value);
    assertEquals(value, field.getAs32Bit());
  }
  
  @Test
  public void testSetValue() {
    FPGARegisterField field = new FPGARegisterField(this.register, "rx_start_time", (short)2, (short)0, 0, 0, "lorem ipsum", true);
    int value = 7;
    field.setFrom32Bit(value);
    assertEquals(value, field.getAs32Bit());
    value = 15;
    field.setFrom32Bit(value);
    assertEquals(7, field.getAs32Bit());
    field.setMsb((short)7);
    field.setLsb((short)4);
    value = -1;
    field.setFrom32Bit(value);
    assertEquals(240, field.getAs32Bit());
    value = -16;
    field.setFrom32Bit(value);
    assertEquals(240, field.getAs32Bit());
    value = -208;
    field.setFrom32Bit(value);
    assertEquals(48, field.getAs32Bit());
  }
  
  @Test
  public void testConstructor() {
    try {
      FPGARegisterField field = new FPGARegisterField(this.register, "rx_start_time", (short)1, (short)28, 0, 0, "lorem ipsum", true);
      fail();
    } catch (IllegalArgumentException e) {
      assertTrue(true);
    } catch (Exception e) {
      e.printStackTrace();
      fail();
    } 
    try {
      FPGARegisterField field = new FPGARegisterField(this.register, "rx_start_time", (short)1, (short)1, 0, 0, "lorem ipsum", true);
      assertTrue(true);
    } catch (IllegalArgumentException e) {
      fail();
    } catch (Exception e) {
      e.printStackTrace();
      fail();
    } 
  }
}
