package ubiik.wp.test;

import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import ubiik.wp.model.fpga.FPGARegister;
import ubiik.wp.model.fpga.FPGARegisterField;

public class FPGARegisterTest extends TestCase {
  FPGARegister register;
  
  @Before
  protected void setUp() throws Exception {
    super.setUp();
    this.register = new FPGARegister(0);
    this.register.addField(new FPGARegisterField(this.register, "rx_start_time", (short)31, (short)18, 0, 0, "lorem ipsum", true));
    this.register.addField(new FPGARegisterField(this.register, "rx_start_time", (short)17, (short)4, 0, 0, "lorem ipsum", true));
    this.register.addField(new FPGARegisterField(this.register, "rx_start_time", (short)3, (short)3, 0, 0, "lorem ipsum", true));
    this.register.addField(new FPGARegisterField(this.register, "rx_start_time", (short)2, (short)2, 0, 0, "lorem ipsum", true));
    this.register.addField(new FPGARegisterField(this.register, "rx_start_time", (short)1, (short)1, 0, 0, "lorem ipsum", true));
    this.register.addField(new FPGARegisterField(this.register, "rx_start_time", (short)0, (short)0, 0, 0, "lorem ipsum", true));
  }
  
  @After
  protected void tearDown() throws Exception {
    super.tearDown();
  }
  
  @Test
  public void testSetFrom32Bits() {
    int value = -1;
    this.register.setFrom32Bit(value);
    assertEquals(value, this.register.getAs32Bit());
    ((FPGARegisterField)this.register.getFields().get(5)).setFrom32Bit(0);
    assertEquals(-2, this.register.getAs32Bit());
    ((FPGARegisterField)this.register.getFields().get(0)).setFrom32Bit(0);
    assertEquals(262142, this.register.getAs32Bit());
    value = 0;
    this.register.setFrom32Bit(value);
    assertEquals(value, this.register.getAs32Bit());
  }
  
  @Test
  public void testCast() {
    byte byteVal = 0;
    byte castByteVal = 0;
    for (int i = 0; i < 256; i++) {
      byteVal = (byte)(0xFF & i);
      castByteVal = (byte)i;
      assertEquals(byteVal, castByteVal);
      assertEquals(byteVal & 0xFF, i);
    } 
  }
  
  @Test
  public void testLongToInt() {
    long longVal = 4294967295L;
    assertEquals(-1, (int)longVal);
  }
}
