package ubiik.wp.test;

import java.util.concurrent.CountDownLatch;
import javafx.application.Platform;
import javafx.embed.swing.JFXPanel;
import javax.swing.SwingUtilities;
import org.junit.rules.TestRule;
import org.junit.runner.Description;
import org.junit.runners.model.Statement;

public class JavaFXThreadingRule implements TestRule {
  private static boolean jfxIsSetup;
  
  public Statement apply(Statement statement, Description description) {
    return new OnJFXThreadStatement(statement);
  }
  
  private static class OnJFXThreadStatement extends Statement {
    private final Statement statement;
    
    private Throwable rethrownException;
    
    public OnJFXThreadStatement(Statement aStatement) {
      this.rethrownException = null;
      this.statement = aStatement;
    }
    
    public void evaluate() throws Throwable {
      if (!JavaFXThreadingRule.jfxIsSetup) {
        setupJavaFX();
        JavaFXThreadingRule.jfxIsSetup = true;
      } 
      final CountDownLatch countDownLatch = new CountDownLatch(1);
      Platform.runLater(new Runnable() {
            public void run() {
              try {
                JavaFXThreadingRule.OnJFXThreadStatement.this.statement.evaluate();
              } catch (Throwable e) {
                JavaFXThreadingRule.OnJFXThreadStatement.this.rethrownException = e;
              } 
              countDownLatch.countDown();
            }
          });
      countDownLatch.await();
      if (this.rethrownException != null)
        throw this.rethrownException; 
    }
    
    protected void setupJavaFX() throws InterruptedException {
      final CountDownLatch latch = new CountDownLatch(1);
      SwingUtilities.invokeLater(new Runnable() {
            public void run() {
              new JFXPanel();
              latch.countDown();
            }
          });
      latch.await();
    }
  }
}
