package ubiik.wp.view;

import java.nio.charset.CharsetEncoder;
import java.nio.charset.StandardCharsets;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import ubiik.utils.view.InputHistoryHandler;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandATCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponse;

public class ATCommandViewController extends TestModeViewController implements PSCommandDelegate {
  @FXML
  private TextField textFieldInput;
  
  @FXML
  private Button btnSend;
  
  private InputHistoryHandler inputCommandHistoryHandler;
  
  public ATCommandViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
    this.inputCommandHistoryHandler = null;
  }
  
  public void initialize() {
    super.initialize();
    this.inputCommandHistoryHandler = new InputHistoryHandler(this.textFieldInput);
  }
  
  public void commandandReceivedResponse(PSCommand command) {
    super.commandandReceivedResponse(command);
    PSCommandResponse resp = command.getResponse();
    if (resp.isSuccess()) {
      setStatusBarCommandSuccess(command);
    } else {
      appendToTextArea(this.textArea, "\nCommand Failed");
      setStatusBarCommandFailed(command);
    } 
  }
  
  private boolean validateInput(String input) {
    boolean valid = false;
    if (input.length() > 0) {
      CharsetEncoder asciiEncoder = StandardCharsets.US_ASCII.newEncoder();
      valid = asciiEncoder.canEncode(input);
    } 
    return valid;
  }
  
  @FXML
  public void btnSendPressed() {
    String input = this.textFieldInput.getText();
    boolean valid = validateInput(input);
    setTextFieldHasError(this.textFieldInput, !valid);
    if (valid) {
      PSCommandATCommand cmd = new PSCommandATCommand(input, this);
      if (input.length() > 0)
        this.inputCommandHistoryHandler.addCommandToHistory(input); 
      this.inputCommandHistoryHandler.resetPosition();
      cleanInput(this.textFieldInput);
      sendCommandAndUpdateStatusBar((PSCommand)cmd);
      appendToTextArea(this.textArea, input + "\n");
    } 
  }
  
  public void updateTestModesFailed() {}
}
