package ubiik.wp.view;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.scene.Parent;
import javafx.scene.control.Alert;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import ubiik.utils.view.ValidationView;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.FileTransferManager;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.firmwareupgrade.BSFirmwareFileManager;
import ubiik.wp.model.firmwareupgrade.BSFirmwareFileValidationResult;
import ubiik.wp.model.firmwareupgrade.FirmwareUpgradeStatus;
import ubiik.wp.model.messages.pscommands.PSCommandDownloadLogs;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandATCommand;
import ubiik.wp.model.messages.pscommands.PSCommandBSFirmwareUpgrade;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandPutFileEx;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseBSFirmwareUpgrade;
import javax.swing.JFileChooser;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
//import net.lingala.zip4j.core.ZipFile;
//import net.lingala.zip4j.model.FileHeader;
//import net.lingala.zip4j.exception.ZipException;
//import javax.swing.UIManager;

public class AppSettingsViewController extends BaseViewController implements PSCommandDelegate {
  private Stage mainStage;
  
  private Alert blockingAlert;
  
  private Timer blockingAlertTimer;
  
  private static final long FIRMWARE_BLOCKING_TIMER_TIMEOUT = 10000L;
  
  private ConnectionManagerViewController connectionManagerViewController;
  
  private static final long DISCONNECT_TIMER = 2000L;
  
  private File selectedFile;
  private File selectedPath;
  
  private Timer firmwareTimer;
  
  @FXML
  private Label labelFwFileSelected;
  
  @FXML
  private Parent containerFirmwareUpgrade;
  
  @FXML
  private Label labelBSVersion;
  
  private static final long FIRMWARE_UPGRADE_TIMEOUT = 120000L;
  
  public static final int REFRESH_TIME_MIN = 50;
  
  public static final int REFRESH_TIME_MAX = 300000;
  
  private boolean silentFw;
  
  @FXML
  private TextField inputRefreshFrequency;
  
  @FXML
  private Label labelRefreshFrequency;
  
  @FXML
  private CheckBox checkboxRefreshViews;
  
  @FXML
  private Parent containerInactiveDevices;
  
  //@FXML
  //private TextField inputTimeBeforePing;
  
  //@FXML
  //private TextField inputPingResponseTimeout;
  
  //@FXML
  //private CheckBox checkboxTrackInactiveDevices;
  
  private FileTransferManager fileTransferManager;
  
  private File selectedFileToSend;
  
  @FXML
  private TextField inputFileName;
  
  @FXML
  private TextField inputUserName;
  
  @FXML
  private TextField inputGroupName;
  
  @FXML
  private TextField inputPermissions;
  
  public AppSettingsViewController(Stage mainStage, ConnectionManagerViewController connectionManagerViewController, DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
    this.mainStage = mainStage;
    this.firmwareTimer = null;
    this.connectionManagerViewController = connectionManagerViewController;
  }
  
  public void initialize() {
    
    this.silentFw = false;
    this.labelRefreshFrequency.setText(Integer.toString(RefreshableViewController.getRefreshFrequency()));
    
    boolean inactiveMonitoringEnabled = this.devicesController.isInactiveDevicesMonitoringEnabled();
    //this.checkboxTrackInactiveDevices.setSelected(inactiveMonitoringEnabled);
    Long timeBeforePing = this.devicesController.getInactiveDevicesInactivePeriod();
    if (timeBeforePing == null)
      timeBeforePing = Long.valueOf(300000L); 
   // this.inputTimeBeforePing.setText(Long.toString(timeBeforePing.longValue() / 1000L));
    Long pingTimeout = this.devicesController.getInactiveDevicesPingTimeout();
    if (pingTimeout == null)
      pingTimeout = Long.valueOf(60000L); 
    //this.inputPingResponseTimeout.setText(Long.toString(pingTimeout.longValue() / 1000L));
    
    this.fileTransferManager = null;
    this.selectedFileToSend = null;
    this.selectedPath = null;
    
  }
  
  private void updateBSVersion() {
    String str = "";
    String sBsVersion = "";
    byte[] bsVersion = this.devicesController.getBsVersion();
    if (bsVersion != null) {
      str = str + "Protocol Stack version ";
      for (int i = 0; i < bsVersion.length; i++) {
        	  sBsVersion = sBsVersion + bsVersion[i] ;
        if (bsVersion.length - 1 != i)
        	sBsVersion = sBsVersion + ".";
      }
      str = str + ": ";
      str = str + sBsVersion;
    } 
    Date bsBuildDate = this.devicesController.getBsBuildDate();
    if (bsBuildDate != null) {
      str = str + "build time ";
      str = str + str;
    } 
    this.labelBSVersion.setText(str);
  }
  
  public void doReloadView() {
    //updateBSVersion();
  }

  public void btnSetRefreshFrequencyPressed() {
    Integer refreshValue = ValidationView.validateAndGetIntegerTextFieldMinMax(this.inputRefreshFrequency, 50, 300000);
    if (refreshValue != null) {
      RefreshableViewController.setRefreshFrequency(refreshValue.intValue());
      this.labelRefreshFrequency.setText(Integer.toString(refreshValue.intValue()));
    } else {
      ValidationView.displayErrorMessage("Invalid Refresh Time", "Refresh Time must be between 50 and 300000 ms");
    } 
  }
  
  public void checkboxEnableRefreshChanged() {
    RefreshableViewController.setRefreshEnabled(this.checkboxRefreshViews.isSelected());
  }
  
  
  public void btnSelectFirmwareFilePressed() {
    FileChooser fileChooser = new FileChooser();
    fileChooser.setTitle("Open Firmware File");
    File selectedFile = fileChooser.showOpenDialog(this.mainStage);
    if (selectedFile != null)
      firmwareFileSelected(selectedFile); 
  }
  

  
  private void firmwareFileSelected(File firmwareFile) {
    byte[] currentVersion = this.devicesController.getBsVersion();
    BSFirmwareFileValidationResult result = BSFirmwareFileManager.validateFirmwareFile(firmwareFile, currentVersion);
    if (result.isValid()) {
      this.labelFwFileSelected.setText(firmwareFile.getPath());
      this.selectedFile = firmwareFile;
      String alertDesc = null;
      if (result.getFileCompareResult() == 0) {
        alertDesc = "The firmware file you are trying to load is the same version that is running currently";
      } else if (result.getFileCompareResult() < 0) {
        alertDesc = "The firmware file you are trying to load is older than the firmweare currently running";
      } 
      if (alertDesc != null)
        displayWarningMessage("Warning", alertDesc); 
    } else {
      displayErrorMessage("Invalid File", "Please select a valid Firmware File");
    } 
  }
  

  
  public void commandandReceivedResponse(PSCommand command) {
    if(command instanceof PSCommandDownloadLogs)
    {
      handleLogFileCommandResponse((PSCommandDownloadLogs)command);
    }
    else if (command instanceof PSCommandPutFileEx) {
      handlePutFileCommandResponse((PSCommandPutFileEx)command);
    } else if (command instanceof PSCommandBSFirmwareUpgrade) {
      if (!this.silentFw) {
        this.containerFirmwareUpgrade.setDisable(false);
      } else {
        this.blockingAlertTimer = new Timer();
        this.blockingAlertTimer.schedule(new TimerTask() {
              public void run() {
                AppSettingsViewController.this.hideBlockingAlert();
              }
            },  10000L);
      } 
      if (this.firmwareTimer != null)
        this.firmwareTimer.cancel(); 
      if (command.getResponse().isSuccess()) {
        if (!this.silentFw)
          setStatusBarCommandSuccess(command); 
        PSCommandResponseBSFirmwareUpgrade resp = (PSCommandResponseBSFirmwareUpgrade)command.getResponse();
        Integer errCode = resp.getFirmwareUpgradeResult();
        boolean success = (errCode != null);
        if (success) {
          if (!this.silentFw && 
            errCode.intValue() == 0)
            displayInfoMessage("Firmware Upgraded", "The new firmware has been installed. Please restart (power off/on) your base station to start running with the new firmware"); 
          success = (errCode.intValue() == 0);
          if (success && this.silentFw) {
            PSCommandATCommand cmd = new PSCommandATCommand("AT+REBOOT", this);
            sendCommandAndUpdateStatusBar((PSCommand)cmd, false);
            Timer disconnectTimer = new Timer();
            disconnectTimer.schedule(new TimerTask() {
                  public void run() {
                    AppSettingsViewController.this.connectionManagerViewController.disconnect();
                    AppSettingsViewController.this.connectionManagerViewController.disableUI(false);
                  }
                },  2000L);
          } 
        } 
        if (!success)
          if (!this.silentFw)
            displayErrorMessage("Firmware Upgrade Failed", FirmwareUpgradeStatus.getErrorForCode(errCode.intValue()));  
      } 
    } 
  }
  

  public void btnSelectFolderPressed() {
    JFileChooser f = new JFileChooser();
    //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    f.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    f.setDialogTitle("Open FOLDER");
    f.showSaveDialog(null);
    System.out.println(f.getCurrentDirectory());
    System.out.println(f.getSelectedFile());
    this.selectedPath = f.getSelectedFile();
  }


  public void btnReceiveFilePressed(){
      // Save the data read from Base Station and save in current device using FileOutputStream.
    try {
      if(this.selectedPath == null)
      {
        throw new Exception("No Folder Selected");
      }
      InputStream stream = new ByteArrayInputStream("GetLogs".getBytes("UTF-8"));
      PSCommandDownloadLogs cmd = new PSCommandDownloadLogs(stream, this);
      sendCommandAndUpdateStatusBar((PSCommand)cmd, true);
      this.commandStatusReceiver.setLastStatus("Downloading log files");
      //byte[] data = cmd.getMessageDataBytes();
      //String delta = new String(data);
      //System.out.println("this is data response: " + delta);
      } catch(Exception e) {
        displayErrorMessage("Error: ", e.getMessage());
      }
  }
  
  /**
  public void btnSendFirmwareFilePressed() {
    if (this.selectedFile != null) {
      this.containerFirmwareUpgrade.setDisable(true);
      try {
        doFWUpgrade(this.selectedFile, true, false);
      } catch (FileNotFoundException e) {
        displayErrorMessage("Invalid file", "Please select a valid file");
      } 
    } else {
      displayErrorMessage("No file selected", "Please select a file");
    } 
  }
  **/

  
  public void doFWUpgrade(File file, boolean displayBar, boolean silent) throws FileNotFoundException {
    doFWUpgrade(new FileInputStream(file), displayBar, silent);
  }
  
  public void doFWUpgrade(InputStream inputStream, boolean displayBar, boolean silent) {
    displayBlockingMessage("Initializing", "Initializing, please wait. Do not disconnect the device.", Alert.AlertType.INFORMATION);
    this.silentFw = silent;
    PSCommandBSFirmwareUpgrade cmd = new PSCommandBSFirmwareUpgrade(inputStream, this);
    sendCommandAndUpdateStatusBar((PSCommand)cmd, displayBar);
    this.firmwareTimer = new Timer();
    this.firmwareTimer.schedule(new TimerTask() {
          public void run() {
            AppSettingsViewController.this.containerFirmwareUpgrade.setDisable(false);
            AppSettingsViewController.this.displayWarningMessage("Firmware Update Timeout", "Firmware Update has timed out");
          }
        },120000L);
  }
  

   /** 
  public void checkboxInactiveDevicesChanged() {
    boolean cbSelected = this.checkboxTrackInactiveDevices.isSelected();
    this.containerInactiveDevices.setDisable(cbSelected);
    if (cbSelected) {
      this.devicesController.startTrackingInactiveDevices(this.messageController);
    } else {
      this.devicesController.stopTrackingInactiveDevices();
    } 
  }
  **/
  
  /** 
  public void btnSetInactivePeriodsPressed() {
    Integer selectedTimeBeforePing = validateAndGetIntegerTextFieldMinMax(this.inputTimeBeforePing, 1, 86400);
    Integer selectedPingTimeout = validateAndGetIntegerTextFieldMinMax(this.inputPingResponseTimeout, 1, 86400);
    if (selectedTimeBeforePing != null && selectedPingTimeout != null) {
      this.devicesController.setInactiveDevicesInactivePeriod(1000L * selectedTimeBeforePing.intValue());
      this.devicesController.setInactiveDevicesPingTimeout(1000L * selectedPingTimeout.intValue());
    } 
  }
  **/
  
  private void displayBlockingMessage(final String title, final String message, final Alert.AlertType alertType) {
    Platform.runLater(new Runnable() {
          public void run() {
            if (AppSettingsViewController.this.blockingAlert != null) {
              AppSettingsViewController.this.blockingAlert.hide();
              AppSettingsViewController.this.blockingAlert = null;
            } 
            AppSettingsViewController.this.blockingAlert = new Alert(alertType);
            AppSettingsViewController.this.blockingAlert.setTitle(title);
            AppSettingsViewController.this.blockingAlert.setContentText(message);
            AppSettingsViewController.this.blockingAlert.show();
          }
        });
  }
  
  private void hideBlockingAlert() {
    Platform.runLater(() -> {
          if (this.blockingAlert != null) {
            this.blockingAlert.hide();
            this.blockingAlert = null;
          } 
        });
  }
  
  public void btnSelectFilePressed() {
    FileChooser fileChooser = new FileChooser();
    fileChooser.setTitle("Open File");
    this.selectedFileToSend = fileChooser.showOpenDialog(this.mainStage);
  }
  
  public void btnSendFilePressed() {
    if (this.selectedFileToSend != null) {
      Integer permissions = ValidationView.validateAndGetIntegerTextFieldMinMax(this.inputPermissions, 0, 778);
      String filename = this.inputFileName.getText();
      if (filename.length() > 0 && permissions != null) {
        try {
          if (permissions.intValue() == 0) {
            permissions = Integer.valueOf(-1);
          } else {
            String strPerm = this.inputPermissions.getText();
            int permTemp = 0;
            for (int i = 0; i < 3; i++) {
              permTemp <<= 3;
              permTemp += strPerm.charAt(i) - 48;
            } 
            permissions = Integer.valueOf(permTemp);
          } 
          this
            .fileTransferManager = new FileTransferManager(this.selectedFileToSend, filename, this.inputUserName.getText(), this.inputGroupName.getText(), permissions.intValue());
          PSCommandPutFileEx cmd = this.fileTransferManager.getNextCommandFileFragment(this);
          sendCommandAndUpdateStatusBar((PSCommand)cmd);
        } catch (Exception e) {
          e.printStackTrace();
          displayErrorMessage("Invalid File", e.getMessage());
        } 
        this.selectedFileToSend = null;
      } else {
        displayErrorMessage("Error", "File Name (absolute path) must not be empty. Permissions must be between 0 and 777");
      } 
    } 
  }
  
  private void handlePutFileCommandResponse(PSCommandPutFileEx command) {
    if (command.getResponse().isSuccess()) {
      PSCommandPutFileEx nextCmd = this.fileTransferManager.getNextCommandFileFragment(this);
      if (nextCmd != null) {
        sendCommandAndUpdateStatusBar((PSCommand)nextCmd);
      } else {
        displayInfoMessage("File Sent", "The file has been sent");
        this.fileTransferManager = null;
      } 
    } else {
      this.fileTransferManager = null;
      displayErrorMessage("Error", "Failed to load file. " + command.getResponse().getErrorCode());
    } 
  }

  private void handleLogFileCommandResponse (PSCommandDownloadLogs command) {
    if (command.getResponse().isSuccess()) {
        try{
          //String delta = new String(command.getResponse().getCommandResponseData());
          //System.out.println("Response data is: " + delta + "\n\n");
          FileOutputStream fileOutputStreamLogs = new FileOutputStream(this.selectedPath + "/logs.zip");
          byte[] downloadData = command.getResponse().getCommandResponseData();
          fileOutputStreamLogs.write(downloadData);
          fileOutputStreamLogs.close();
          //String Pass = new String("Ubiik@Logs#");
          unzip(this.selectedPath + "/logs.zip", this.selectedPath.getAbsolutePath());
          File myObj = new File(this.selectedPath + "/logs.zip");
          myObj.delete();
          this.commandStatusReceiver.setLastStatus("Log Download Complete");
          displayInfoMessage("Log: ", "Log Download Completed.");
        }catch(Exception e) {
        displayErrorMessage("Error: ", e.getMessage());
      }
    }
    else
    {
      displayErrorMessage("Error: ", "could not download logs.");
    }
  }

  private static void unzip(String zipFilePath, String destDir) {
    File dir = new File(destDir);
    // create output directory if it doesn't exist
    if(!dir.exists()) dir.mkdirs();
    FileInputStream fis;
    //buffer for read and write data to file
    byte[] buffer = new byte[1024];
    try {
        fis = new FileInputStream(zipFilePath);
        ZipInputStream zis = new ZipInputStream(fis);
        ZipEntry ze = zis.getNextEntry();
        while(ze != null){
            String fileName = ze.getName();
            File newFile = new File(destDir + File.separator + fileName);
            System.out.println("Unzipping to "+newFile.getAbsolutePath());
            //create directories for sub directories in zip
            new File(newFile.getParent()).mkdirs();
            FileOutputStream fos = new FileOutputStream(newFile);
            int len;
            while ((len = zis.read(buffer)) > 0) {
            fos.write(buffer, 0, len);
            }
            fos.close();
            //close this ZipEntry
            zis.closeEntry();
            ze = zis.getNextEntry();
        }
        //close last ZipEntry
        zis.closeEntry();
        zis.close();
        fis.close();
    } catch (IOException e) {
        e.printStackTrace();
    }
}
}
