package ubiik.wp.view;

import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.controller.PSMessageStorage;
import ubiik.wp.util.ApplicationLogger;

public abstract class BaseApplication extends Application {
  protected static final int STAGE_MIN_W = 1024;
  
  protected static final int STAGE_MIN_H = 650;
  
  protected ConnectionManagerViewController mainController;
  
  protected FXMLLoader mainLoader;
  
  public void start(Stage primaryStage) {
    try {
      DevicesController dc = new DevicesController();
      MessageController mc = new MessageController(dc);
      PSMessageStorage.getInstance().setDevicesController(dc);
      doSetUpMainController(primaryStage, dc, mc);
      Parent root = this.mainLoader.<Parent>load();
      Scene scene = new Scene(root, 1024.0D, 650.0D);
      primaryStage.getIcons().add(new Image(getClass().getResourceAsStream("/res/ubiik_icon_152.png")));
      primaryStage.setScene(scene);
      primaryStage.show();
      primaryStage.setMinWidth(1024.0D);
      primaryStage.setMinHeight(650.0D);
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }
  
  protected abstract void doSetUpMainController(Stage paramStage, DevicesController paramDevicesController, MessageController paramMessageController);
  
  public void stop() throws Exception {
    super.stop();
    ApplicationLogger.debug("Application is closing");
    if (this.mainController != null)
      this.mainController.disconnect(); 
  }
}
