package ubiik.wp.view;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import ubiik.utils.view.TableUtils;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.capacity.MessageCapacityTest;
import ubiik.wp.model.capacity.MessageCapacityTestFactory;
import ubiik.wp.model.devices.DevicesStorage;
import ubiik.wp.model.devices.UUEndDevice;
import ubiik.wp.persistence.MessageCapacityTestJSONParser;

public class CapacityDownlinkViewController extends BaseViewController {
  @FXML
  private TableView<MessageCapacityTest> tableView;
  
  @FXML
  private TextField textFieldInterval;
  
  @FXML
  private TextField textFieldDataSize;
  
  @FXML
  private ChoiceBox<String> choiceBoxChannel;
  
  @FXML
  private ChoiceBox<UUEndDevice> choiceBoxEndDevice;
  
  @FXML
  private TextField textFieldTotalMessages;
  
  private final ObservableList<MessageCapacityTest> capacityTests = FXCollections.observableArrayList();
  
  private final Map<MessageCapacityTest, Button> startStopButtons;
  
  private Stage mainStage;
  
  public CapacityDownlinkViewController(Stage mainStage, DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
    this.startStopButtons = new HashMap<>();
    this.mainStage = mainStage;
  }
  
  public void doReloadView() {
    this.tableView.refresh();
    Collection<UUEndDevice> collection = DevicesStorage.getInstance().getDevices();
    if (this.choiceBoxEndDevice.getItems().size() != collection.size()) {
      this.choiceBoxEndDevice.getItems().clear();
      this.choiceBoxEndDevice.getItems().addAll(collection);
    } 
  }
  
  public void btnImportPressed() {
    FileChooser fileChooser = new FileChooser();
    fileChooser.setTitle("Open Multicast Groups File");
    File selectedFile = fileChooser.showOpenDialog(this.mainStage);
    if (selectedFile != null)
      try {
        InputStream is = new FileInputStream(selectedFile);
        List<MessageCapacityTest> capacityTests = MessageCapacityTestJSONParser.parseJSON(new InputStreamReader(is), this.messageController, this.devicesController);
        is.close();
        this.capacityTests.addAll(capacityTests);
      } catch (Exception e) {
        StringBuilder errMsg = new StringBuilder("Make sure you are trying to open a valid Capacity Tests File.");
        if (e.getMessage() != null) {
          errMsg.append("\nError: ");
          errMsg.append(e.getMessage());
        } 
        displayErrorMessage("Error parsing file", errMsg.toString());
      }  
  }
  
  public void btnExportPressed() {
    FileChooser fileChooser = new FileChooser();
    File file = fileChooser.showSaveDialog(this.mainStage);
    if (file != null)
      try {
        MessageCapacityTestJSONParser.capacityTestsToFile(this.capacityTests, file.getAbsolutePath());
      } catch (IOException e) {
        displayErrorMessage("Error saving file", e.getMessage());
      }  
  }
  
  public void btnStopAllPressed() {
    for (MessageCapacityTest mct : this.capacityTests)
      mct.stop(); 
  }
  
  public void btnCreatePressed() {
    int minDataSize, selectedChannelIdx = this.choiceBoxChannel.getSelectionModel().getSelectedIndex();
    Integer interval = validateAndGetIntegerTextFieldMinMax(this.textFieldInterval, 1000, 2147483647);
    if (selectedChannelIdx == 0 || selectedChannelIdx == 1) {
      minDataSize = 5;
    } else {
      minDataSize = 0;
    } 
    Integer dataSize = validateAndGetIntegerTextFieldMinMax(this.textFieldDataSize, minDataSize, 4096);
    Integer totalMsg = validateAndGetIntegerTextFieldMinMax(this.textFieldTotalMessages, 0, 2147483647);
    UUEndDevice ed = this.choiceBoxEndDevice.getValue();
    if (interval != null && dataSize != null) {
      MessageCapacityTest mcTest = MessageCapacityTestFactory.createTest(interval
          .intValue(), dataSize.intValue(), ed, selectedChannelIdx, this.messageController, this.devicesController, totalMsg.intValue());
      this.capacityTests.add(mcTest);
    } 
  }
  
  public void initialize() {
    this.tableView.getColumns().clear();
    TableColumn<MessageCapacityTest, UUEndDevice> deviceCol = new TableColumn<>("Device");
    deviceCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.25D));
    deviceCol.setSortable(true);
    deviceCol.setCellValueFactory(new PropertyValueFactory<>("endDevice"));
    TableColumn<MessageCapacityTest, Integer> dataSizeCol = new TableColumn<>("Data Size");
    dataSizeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    dataSizeCol.setSortable(true);
    dataSizeCol.setCellValueFactory(new PropertyValueFactory<>("dataSize"));
    TableColumn<MessageCapacityTest, String> channelCol = new TableColumn<>("Channel");
    channelCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    channelCol.setSortable(true);
    channelCol.setCellValueFactory(new PropertyValueFactory<>("channelName"));
    TableColumn<MessageCapacityTest, Integer> frequencyCol = new TableColumn<>("Interval (ms)");
    frequencyCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    frequencyCol.setSortable(true);
    frequencyCol.setCellValueFactory(new PropertyValueFactory<>("frequencyCol"));
    TableColumn<MessageCapacityTest, Boolean> statusCol = new TableColumn<>("Running");
    statusCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    statusCol.setSortable(true);
    statusCol.setCellValueFactory(new PropertyValueFactory<>("running"));
    TableColumn<MessageCapacityTest, Long> totalCol = new TableColumn<>("Sent");
    totalCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    totalCol.setSortable(true);
    totalCol.setCellValueFactory(new PropertyValueFactory<>("counter"));
    TableColumn<MessageCapacityTest, MessageCapacityTest> startStopColumn = TableUtils.column("", javafx.beans.property.ReadOnlyObjectWrapper::new, 60.0D);
    startStopColumn.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    startStopColumn.setSortable(false);
    startStopColumn.setCellFactory(col -> new TableCell<MessageCapacityTest, MessageCapacityTest>() {
          public void updateItem(MessageCapacityTest mct, boolean empty) {
            super.updateItem(mct, empty);
            Button startStopButton = (Button)CapacityDownlinkViewController.this.startStopButtons.get(mct);
            if (startStopButton == null) {
              startStopButton = new Button();
              CapacityDownlinkViewController.this.startStopButtons.put(mct, startStopButton);
            } 
            if (empty) {
              setGraphic((Node)null);
            } else {
              if (mct.isRunning()) {
                startStopButton.setText("STOP");
              } else {
                startStopButton.setText("START");
              } 
              startStopButton.setOnMousePressed(e -> CapacityDownlinkViewController.this.toggleMessageCapacityTest(mct));
              setGraphic(startStopButton);
            } 
          }
        });
    this.tableView.getColumns().addAll(new TableColumn[] { deviceCol, dataSizeCol, channelCol, statusCol, totalCol, startStopColumn });
    this.tableView.setItems(this.capacityTests);
    this.choiceBoxChannel.getItems().addAll(MessageCapacityTest.CHANNEL_NAMES);
  }
  
  public void toggleMessageCapacityTest(MessageCapacityTest mct) {
    if (mct.isRunning()) {
      mct.stop();
    } else {
      Thread t = new Thread((Runnable)mct);
      t.start();
    } 
  }
}
