package ubiik.wp.view;

import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.stage.Stage;
import javafx.util.Callback;
import ubiik.utils.view.ColumnFormatter;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.controller.PSMessageStorage;
import ubiik.wp.model.capacity.UplinkCapacitySample;
import ubiik.wp.model.messages.PSUplink;

public class CapacityUplinkViewController extends BaseViewController {
  protected PSUplink lastUplinkProcessed;
  
  @FXML
  TableView<UplinkCapacitySample> tableView;
  
  @FXML
  private Button btnToggleProcessing;
  
  @FXML
  private Label labelAvgDataRate;
  
  @FXML
  private Label labelPER;
  
  private ObservableList<UplinkCapacitySample> samples;
  
  private AtomicBoolean processing;
  
  private Double PER;
  
  private Double AvgDataRate;
  
  public CapacityUplinkViewController(Stage mainStage, DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
    this.processing = new AtomicBoolean(false);
    this.samples = FXCollections.observableArrayList();
  }
  
  public void initialize() {
    TableColumn<UplinkCapacitySample, Integer> sequenceCol = new TableColumn<>("#");
    TableColumn<UplinkCapacitySample, String> deviceCol = new TableColumn<>("Device");
    TableColumn<UplinkCapacitySample, Boolean> statusCol = createStatusCol(this.tableView, 0.1D);
    TableColumn<UplinkCapacitySample, String> dataCol = new TableColumn<>("Data");
    TableColumn<UplinkCapacitySample, Integer> dataSizeCol = new TableColumn<>("Bytes");
    TableColumn<UplinkCapacitySample, Integer> rssiCol = new TableColumn<>("RSSI");
    TableColumn<UplinkCapacitySample, String> modeCol = new TableColumn<>("Mode");
    TableColumn<UplinkCapacitySample, Date> datelCol = new TableColumn<>("Date");
    sequenceCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.07D));
    deviceCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.29D));
    dataCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.29D));
    dataSizeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.05D));
    rssiCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.05D));
    modeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    datelCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.15D));
    sequenceCol.setSortable(true);
    deviceCol.setSortable(true);
    dataCol.setSortable(false);
    dataSizeCol.setSortable(false);
    rssiCol.setSortable(true);
    modeCol.setSortable(true);
    datelCol.setSortable(true);
    sequenceCol.setCellValueFactory(new PropertyValueFactory<>("seqNumber"));
    deviceCol.setCellValueFactory(new PropertyValueFactory<>("deviceName"));
    dataCol.setCellValueFactory(new PropertyValueFactory<>("applicationDataString"));
    dataSizeCol.setCellValueFactory(new PropertyValueFactory<>("applicationDataSize"));
    rssiCol.setCellValueFactory(new PropertyValueFactory<>("rssi"));
    modeCol.setCellValueFactory(new PropertyValueFactory<>("endpointStr"));
    datelCol.setCellValueFactory(new PropertyValueFactory<>("timestamp"));
    datelCol.setCellFactory((Callback<TableColumn<UplinkCapacitySample, Date>, TableCell<UplinkCapacitySample, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    this.tableView.getColumns().addAll(new TableColumn[] { sequenceCol, deviceCol, statusCol, dataCol, dataSizeCol, rssiCol, modeCol, datelCol });
    this.tableView.setItems(this.samples);
  }
  
  public void update() {
    refreshData();
  }
  
  public void doReloadView() {
    update();
  }
  
  protected synchronized void processUplink(PSUplink uplink) {
    if (this.processing.get()) {
      UplinkCapacitySample ulSample = UplinkCapacitySample.createUplinkCapacitySampleFromUplink(uplink);
      if (ulSample != null) {
        this.samples.add(ulSample);
        if (this.samples.size() > 1) {
          if (this.AvgDataRate == null)
            this.AvgDataRate = Double.valueOf(0.0D); 
          long totalData = 0L;
          Date firstDate = ((UplinkCapacitySample)this.samples.get(0)).getTimestamp();
          for (UplinkCapacitySample ucs : this.samples)
            totalData += ucs.getUplink().getApplicationDataSize(); 
          this.AvgDataRate = Double.valueOf((8L * totalData) / (ulSample.getTimestamp().getTime() - firstDate.getTime()) / 1000.0D);
        } 
        if (this.samples.size() > 0)
          this.PER = Double.valueOf(1.0D - this.samples.stream().filter(s -> s.isValid()).count() / this.samples.size()); 
      } 
    } 
  }
  
  protected void refreshData() {
    PSMessageStorage ms = PSMessageStorage.getInstance();
    List<PSUplink> uplinks = ms.getUplinks();
    int idx = 0;
    if (this.lastUplinkProcessed != null) {
      int lastIdx = uplinks.indexOf(this.lastUplinkProcessed);
      if (lastIdx >= 0)
        idx = lastIdx + 1; 
    } 
    while (idx < uplinks.size()) {
      PSUplink uplink = uplinks.get(idx);
      this.lastUplinkProcessed = uplink;
      processUplink(uplink);
      idx++;
    } 
    refreshStatsLabels();
  }
  
  public void btnToggleProcessingPressed() {
    boolean isProcessing = this.processing.get();
    this.processing.set(!isProcessing);
    String btnText = "";
    if (isProcessing) {
      btnText = "START";
    } else {
      btnText = "STOP";
    } 
    this.btnToggleProcessing.setText(btnText);
  }
  
  public synchronized void btnClearListPressed() {
    this.samples.clear();
    this.PER = null;
    this.AvgDataRate = null;
  }
  
  public void btnRefreshPressed() {
    doReloadView();
  }
  
  public void refreshStatsLabels() {
    String text = "-";
    if (this.AvgDataRate != null)
      text = this.AvgDataRate.toString(); 
    setLabelText(this.labelAvgDataRate, text);
    String textPER = "-";
    if (this.PER != null)
      textPER = this.PER.toString(); 
    setLabelText(this.labelPER, textPER);
  }
  
  public static TableColumn<UplinkCapacitySample, Boolean> createStatusCol(TableView<UplinkCapacitySample> tableView, double widthProperty) {
    TableColumn<UplinkCapacitySample, Boolean> statusCol = new TableColumn<>("Status");
    statusCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    statusCol.setSortable(true);
    statusCol.setCellValueFactory(new PropertyValueFactory<>("valid"));
    statusCol.setCellFactory(column -> new TableCell<UplinkCapacitySample, Boolean>() {
          protected void updateItem(Boolean item, boolean empty) {
            super.updateItem(item, empty);
            if (item == null || empty) {
              setText((String)null);
              setStyle("");
            } else {
              String color;
              if (item.booleanValue()) {
                setText("OK");
                color = DevicesViewController.DEVICE_STATUS_COLORS.get(Byte.valueOf((byte)2));
              } else {
                setText("ERROR");
                color = DevicesViewController.DEVICE_STATUS_COLORS.get(Byte.valueOf((byte)3));
              } 
              setStyle("-fx-background-color:" + color);
            } 
          }
        });
    return statusCol;
  }
}
