package ubiik.wp.view;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.controller.UserManager;
import ubiik.wp.model.bsparameters.BSParameter;
import ubiik.wp.model.devices.UUBaseStation;
import ubiik.wp.model.messages.PSMessage;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandBlacklistChannels;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandGetBaseChannel;
import ubiik.wp.model.messages.pscommands.PSCommandGetULBaseChannel;
import ubiik.wp.model.messages.pscommands.PSCommandParameter;
import ubiik.wp.model.messages.pscommands.PSCommandReadBlacklistedChannels;
import ubiik.wp.model.messages.pscommands.PSCommandReadMulticastGroups;
import ubiik.wp.model.messages.pscommands.PSCommandRemoveBlacklistChannel;
import ubiik.wp.model.messages.pscommands.PSCommandRunMode;
import ubiik.wp.model.messages.pscommands.PSCommandSetBaseChannel;
import ubiik.wp.model.messages.pscommands.PSCommandSetBlacklistedChannels;
import ubiik.wp.model.messages.pscommands.PSCommandSetCurrentSettingsAsDefault;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseGetBaseChannel;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseGetULBaseChannel;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseReadBlacklistedChannels;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseReadMulticastGroups;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseRunMode;

public class ConfigurationViewController extends BaseViewController implements PSCommandDelegate {
  @FXML
  private Label labelBSVersion;

  @FXML
  private VBox commandsContainer;

  @FXML
  private VBox baseFrequencyContainer;

  @FXML
  private TextField inputNewFrequencyValue;

  @FXML
  private Button buttonSetBaseFrequency;

  @FXML
  private Label labelLastFreqValue;

  @FXML
  private Label UlLastFreqValue;

  @FXML
  private Label DlLastFreqValue;

  @FXML
  private Label labelBaseFreqMaxVal;

  @FXML
  private Label labelBaseFreqMinVal;

  @FXML
  private Pane blacklistChannelContainer;

  @FXML
  private TextField inputNewBlacklistedChannel;

  @FXML
  private Button buttonBlacklistChannel;

  @FXML
  private Label labelLastRunMode;

  public static final String[] RUN_MODE_STATUS = new String[] { "DISCONNECTED", "CONNECTED" };

  private ArrayList<BSParameterViewController> bsParameterVCs;

  @FXML
  private Pane bsParametersContainer;

  public ConfigurationViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
    this.bsParameterVCs = new ArrayList<>();
  }

  public void initialize() {
    if (!UserManager.getInstance().getPsAuth().isCommandAvailable(PSCommandSetBaseChannel.class))
      this.commandsContainer.getChildren().remove(this.baseFrequencyContainer);
    if (!UserManager.getInstance().getPsAuth().isCommandAvailable(PSCommandReadBlacklistedChannels.class))
      this.commandsContainer.getChildren().remove(this.blacklistChannelContainer);
    if (UserManager.getInstance().getPsAuth()
      .isCommandAvailable(PSCommandParameter.class))
      loadBSParameters();
  }

  private void loadBSParameters() {
    ArrayList<BSParameter> parameters = this.devicesController.getBSParameters();
    for (int i = 0; i < parameters.size(); i++) {
      if (((BSParameter)parameters.get(i)).getCategory().compareTo("UT") != 0) {
        BSParameterViewController vc = new BSParameterViewController(this.devicesController, this.messageController, this.commandStatusReceiver, i);
        this.bsParameterVCs.add(vc);
        String res;
        if(((BSParameter)parameters.get(i)).getName().compareTo("BS_TX_PWR") != 0 )
            res = "/main/resources/ubiik/wp/admin/BSParameterGetView.fxml";
            //FXMLLoader loader = new FXMLLoader(getClass().getResource("/main/resources/ubiik/wp/admin/BSParameterGetView.fxml"));
        else
            res = "/main/resources/ubiik/wp/admin/BSParameterView.fxml";  
        FXMLLoader loader = new FXMLLoader(getClass().getResource(res)); 
        loader.setController(vc);
        try {
          this.bsParametersContainer.getChildren().add(loader.<Node>load());
        } catch (IOException e) {
          e.printStackTrace();
        }
      }
    }
  }

  public void sendCommandGetAllParameters() {
    for (BSParameterViewController vc : this.bsParameterVCs)
      vc.sendCommandGetParameter(false);
  }

  public void readStatus() {
    List<PSMessage> messages = new LinkedList<>();
    messages.add(new PSCommandRunMode(this, (byte)0, 0));
    messages.add(new PSCommandGetBaseChannel(this));
    messages.add(new PSCommandGetULBaseChannel(this));
    for (BSParameterViewController vc : this.bsParameterVCs)
      messages.add(vc.getCommandGetParameter());
    //messages.add(new PSCommandReadBlacklistedChannels(this));
    //messages.add(new PSCommandReadMulticastGroups(this));
    this.messageController.sendMessages(messages);
  }

  @FXML
  public void btnGetULEARFCNPressed() {
    sendCommandGetULBaseChannel();
  }
  @FXML
  public void btnGetDLEARFCNPressed() {
    sendCommandGetBaseChannel();
  }

  public void sendCommandGetBaseChannel() {
    PSCommandGetBaseChannel cmd = new PSCommandGetBaseChannel(this);
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  public void sendCommandGetULBaseChannel() {
    PSCommandGetULBaseChannel cmd = new PSCommandGetULBaseChannel(this);
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  @FXML
  public void btnSetBaseFrequencyPressed() {
    boolean valid = true;
    short channel = -1;
    try {
      channel = Short.parseShort(this.inputNewFrequencyValue.getText());
    } catch (NumberFormatException e) {
      valid = false;
    }
    valid = UUBaseStation.validateBaseChannel(channel);
    setTextFieldHasError(this.inputNewFrequencyValue, !valid);
    if (valid) {
      Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
      alert.setTitle("Set Base Channel");
      alert.setHeaderText("Setting the Base Channel will cause the Base Station to restart. All End Devices connected must also be restarted and set to the new channel.");
      alert.setContentText("Continue?");
      Optional<ButtonType> result = alert.showAndWait();
      if (result.isPresent() && result.get() == ButtonType.OK) {
        cleanInput(this.inputNewFrequencyValue);
        PSCommandSetBaseChannel cmd = new PSCommandSetBaseChannel(channel, this);
        sendCommandAndUpdateStatusBar((PSCommand)cmd);
      }
    }
  }

  @FXML
  public void btnStartPressed() {
    Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
    alert.setTitle("Restart goRAN");
    alert.setHeaderText("Please click the Connect button after 10 seconds");
    alert.setContentText("");
    Optional<ButtonType> result = alert.showAndWait();
    if (result.isPresent() && result.get() == ButtonType.OK)
      sendCommandAndUpdateStatusBar((PSCommand)new PSCommandRunMode(this, (byte)1, 1));
  }

  @FXML
  public void btnStopPressed() {
    sendCommandAndUpdateStatusBar((PSCommand)new PSCommandRunMode(this, (byte)1, 0));
  }

  @FXML
  public void getRunModePressed() {
    sendCommandGetRunMode();
  }

  public void sendCommandGetRunMode() {
    sendCommandAndUpdateStatusBar((PSCommand)new PSCommandRunMode(this, (byte)0, 0));
  }

  private void updateLabel(final Label label, final String text) {
    Platform.runLater(new Runnable() {
          public void run() {
            label.setText(text);
          }
        });
  }

  private void addBlacklistedChannel(int[] channels) {
    for (int channel : channels) {
      if (this.devicesController.addBlacklistedChannel(channel)) {
        Button btnRemove = new Button();
        btnRemove.setId(Integer.toString(channel));
        btnRemove.setText("Remove");
        EventHandler<ActionEvent> btnHandler = new EventHandler<ActionEvent>() {
            public void handle(ActionEvent event) {
              Object source = event.getSource();
              if (source instanceof Button) {
                Button clickedBtn = (Button)source;
                ConfigurationViewController.this.btnRemoveBlacklistedChannelPressed(
                    Short.parseShort(clickedBtn.getId()));
              }
              event.consume();
            }
          };
        btnRemove.setOnAction(btnHandler);
        final HBox channelContainer = new HBox();
        channelContainer.setAlignment(Pos.CENTER_LEFT);
        channelContainer.setSpacing(5.0D);
        channelContainer.setId("channel_container_" + Integer.toString(channel));
        channelContainer.getChildren().add(new Label("Channel " +
              Integer.toString(channel)));
        channelContainer.getChildren().add(btnRemove);
        Platform.runLater(new Runnable() {
              public void run() {
                ConfigurationViewController.this.blacklistChannelContainer.getChildren().add(channelContainer);
              }
            });
      }
    }
  }

  private void removeBlacklistedChannel(int[] channels) {
    for (int channel : channels) {
      this.devicesController.removeBlacklistedChannel(channel);
      final HBox channelContainer = (HBox)this.blacklistChannelContainer.lookup("#channel_container_" + Integer.toString(channel));
      if (channelContainer != null)
        Platform.runLater(new Runnable() {
              public void run() {
                ConfigurationViewController.this.blacklistChannelContainer.getChildren()
                  .remove(channelContainer);
              }
            });
    }
  }

  private void clearBlacklistedChannels() {
    removeBlacklistedChannel(this.devicesController
        .getBlacklistedChannels()
        .stream().mapToInt(i -> i.intValue()).toArray());
  }

  @FXML
  public void btnReadBlacklistChannelsPressed() {
    sendCommandGetBlacklistedChannels();
  }

  public void sendCommandGetBlacklistedChannels() {
    sendCommandAndUpdateStatusBar((PSCommand)new PSCommandReadBlacklistedChannels(this));
  }

  @FXML
  public void btnReadMulticastGroupsPressed() {
    sendCommandReadMulticastGroups();
  }

  public void sendCommandReadMulticastGroups() {
    PSCommandReadMulticastGroups cmd = new PSCommandReadMulticastGroups(this);
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  @FXML
  public void btnAddBlacklistedChannelPressed() {
    boolean valid = true;
    ArrayList<Integer> channels = new ArrayList<>();
    try {
      List<String> items = Arrays.asList(this.inputNewBlacklistedChannel
          .getText().split("\\s*,\\s*"));
      for (String item : items) {
        short channel = Short.parseShort(item);
        if (channel > 9297 || channel < 1382) {
          valid = false;
          continue;
        }
        channels.add(Integer.valueOf(channel));
      }
    } catch (NumberFormatException e) {
      valid = false;
    }
    setTextFieldHasError(this.inputNewBlacklistedChannel, !valid);
    if (valid)
      sendCommandAndUpdateStatusBar((PSCommand)new PSCommandBlacklistChannels(channels

            .stream().mapToInt(i -> i.intValue()).toArray(), this));
  }

  @FXML
  public void btnSetBlacklistedChannelsPressed() {
    Alert alert = new Alert(Alert.AlertType.WARNING);
    alert.setTitle("Set Blacklisted Channels");
    alert.setHeaderText("You are about to override the Blacklisted Channels");
    List<Integer> blacklistedChannels = this.devicesController.getBlacklistedChannels();
    StringBuilder sb = new StringBuilder();
    if (blacklistedChannels.size() != 0) {
      sb.append("The following channels will be set as blacklisted:\n");
      sb.append(blacklistedChannels.stream().map(Object::toString)
          .collect(Collectors.joining(", ")));
      sb.append("\nAll other channels will not be blacklisted any longer.");
    } else {
      sb.append("There will be no blacklisted channels after this action");
    }
    sb.append("\n\nContinue?");
    alert.setContentText(sb.toString());
    Optional<ButtonType> result = alert.showAndWait();
    if (result.get() == ButtonType.OK) {
      int[] channels = this.devicesController.getBlacklistedChannels().stream().mapToInt(i -> i.intValue()).toArray();
      sendCommandAndUpdateStatusBar((PSCommand)new PSCommandSetBlacklistedChannels(channels, this));
    }
  }

  public void btnRemoveBlacklistedChannelPressed(short channel) {
    int[] channels = { channel };
    sendCommandAndUpdateStatusBar((PSCommand)new PSCommandRemoveBlacklistChannel(channels, this));
  }


  
   
  public void commandandReceivedResponse(PSCommand command) {
    //Mobin's old codes
    /* 
    if (command.getResponse().isSuccess()) {
      setStatusBarCommandSuccess(command);
      if (command instanceof PSCommandSetBaseChannel) {
        PSCommandSetBaseChannel bsfreqcommand = (PSCommandSetBaseChannel)command;
        updateLabel(this.labelLastFreqValue,
            Integer.toString(bsfreqcommand.getChannel()));
        this.devicesController.setBaseFrequency(bsfreqcommand.getChannel());
      } else if (command instanceof PSCommandGetULBaseChannel) {
        if (command.getResponse() instanceof PSCommandResponseGetULBaseChannel) {
          int baseFrequency = ((PSCommandResponseGetULBaseChannel)command.getResponse()).getBaseChannel();
          updateLabel(this.UlLastFreqValue,
              Integer.toString(baseFrequency));
          this.devicesController.setBaseFrequency(baseFrequency);
        }
      } else if (command instanceof PSCommandGetBaseChannel)
      {
       if (command.getResponse() instanceof PSCommandResponseGetBaseChannel) {
          int baseFrequency = ((PSCommandResponseGetBaseChannel)command.getResponse()).getBaseChannel();
          updateLabel(this.DlLastFreqValue,
              Integer.toString(baseFrequency));
          this.devicesController.setBaseFrequency(baseFrequency);
        }
      }
      else if (command instanceof PSCommandRemoveBlacklistChannel) {
        removeBlacklistedChannel(((PSCommandRemoveBlacklistChannel)command).getChannel());
      } else if (command instanceof PSCommandBlacklistChannels) {
        addBlacklistedChannel(((PSCommandBlacklistChannels)command).getChannel());
      } else if (command instanceof PSCommandReadBlacklistedChannels) {
        try {
          clearBlacklistedChannels();
          PSCommandResponseReadBlacklistedChannels response = (PSCommandResponseReadBlacklistedChannels)command.getResponse();
          addBlacklistedChannel(response.getChannels());
        } catch (Exception e) {
          e.printStackTrace();
          displayErrorMessage("Command Failed", "Read Blacklisted Channels failed");
        }
      } else if (command instanceof PSCommandReadMulticastGroups) {
        try {
          PSCommandResponseReadMulticastGroups resp = (PSCommandResponseReadMulticastGroups)command.getResponse();
          this.devicesController.setMulticastGroups(resp.getMulticastGroups(), resp.getDevicesStatusInMulticastGroups());
        } catch (Exception e) {
          e.printStackTrace();
          displayErrorMessage("Error", "Error reading command response for Read Multicast Groups");
        }
      } else if (command instanceof PSCommandRunMode) {
        PSCommandRunMode runModeCmd = (PSCommandRunMode)command;
        if (runModeCmd.isSet()) {
          this.devicesController.setRunMode(runModeCmd.getMode());
        } else if (runModeCmd.isGet()) {
          PSCommandResponseRunMode resp = (PSCommandResponseRunMode)command.getResponse();
          this.devicesController.setRunMode(resp.getMode());
        }
      }
    } else {
      displayErrorMessage("Error", "Error reading command response");
      setStatusBarCommandFailed(command);
    }
    */
  }
  

  @FXML
  public void btnSaveCurrentAsDefaultPressed() {
    sendCommandSaveCurrentAsDefault();
  }

  public void sendCommandSaveCurrentAsDefault() {
    PSCommandSetCurrentSettingsAsDefault cmd = new PSCommandSetCurrentSettingsAsDefault(this);
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  private void updateBsParameters() {
    for (BSParameterViewController bsParameterViewController : this.bsParameterVCs)
      bsParameterViewController.updateInput();
  }

  private void updateRunMode() {
    Integer mode = this.devicesController.getRunMode();
    String modeStr = "Connecting.....";
    if (mode != null &&
      mode.intValue() < RUN_MODE_STATUS.length)
      modeStr = RUN_MODE_STATUS[mode.intValue()]; 
    setLabelText(this.labelLastRunMode, modeStr);
  }

  private void updateBSVersion() {
    String str = "";
    String sBsVersion = "";
    byte[] bsVersion = this.devicesController.getBsVersion();
    if (bsVersion != null) {
      str = str + "ENB Controller Version ";
      for (int i = 0; i < bsVersion.length; i++) {
    	  sBsVersion = sBsVersion + bsVersion[i] ;
        if (bsVersion.length - 1 != i)
        	sBsVersion = sBsVersion + ".";
      }
      str = str + ": ";
      str = str + sBsVersion;
    } else {
      str = "No ENB Controller Version" ;
    }
    this.labelBSVersion.setText(str);
  }

  public void doReloadView() {
    updateBsParameters();
    //Mobin's old codes
    //updateRunMode();
    updateBSVersion();
  }
}
