package ubiik.wp.view;

import java.util.ArrayList;
import java.util.List;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.scene.Parent;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.TextField;
import javafx.scene.control.Toggle;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.Pane;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MCSController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.devices.ScheduleTransferConfig;
import ubiik.wp.model.devices.UUEndDevice;
import ubiik.wp.model.devices.mcs.MCSSetting;
import ubiik.wp.model.messages.PSPong;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandEDRASlots;
import ubiik.wp.model.messages.pscommands.PSCommandGetScheduleTransfer;
import ubiik.wp.model.messages.pscommands.PSCommandMCSSetting;
import ubiik.wp.model.messages.pscommands.PSCommandMCSValue;
import ubiik.wp.model.messages.pscommands.PSCommandPingEndDevice;
import ubiik.wp.model.messages.pscommands.PSCommandSetScheduleTransfer;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseEDRASlots;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseGetScheduleTransfer;
import ubiik.wp.util.ApplicationLogger;

public class DeviceViewController extends BaseViewController implements PSCommandDelegate {
  private UUEndDevice uuEndDevice;

  @FXML
  private TextField labelTitle;

  @FXML
  private Label labelUplinksCounter;

  @FXML
  private Label labelDownlinksCounter;

  @FXML
  private Label labelLastUplink;

  @FXML
  private Label labelLastDownlink;

  @FXML
  private Label labelStatus;

  @FXML
  private TextField inputPingDataSize;

  @FXML
  private Label labelLastPing;

  @FXML
  private Label labelLastPingDelay;

  @FXML
  private Parent containerInputsIntervalWindow;

  @FXML
  private TextField inputInterval;

  @FXML
  private TextField inputWindow;

  @FXML
  private RadioButton radioSet;

  @FXML
  private RadioButton radioRelease;

  @FXML
  private Label labelScheduleTransferFlag;

  @FXML
  private Parent containerScheduleTWindowInterval;

  @FXML
  private Label labelCurrentSTInterval;

  @FXML
  private Label labelCurrentSTWindow;

  @FXML
  private Button btnGetScheduleTransfer;

  @FXML
  private Pane containerAllowedDLMCSValues;

  @FXML
  private Pane containerAllowedULMCSValues;

  @FXML
  private TextField inputDLMCS;

  @FXML
  private TextField inputULMCS;

  @FXML
  private ChoiceBox<String> choiceboxULMCSValues;

  @FXML
  private ChoiceBox<String> choiceboxDLMCSValues;

  @FXML
  private ChoiceBox<String> choiceboxULMCSValuesNB;

  @FXML
  private Label labelCurrentULMCSValue;

  @FXML
  private Label labelCurrentULMCSValueNB;

  @FXML
  private Label labelCurrentDLMCSValue;

  @FXML
  private Label labelDLMCScurrentAllowed;

  @FXML
  private Label labelULMCScurrentAllowed;

  @FXML
  private Label labelRAStatus;

  @FXML
  private TextField inputRASlots;

  @FXML
  private TextField inputRASlotsSlow;

  @FXML
  private RadioButton radioRAGlobal;

  @FXML
  private RadioButton radioRAUseAll;

  @FXML
  private RadioButton radioRAValues;

  private List<CheckBox> ulMCSCheckboxes;

  private List<CheckBox> dlMCSCheckboxes;

  private ToggleGroup radioGroup;

  private MCSController mcsController;

  public DeviceViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver, UUEndDevice uuEndDevice) {
    super(devicesController, messageController, commandStatusReceiver);
    this.uuEndDevice = uuEndDevice;
    this.mcsController = new MCSController(devicesController);
  }

  public void initialize() {
    this.dlMCSCheckboxes = new ArrayList<>();
    this.ulMCSCheckboxes = new ArrayList<>();
    this.btnGetScheduleTransfer.setVisible(false);
    this.labelTitle.setText("End Device " + this.uuEndDevice.getUUIDString());
    this.radioGroup = new ToggleGroup();
    this.radioSet.setUserData(Byte.valueOf((byte)1));
    this.radioSet.setToggleGroup(this.radioGroup);
    this.radioRelease.setUserData(Byte.valueOf((byte)0));
    this.radioRelease.setToggleGroup(this.radioGroup);
    this.radioGroup.selectedToggleProperty().addListener(new ChangeListener<Toggle>() {
          public void changed(ObservableValue<? extends Toggle> ov, Toggle old_toggle, Toggle new_toggle) {
            if (DeviceViewController.this.radioGroup.getSelectedToggle() != null) {
              Byte i_flag = (Byte)DeviceViewController.this.radioGroup.getSelectedToggle().getUserData();
              boolean disableInputs = true;
              if (i_flag != null && i_flag.byteValue() == 1)
                disableInputs = false;
              DeviceViewController.this.containerInputsIntervalWindow.setDisable(disableInputs);
            }
          }
        });
    setMCSCheckboxes();
    reloadView();
  }

  protected void setConnectionLabels() {
    this.labelUplinksCounter.setText(Long.toString(this.uuEndDevice.getUplinksCounter()));
    String lastUplinkStr = (this.uuEndDevice.getLastUplinkDate() != null) ? DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE.format(this.uuEndDevice.getLastUplinkDate()) : "-";
    this.labelLastUplink.setText(lastUplinkStr);
    this.labelDownlinksCounter.setText(Long.toString(this.uuEndDevice.getDownlinksCounter()));
    String lastDownlinkStr = (this.uuEndDevice.getLastDownlinkDate() != null) ? DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE.format(this.uuEndDevice.getLastDownlinkDate()) : "-";
    this.labelLastDownlink.setText(lastDownlinkStr);
    this.labelStatus.setText(this.uuEndDevice.getStatusString());
  }

  protected void setScheduleTransferUI() {
    ScheduleTransferConfig stConf = this.uuEndDevice.getScheduleTransferConfig();
    boolean displayWindowInterval = false;
    String stFlag = "";
    if (stConf != null) {
      if (stConf.getFlag() == 0) {
        stFlag = "Release";
        displayWindowInterval = false;
      } else {
        stFlag = "Set";
        displayWindowInterval = true;
        this.labelCurrentSTInterval.setText(Integer.toString(stConf.getInterval()));
        this.labelCurrentSTWindow.setText(Integer.toString(stConf.getWindow()));
      }
    } else {
      stFlag = "---";
      displayWindowInterval = false;
    }
    this.containerScheduleTWindowInterval.setVisible(displayWindowInterval);
    this.labelScheduleTransferFlag.setText(stFlag);
  }

  @FXML
  public void btnGetScheduleTransferPressed() {
    PSCommandGetScheduleTransfer cmd = new PSCommandGetScheduleTransfer(this, this.uuEndDevice.getUuid());
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  @FXML
  public void btnSetScheduleTransferPressed() {
    Byte flag = (Byte)this.radioGroup.getSelectedToggle().getUserData();
    if (flag != null) {
      PSCommandSetScheduleTransfer pSCommandSetScheduleTransfer = null;
      PSCommand cmd = null;
      if (flag.byteValue() == 0) {
        pSCommandSetScheduleTransfer = new PSCommandSetScheduleTransfer(this, this.uuEndDevice.getUuid(), flag.byteValue(), 0, 0);
      } else if (flag.byteValue() == 1) {
        Integer interval = validateAndGetIntegerTextFieldMinMax(this.inputInterval, 2, 1024);
        Integer window = null;
        if (interval != null) {
          window = validateAndGetIntegerTextFieldMinMax(this.inputWindow, 2, interval

              .intValue());
          if (window != null) {
            boolean valid = ScheduleTransferConfig.validateIntervalVsWindow(interval.intValue(), window.intValue());
            if (valid) {
              pSCommandSetScheduleTransfer = new PSCommandSetScheduleTransfer(this, this.uuEndDevice.getUuid(), flag.byteValue(), interval.intValue(), window.intValue());
            } else {
              setTextFieldHasError(this.inputInterval, true);
              setTextFieldHasError(this.inputWindow, true);
            }
          }
        }
      }
      if (pSCommandSetScheduleTransfer != null) {
        sendCommandAndUpdateStatusBar((PSCommand)pSCommandSetScheduleTransfer);
        this.inputInterval.setText("");
        this.inputWindow.setText("");
      }
    }
  }

  public void commandandReceivedResponse(PSCommand command) {
    if (command.getResponse().isSuccess()) {
      setStatusBarCommandSuccess(command);
      if (command instanceof PSCommandSetScheduleTransfer) {
        PSCommandSetScheduleTransfer cmd = (PSCommandSetScheduleTransfer)command;
        this.devicesController.setScheduleTransferForDevice(cmd.getDeviceUuid(), cmd
            .getFlag(), cmd
            .getWindow(), cmd
            .getInterval());
      } else if (command instanceof PSCommandGetScheduleTransfer) {
        if (command.getResponse() instanceof PSCommandResponseGetScheduleTransfer) {
          PSCommandResponseGetScheduleTransfer cmdResp = (PSCommandResponseGetScheduleTransfer)command.getResponse();
          this.devicesController.setScheduleTransferForDevice(cmdResp.getDeviceUuid(), cmdResp
              .getFlag(), cmdResp
              .getWindow(), cmdResp
              .getInterval());
        }
      } else if (command instanceof PSCommandMCSValue) {
        this.mcsController.processMCSValuesCommandResponse((PSCommandMCSValue)command);
      } else if (command instanceof PSCommandMCSSetting) {
        this.mcsController.processMCSCommandResponse((PSCommandMCSSetting)command);
      } else if (command instanceof PSCommandEDRASlots) {
        handleRACommandResponse((PSCommandEDRASlots)command);
      }
    } else {
      displayErrorMessage("Error", "The command failed");
      setStatusBarCommandFailed(command);
    }
  }

  public void doReloadView() {
    setConnectionLabels();
    setScheduleTransferUI();
    setMCSLabels();
    setPingLabels();
  }

  public void setMCSCheckboxes() {
    MCSViewController.setMCSSettingsView(this.uuEndDevice, this.choiceboxULMCSValues, this.choiceboxULMCSValuesNB, this.choiceboxDLMCSValues, this.ulMCSCheckboxes, this.dlMCSCheckboxes, this.containerAllowedDLMCSValues, this.containerAllowedULMCSValues);
  }

  public void setMCSLabels() {
    MCSSetting mcs = this.uuEndDevice.getMCSSetting();
    if (mcs != null) {
      if (mcs.isCurrentDownlinkMcsValueSet())
        this.labelCurrentDLMCSValue.setText(MCSSetting.DL_MCS_NAMES.get(mcs.getCurrentDownlinkMcsValue()));
      if (mcs.isCurrentuplinkMcsValueSet())
        this.labelCurrentULMCSValue.setText(MCSSetting.UL_MCS_NAMES.get(mcs.getCurrentuplinkMcsValue()));
      if (mcs.isCurrentuplinkMcsValueNBSet())
        this.labelCurrentULMCSValueNB.setText(MCSSetting.UL_MCS_NAMES_NB.get(mcs.getCurrentuplinkMcsValueNB()));
      StringBuilder sb = new StringBuilder();
      for (Integer dlmcs : mcs.getIndexesDLMCSAvailable()) {
        sb.append(MCSSetting.DL_MCS_NAMES.get(dlmcs.intValue()));
        sb.append(" ");
      }
      this.labelDLMCScurrentAllowed.setText(sb.toString());
      sb = new StringBuilder();
      for (Integer ulmcs : mcs.getIndexesULMCSAvailable()) {
        sb.append(MCSSetting.UL_MCS_NAMES.get(ulmcs.intValue()));
        sb.append(" ");
      }
      this.labelULMCScurrentAllowed.setText(sb.toString());
    }
  }

  private void setPingLabels() {
    PSPong pong = this.uuEndDevice.getLastPong();
    if (pong != null) {
      this.labelLastPing.setText(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE.format(pong.getPingTimestamp()));
      this.labelLastPingDelay.setText("" + pong.getRoundtripDelay() + " ms");
    }
  }

  public void btnGetCurrentMCSValuePressed() {
    PSCommandMCSValue cmd = new PSCommandMCSValue(this, (byte)0, this.uuEndDevice.getUuid());
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  public void btnSetCurrentMCSValuePressed() {
    int dlIdx = this.choiceboxDLMCSValues.getSelectionModel().getSelectedIndex();
    int ulIdx = this.choiceboxULMCSValues.getSelectionModel().getSelectedIndex();
    int ulNBIdx = this.choiceboxULMCSValuesNB.getSelectionModel().getSelectedIndex();
    if (dlIdx >= 0 && ulIdx >= 0 && ulNBIdx >= 0) {
      PSCommandMCSValue cmd = new PSCommandMCSValue(this, (byte)1, this.uuEndDevice.getUuid(), (byte)dlIdx, (byte)ulIdx, (byte)ulNBIdx);
      sendCommandAndUpdateStatusBar((PSCommand)cmd);
    } else {
      displayErrorMessage("Select values", "Please select a value for Uplink and Downlink MCS");
    }
  }

  public void btnGetAllowedMCSPressed() {
    PSCommandMCSSetting cmd = new PSCommandMCSSetting(this, (byte)0, this.uuEndDevice.getUuid());
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  public void btnSetAllowedMCSPressed() {
    int flagsUl = MCSViewController.getFlagsFromCheckboxes(this.ulMCSCheckboxes);
    int flagsDl = MCSViewController.getFlagsFromCheckboxes(this.dlMCSCheckboxes);
    if (flagsUl != 0 && flagsDl != 0) {
      PSCommandMCSSetting cmd = new PSCommandMCSSetting(this, (byte)1, this.uuEndDevice.getUuid(), (short)flagsUl, (short)flagsDl);
      sendCommandAndUpdateStatusBar((PSCommand)cmd);
      clearMCSCheckboxes();
    } else {
      displayErrorMessage("Select values", "Please select at least one value for Uplink and Downlink MCS");
    }
  }

  private void clearMCSCheckboxes() {
    for (CheckBox cb : this.dlMCSCheckboxes)
      cb.setSelected(false);
    for (CheckBox cb : this.ulMCSCheckboxes)
      cb.setSelected(false);
  }

  public void btnPingPressed() {
    Integer dataSize = validateAndGetIntegerTextFieldMinMax(this.inputPingDataSize, 0, 4096);
    if (dataSize != null) {
      PSCommandPingEndDevice cmd = new PSCommandPingEndDevice(this, this.uuEndDevice.getUuid(), dataSize.intValue(), (int)this.devicesController.getNextPingId());
      this.devicesController.addPing(cmd);
      sendCommandAndUpdateStatusBar((PSCommand)cmd);
      cleanInput(this.inputPingDataSize);
    }
  }

  public void btnGetRASlots() {
    PSCommandEDRASlots cmd = new PSCommandEDRASlots(this, (byte)0, this.uuEndDevice.getUuid());
    sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }

  public void btnSetRASlots() {
    short raSlots = 0;
    short raSlotsSlow = 0;
    boolean valid = true;
    if (this.radioRAGlobal.isSelected()) {
      raSlots = 0;
      raSlotsSlow = 0;
    } else if (this.radioRAUseAll.isSelected()) {
      raSlots = -1;
      raSlotsSlow = -1;
    } else {
      Integer intRaSlots = validateAndGetIntegerTextFieldMinMax(this.inputRASlots, 1, 15);
      Integer intRaSlotsSlow = validateAndGetIntegerTextFieldMinMax(this.inputRASlotsSlow, 1, 15);
      if (intRaSlots != null && intRaSlotsSlow != null) {
        raSlots = intRaSlots.shortValue();
        raSlotsSlow = intRaSlotsSlow.shortValue();
      } else {
        valid = false;
      }
    }
    if (valid) {
      PSCommandEDRASlots cmd = new PSCommandEDRASlots(this, (byte)1, this.uuEndDevice.getUuid(), raSlots, raSlotsSlow);
      sendCommandAndUpdateStatusBar((PSCommand)cmd);
    }
  }

  private void handleRACommandResponse(PSCommandEDRASlots cmd) {
    short raSlots = 0;
    short raSlotsSlow = 0;
    boolean valid = false;
    if (cmd.getResponse().isSuccess()) {
      if (cmd.isGet()) {
        if (cmd.getResponse() instanceof PSCommandResponseEDRASlots) {
          PSCommandResponseEDRASlots resp = (PSCommandResponseEDRASlots)cmd.getResponse();
          raSlots = resp.getRASlots();
          raSlotsSlow = resp.getRASlotsSlowMcs();
          valid = true;
        } else {
          ApplicationLogger.debug("Received invalid response for PSCommandEDRASlots");
        }
      } else if (cmd.isSet()) {
        raSlots = cmd.getNumRaSlots();
        raSlotsSlow = cmd.getNumRaSlotsSlowMcs();
        valid = true;
      } else {
        ApplicationLogger.debug("Received invalid response operation for PSCommandEDRASlots");
      }
      String labelText = "";
      if (valid) {
        if (raSlots == 0 && raSlotsSlow == 0) {
          labelText = "Using default RA settings";
        } else if (raSlots == -1 && raSlotsSlow == -1) {
          labelText = "Using all available RA slots";
        } else {
          labelText = "RA slots: " + raSlots + " | RA slots slow MCS: " + raSlotsSlow;
        }
        setLabelText(this.labelRAStatus, labelText);
      }
    }
  }
}
