package ubiik.wp.view;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.util.Callback;
import ubiik.utils.view.ColumnFormatter;
import ubiik.utils.view.TableUtils;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.controller.PSMessageStorage;
import ubiik.wp.model.devices.DevicesStorage;
import ubiik.wp.model.devices.UUEndDevice;

public class DevicesViewController extends BaseViewController {
  private PSMessageStorage messageStorage;
  
  private MainViewController mainViewController;
  
  @FXML
  private Label labelEDCount;
  
  @FXML
  private Label labelEDsRegistered;
  
  @FXML
  private Label labelEDsConnected;
  
  @FXML
  private Label labelUplinksCount;
  
  @FXML
  private Label labelDownlinksCount;
  
  @FXML
  private Label labelLastChannel;
  
  @FXML
  private TableView<UUEndDevice> tableView;
  
  private final ObservableList<UUEndDevice> data = FXCollections.observableArrayList();
  
  public static final Map<Byte, String> DEVICE_STATUS_COLORS = new HashMap<>();
  
  static {
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)2), "#71A1F0");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)1), "#FEDF00");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)3), "#FFBE92");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)0), "#BBBBBB");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)-2), "#BBBBBB");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)4), "#FF6600");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)5), "#FF6600");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)6), "#90EE90");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)7), "#CC00CC");
    DEVICE_STATUS_COLORS.put(Byte.valueOf((byte)-1), "#F0833E");
  }
  
  public DevicesViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver, MainViewController mainViewController) {
    super(devicesController, messageController, commandStatusReceiver);
    this.messageStorage = PSMessageStorage.getInstance();
    this.mainViewController = mainViewController;
  }
  
  public void initialize() {
    this.tableView.getColumns().clear();
    TableColumn<UUEndDevice, Integer> idxCol = createIdxCol(this.tableView, 0.05D);
    TableColumn<UUEndDevice, String> deviceCol = createNameCol(this.tableView, 0.20D);
    TableColumn<UUEndDevice, Byte> statusCol = createStatusCol(this.tableView, 0.10D);
    TableColumn<UUEndDevice, Long> uplinksCol = createUplinksCol(this.tableView, 0.125D);
    TableColumn<UUEndDevice, Long> downlinksCol = createDownlinksCol(this.tableView, 0.125D);
    TableColumn<UUEndDevice, Date> datelCol = createDateCreatedCol(this.tableView, 0.15D);
    //TableColumn<UUEndDevice, Short> regRssiCol = createRegRssiCol(this.tableView, 0.055D);
    //TableColumn<UUEndDevice, Date> lastActivityCol = createLastActivityCol(this.tableView, 0.125D);
    TableColumn<UUEndDevice, Short> ulRssiCol = createUlRssiCol(this.tableView, 0.125D);
    TableColumn<UUEndDevice, String> endDeviceTypeCol = createEndDeviceTypeCol(this.tableView, 0.1D);
    //TableColumn<UUEndDevice, String> firmwareVersionCol = createFirmwareVersionCol(this.tableView, 0.075D);
    //TableColumn<UUEndDevice, Long> joinFoiCol = createJoinFoiCol(this.tableView, 0.055D);
    TableColumn<UUEndDevice, UUEndDevice> editColumn = TableUtils.column("", javafx.beans.property.ReadOnlyObjectWrapper::new, 60.0D);
    editColumn.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    editColumn.setSortable(false);
    editColumn.setCellFactory(col -> {
          //final Button editButton = new Button("Details");
          TableCell<UUEndDevice, UUEndDevice> cell = new TableCell<UUEndDevice, UUEndDevice>() {
              public void updateItem(UUEndDevice ed, boolean empty) {
                super.updateItem(ed, empty);
                if (empty) {
                  setGraphic((Node)null);
                } else {
            //      setGraphic(editButton);
                } 
              }
            };
          //editButton.setOnMousePressed((e) -> {
        	 // showUuedDetailView(cell.getTableRow().getItem());
          //});
          return cell;
        });
   // this.tableView.getColumns().addAll(new TableColumn[] { idxCol, endDeviceTypeCol,
   // 		deviceCol, statusCol, datelCol, regRssiCol, lastActivityCol, ulRssiCol, uplinksCol, downlinksCol, firmwareVersionCol, joinFoiCol, editColumn });
        this.tableView.getColumns().addAll(new TableColumn[] { idxCol, endDeviceTypeCol,
          deviceCol, statusCol,datelCol,ulRssiCol, uplinksCol, downlinksCol });
    this.tableView.setItems(this.data);
    reloadView();
  }
  
  public void showUuedDetailView(UUEndDevice uued) {
    this.mainViewController.displayEndDeviceView(uued);
  }
  
  public void doReloadView() {
    UUEndDevice selectedDevice = this.tableView.getSelectionModel().getSelectedItem();
    this.labelUplinksCount.setText(Long.toString(this.messageStorage.getUplinksCount()));
    this.labelDownlinksCount.setText(Long.toString(this.messageStorage.getDownlinksCount()));
    this.labelEDCount.setText(Long.toString(DevicesStorage.getInstance().getDevices().size()));
    long connectedCount = DevicesStorage.getInstance().getDevices().stream().filter(ed -> (ed.getStatus() == 2)).count();
    this.labelEDsConnected.setText(Long.toString(connectedCount));
    long registeredCount = DevicesStorage.getInstance().getDevices().stream().filter(ed -> (ed.getStatus() == 6)).count();
    this.labelEDsRegistered.setText(Long.toString(registeredCount));
    this.data.clear();
    for (UUEndDevice uuEndDevice : DevicesStorage.getInstance().getDevices())
      this.data.add(uuEndDevice);
    this.tableView.sort();
    this.tableView.refresh();
    this.tableView.getSelectionModel().select(selectedDevice);
  }
  
  public void btnRefreshPressed() {
    doReloadView();
  }
  
  public static TableColumn<UUEndDevice, Byte> createStatusCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Byte> statusCol = new TableColumn<>("Status");
    statusCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    statusCol.setSortable(true);
    statusCol.setCellValueFactory(new PropertyValueFactory<>("status"));
    statusCol.setCellFactory(column -> new TableCell<UUEndDevice, Byte>() {
          protected void updateItem(Byte item, boolean empty) {
            super.updateItem(item, empty);
            if (item == null || empty) {
              setText((String)null);
              setStyle("");
            } else {
              setText(UUEndDevice.getStringForStatus(item));
              String color = DevicesViewController.DEVICE_STATUS_COLORS.get(item);
              if (color != null) {
                setStyle("-fx-background-color:" + color);
              } else {
                setStyle("");
              } 
            } 
          }
        });
    return statusCol;
  }
  
  public static TableColumn<UUEndDevice, Integer> createIdxCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Integer> idxCol = new TableColumn<>("#");
    idxCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    idxCol.setSortable(true);
    idxCol.setCellValueFactory(new PropertyValueFactory<>("idx"));
    return idxCol;
  }
  
  public static TableColumn<UUEndDevice, String> createNameCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, String> deviceCol = new TableColumn<>("UE IMSI");
    deviceCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    deviceCol.setSortable(true);
    deviceCol.setCellValueFactory(new PropertyValueFactory<>("name"));
    return deviceCol;
  }
  
  public static TableColumn<UUEndDevice, Long> createUplinksCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Long> uplinksCol = new TableColumn<>("UL Bytes");
    uplinksCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    uplinksCol.setSortable(true);
    uplinksCol.setCellValueFactory(new PropertyValueFactory<>("uplinksCounter"));
    return uplinksCol;
  }
  
  public static TableColumn<UUEndDevice, Long> createDownlinksCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Long> downlinksCol = new TableColumn<>("DL Bytes");
    downlinksCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    downlinksCol.setSortable(true);
    downlinksCol.setCellValueFactory(new PropertyValueFactory<>("downlinksCounter"));
    return downlinksCol;
  }
  
   
  public static TableColumn<UUEndDevice, Date> createDateCreatedCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Date> datelCol = new TableColumn<>("Last Updated");
    datelCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    datelCol.setSortable(true);
    datelCol.setCellValueFactory(new PropertyValueFactory<>("dateAdded"));
    datelCol.setCellFactory((Callback<TableColumn<UUEndDevice, Date>, TableCell<UUEndDevice, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    return datelCol;
  }
  
  
  public static TableColumn<UUEndDevice, Short> createRegRssiCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Short> regRssiCol = new TableColumn<>("Reg RSSI");
    regRssiCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    regRssiCol.setSortable(true);
    regRssiCol.setCellValueFactory(new PropertyValueFactory<>("registrationRssi"));
    return regRssiCol;
  }
  
  public static TableColumn<UUEndDevice, Date> createLastUplinklCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Date> lastUplinklCol = new TableColumn<>("Last Uplink");
    lastUplinklCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    lastUplinklCol.setSortable(true);
    lastUplinklCol.setCellValueFactory(new PropertyValueFactory<>("lastUplinkDate"));
    lastUplinklCol.setCellFactory((Callback<TableColumn<UUEndDevice, Date>, TableCell<UUEndDevice, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    return lastUplinklCol;
  }
  
  public static TableColumn<UUEndDevice, Date> createLastActivityCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Date> lastUplinklCol = new TableColumn<>("Last Activity");
    lastUplinklCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    lastUplinklCol.setSortable(true);
    lastUplinklCol.setCellValueFactory(new PropertyValueFactory<>("lastActivity"));
    lastUplinklCol.setCellFactory((Callback<TableColumn<UUEndDevice, Date>, TableCell<UUEndDevice, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    return lastUplinklCol;
  }
   
  public static TableColumn<UUEndDevice, Short> createUlRssiCol(TableView<UUEndDevice> tableView, double widthProperty) {
    TableColumn<UUEndDevice, Short> ulRssiCol = new TableColumn<>("UL SNR");
    ulRssiCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    ulRssiCol.setSortable(true);
    ulRssiCol.setCellValueFactory(new PropertyValueFactory<>("lastUplinkRssi"));
    return ulRssiCol;
  }
  
  public static TableColumn<UUEndDevice, String> createEndDeviceTypeCol(TableView<UUEndDevice> tableView, double widthProperty) {
	TableColumn<UUEndDevice, String> endDeviceTypeCol = new TableColumn<>("Device Type");
	endDeviceTypeCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
	endDeviceTypeCol.setSortable(true);
	endDeviceTypeCol.setCellValueFactory(new PropertyValueFactory<>("endDeviceType"));
	return endDeviceTypeCol;
  }
   
  public static TableColumn<UUEndDevice, String> createFirmwareVersionCol(TableView<UUEndDevice> tableView, double widthProperty) {
	TableColumn<UUEndDevice, String> firmwareVersionCol = new TableColumn<>("Firmware Version");
	firmwareVersionCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
	firmwareVersionCol.setSortable(true);
	firmwareVersionCol.setCellValueFactory(new PropertyValueFactory<>("firmwareVersion"));
	return firmwareVersionCol;
  }
  
  
  public static TableColumn<UUEndDevice, Long> createJoinFoiCol(TableView<UUEndDevice> tableView, double widthProperty) {
		TableColumn<UUEndDevice, Long> joinFoiCol = new TableColumn<>("joinFoi");
		joinFoiCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
		joinFoiCol.setSortable(true);
		joinFoiCol.setCellValueFactory(new PropertyValueFactory<>("joinFoi"));
		return joinFoiCol;
	  }
}

