package ubiik.wp.view;

import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableRow;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.TextFormatter;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.Clipboard;
import javafx.scene.input.ClipboardContent;
import javafx.scene.layout.Pane;
import javafx.stage.FileChooser;
import javafx.util.Callback;
import org.controlsfx.control.textfield.TextFields;
import ubiik.utils.DatatypeConverter;
import ubiik.utils.view.ColumnFormatter;
import ubiik.utils.view.InputHistoryHandler;
import ubiik.utils.view.TableUtils;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.controller.PSMessageStorage;
import ubiik.wp.model.devices.DevicesStorage;
import ubiik.wp.model.devices.UUEndDevice;
import ubiik.wp.model.messages.DownlinkAcknowledgement;
import ubiik.wp.model.messages.PSDownlink;
import ubiik.wp.model.messages.PSDownlinkATCommand;
import ubiik.wp.model.messages.PSMessage;
import ubiik.wp.model.messages.PSMulticastDownlink;

public class DownlinksViewController extends BaseViewController {
  private static final int DOWNLINKS_IN_LIST = 500;
  
  @FXML
  protected CheckBox checkboxMulticast;
  
  @FXML
  protected CheckBox checkboxAck;
  
  @FXML
  private TextField inputEDUUIDs;
  
  @FXML
  protected TextField inputDownlinkData;
  
  @FXML
  protected Pane downlinkActionsContainer;
  
  @FXML
  protected Pane containerDownlinkExtraAction;
  
  @FXML
  protected ComboBox<Object> comboEUUID;
  
  @FXML
  protected CheckBox checkboxAllDevices;
  
  @FXML
  protected CheckBox checkboxListDevices;
  
  private List<UUEndDevice> comboEds;
  
  @FXML
  protected Label labelFileSelected;
  
  @FXML
  protected TableView<PSDownlink> tableView;
  
  private File currentFirmwareFile;
  
  private final ObservableList<PSDownlink> data = FXCollections.observableArrayList();
  
  private boolean enableExtraAction;
  
  @FXML
  protected TextField inputDownlinkExtraAction;
  
  @FXML
  private Label labelDownlinkExtraAction;
  
  @FXML
  private Button btnExtraDownlinkAction;
  
  private InputHistoryHandler inputDataHistoryHandler;
  
  private InputHistoryHandler inputExtraHistoryHandler;
  
  public DownlinksViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver, boolean enableExtraAction) {
    super(devicesController, messageController, commandStatusReceiver);
    this.currentFirmwareFile = null;
    this.enableExtraAction = enableExtraAction;
    this.comboEds = null;
    this.inputDataHistoryHandler = null;
    this.inputExtraHistoryHandler = null;
  }
  
  public void initialize() {
    this.tableView.getColumns().clear();
    TableColumn<PSDownlink, String> idCol = new TableColumn<>("ID");
    TableColumn<PSDownlink, String> deviceCol = new TableColumn<>("Device / Group");
    TableColumn<PSDownlink, String> dataCol = new TableColumn<>("Data");
    TableColumn<PSDownlink, Integer> dataSizeCol = new TableColumn<>("Bytes");
    TableColumn<PSDownlink, Integer> modeCol = new TableColumn<>("Mode");
    TableColumn<PSDownlink, Integer> statusCol = new TableColumn<>("Status");
    TableColumn<PSDownlink, Date> datelCol = new TableColumn<>("Date");
    idCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.07D));
    deviceCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.25D));
    dataCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.25D));
    dataSizeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    modeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    statusCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.08D));
    datelCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.15D));
    idCol.setSortable(true);
    deviceCol.setSortable(true);
    dataCol.setSortable(false);
    dataSizeCol.setSortable(false);
    modeCol.setSortable(true);
    statusCol.setSortable(true);
    datelCol.setSortable(true);
    idCol.setCellValueFactory(new PropertyValueFactory<>("applicationId"));
    deviceCol.setCellValueFactory(new PropertyValueFactory<>("targetName"));
    dataCol.setCellValueFactory(new PropertyValueFactory<>("applicationDataString"));
    dataSizeCol.setCellValueFactory(new PropertyValueFactory<>("applicationDataSize"));
    modeCol.setCellValueFactory(new PropertyValueFactory<>("endpointStr"));
    datelCol.setCellValueFactory(new PropertyValueFactory<>("timestamp"));
    datelCol.setCellFactory((Callback<TableColumn<PSDownlink, Date>, TableCell<PSDownlink, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    statusCol.setCellValueFactory(new PropertyValueFactory<>("ackStatus"));
    statusCol.setCellFactory(column -> new TableCell<PSDownlink, Integer>() {
          protected void updateItem(Integer item, boolean empty) {
            super.updateItem(item, empty);
            if (item == null || empty) {
              setText((String)null);
              setStyle("");
            } else {
              String intHex = Integer.toHexString(item.intValue());
              setText("0x" + ((intHex.length() % 2 == 0) ? intHex : ("0" + intHex)));
              setText(DownlinkAcknowledgement.getNameForStatus(item));
              if (item.intValue() == 0) {
                setStyle("-fx-background-color: #90EE90");
              } else {
                setStyle("-fx-background-color: tomato");
              } 
            } 
          }
        });
    this.tableView.getColumns().addAll(new TableColumn[] { idCol, deviceCol, dataCol, dataSizeCol, modeCol, statusCol, datelCol });
    this.tableView.setRowFactory(tv -> new TableRow<PSDownlink>() {
          public void updateItem(PSDownlink item, boolean empty) {
            super.updateItem(item, empty);
            if (item == null) {
              setStyle("");
            } else if (item.getTargetExists() != null && 
              !item.getTargetExists().booleanValue()) {
              setStyle("-fx-background-color: tomato;");
            } 
          }
        });
    this.tableView.setItems(this.data);
    this.checkboxMulticast.setSelected(false);
    setupIdInput(false);
    if (!this.enableExtraAction)
      this.downlinkActionsContainer.getChildren().remove(this.containerDownlinkExtraAction); 
    this.inputEDUUIDs.setTextFormatter(new TextFormatter(c -> {
            String text = ((TextFormatter.Change)c).getText();
            int oldAnchor = ((TextFormatter.Change)c).getAnchor();
            int oldCaretPos = ((TextFormatter.Change)c).getCaretPosition();
            int initialLength = text.length();
            text = text.replaceAll("\t", " ");
            text = text.replaceAll("\n", " ");
            text = text.replaceAll("\r", " ");
            ((TextFormatter.Change)c).setText(text);
            ((TextFormatter.Change)c).setAnchor(oldAnchor + text.length() - initialLength);
            ((TextFormatter.Change)c).setCaretPosition(oldCaretPos + text.length() - initialLength);
            return c;
          }));
    this.labelDownlinkExtraAction.setText("AT Command");
    cleanInput(this.inputDownlinkExtraAction, "AT");
    this.inputDownlinkExtraAction.setPromptText("AT Command");
    this.btnExtraDownlinkAction.setText("Send Command");
    this.containerDownlinkExtraAction.setVisible(true);
    reloadView();
    TableUtils.installCopyPasteHandler(this.tableView);
    this.inputDataHistoryHandler = new InputHistoryHandler(this.inputDownlinkData);
    this.inputExtraHistoryHandler = new InputHistoryHandler(this.inputDownlinkExtraAction);
  }
  
  private void setupIdInput(final boolean isMulticast) {
    Platform.runLater(new Runnable() {
          public void run() {
            DownlinksViewController.this.comboEUUID.getItems().clear();
            if (isMulticast) {
              DownlinksViewController.this.comboEUUID.getItems().addAll((Collection)DownlinksViewController.this.devicesController
                  .getMulticastGroups().keySet()
                  .stream().map(s -> s.toString()).collect(Collectors.toList()));
              DownlinksViewController.this.comboEUUID.setEditable(false);
            } else {
              DownlinksViewController.this.comboEds = new LinkedList(DevicesStorage.getInstance().getDevices());
              DownlinksViewController.this.comboEUUID.getItems().addAll(DownlinksViewController.this.comboEds);
              DownlinksViewController.this.comboEUUID.setEditable(true);
              TextFields.bindAutoCompletion(DownlinksViewController.this.comboEUUID.getEditor(), DownlinksViewController.this.comboEUUID.getItems());
              if (DownlinksViewController.this.comboEUUID.getItems().size() > 0)
                DownlinksViewController.this.comboEUUID.getSelectionModel().select(0); 
            } 
          }
        });
  }
  
  private void setupDataInput() {
    this.inputDownlinkData.setText("");
  }
  
  @FXML
  public void checkboxMulticastChanged() {
    setupIdInput(this.checkboxMulticast.isSelected());
    if (this.checkboxMulticast.isSelected()) {
      this.checkboxListDevices.setSelected(false);
      this.checkboxAllDevices.setSelected(false);
    } 
  }
  
  @FXML
  public void checkboxListDevicesChanged() {
    if (this.checkboxListDevices.isSelected()) {
      this.checkboxMulticast.setSelected(false);
      this.checkboxAllDevices.setSelected(false);
    } 
  }
  
  @FXML
  public void checkboxAllDevicesChanged() {
    if (this.checkboxAllDevices.isSelected()) {
      this.checkboxMulticast.setSelected(false);
      this.checkboxListDevices.setSelected(false);
    } 
  }
  
  protected boolean validateAndSendDownlink(byte[] dataToSend, int endpoint) {
    byte[] target = null;
    byte[] data = dataToSend;
    List<byte[]> devices = (List)new LinkedList<>();
    List<PSDownlink> downlinks = new LinkedList<>();
    boolean valid = true;
    String error = "";
    boolean isMulticast = this.checkboxMulticast.isSelected();
    boolean sendToAllEDs = this.checkboxAllDevices.isSelected();
    boolean sendToListOfEDs = this.checkboxListDevices.isSelected();
    boolean dataFromInput = false;
    if (isMulticast) {
      try {
        int groupId = Integer.parseInt((String)this.comboEUUID.getValue());
        if (groupId < 1 || groupId > 65535) {
          valid = false;
          error = "Multicast group must be a value between 1 and 65535";
        } else {
          target = ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN).putShort((short)groupId).array();
        } 
      } catch (Exception e) {
        valid = false;
        error = "Please Select a Multicast group";
      } 
    } else if (sendToAllEDs) {
      devices = (List<byte[]>)DevicesStorage.getInstance().getDevices().stream().filter(ed -> (ed.getStatus() == 2)).map(UUEndDevice::getUuid).collect(Collectors.toList());
    } else if (sendToListOfEDs) {
      String[] uueds = this.inputEDUUIDs.getText().replaceAll("-", "").split("\\s+");
      try {
        devices = (List<byte[]>)Arrays.<String>stream(uueds).map(uueid -> DatatypeConverter.parseHexBinary(uueid)).collect(Collectors.toList());
        for (byte[] uueid : devices) {
          if (uueid.length != 16) {
            error = error + "Invalid UUEID format. End Devices must include 32 Hexadecimal characters\n";
            valid = false;
            break;
          } 
        } 
      } catch (Exception e) {
        error = error + "Invalid UUEID format. End Devices must include 32 Hexadecimal characters\n";
        valid = false;
      } 
    } else {
      try {
        target = ((UUEndDevice)this.comboEds.get(this.comboEUUID.getSelectionModel().getSelectedIndex())).getUuid();
      } catch (Exception e) {
        e.printStackTrace();
        error = error + "Please Select an End Device\n";
        valid = false;
      } 
    } 
    try {
      if (data == null) {
        dataFromInput = true;
        data = DatatypeConverter.parseHexBinary(this.inputDownlinkData.getText());
        if (data.length == 0) {
          error = error + "Please type your data\n";
          valid = false;
        } 
      } 
    } catch (Exception e) {
      error = error + "Invalid format.HEX Data must include only Hexadecimal characters\n";
      valid = false;
    } 
    if (target != null) {
      if (isMulticast) {
        if (target.length == 2) {
          downlinks.add(new PSMulticastDownlink(data, target, endpoint));
        } else {
          error = error + "Multicast group ID must be 2 bytes\n";
          valid = false;
        } 
      } else if (target.length == 16) {
        downlinks.add(new PSDownlink(data, target, endpoint));
      } else {
        error = error + "Please Select an End Device\n";
        valid = false;
      } 
    } else if (sendToAllEDs || sendToListOfEDs) {
      if (devices.size() > 0) {
        for (byte[] uueid : devices) {
          downlinks.add(new PSDownlink(data, uueid, endpoint));
          try {
            Thread.sleep(1L);
          } catch (InterruptedException e) {
            e.printStackTrace();
          } 
        } 
      } else {
        error = error + "No End Devices Found";
        valid = false;
      } 
    } 
    if (valid) {
      for (PSDownlink downlink : downlinks)
        this.messageController.sendMessage((PSMessage)downlink); 
      setupDataInput();
      this.commandStatusReceiver.setLastStatus("Downlink Sent");
      if (dataFromInput) {
        this.inputDataHistoryHandler.addCommandToHistory(DatatypeConverter.printHexBinary(data));
        this.inputDataHistoryHandler.resetPosition();
      } 
    } else {
      Alert alert = new Alert(Alert.AlertType.ERROR);
      alert.setTitle("Couldn't create downlink");
      alert.setContentText(error);
      alert.showAndWait();
    } 
    return valid;
  }
  
  @FXML
  public void btnSendDownLinkPressed() {
    int port = this.checkboxAck.isSelected() ? 1 : 0;
    validateAndSendDownlink((byte[])null, port);
  }
  
  @FXML
  public void btnExtraDownlinkActionPressed() {
    int port = this.checkboxAck.isSelected() ? 1 : 0;
    String atCmd = this.inputDownlinkExtraAction.getText();
    byte[] atCmdData = PSDownlinkATCommand.createDownlinkATData(atCmd);
    if (atCmdData != null) {
      boolean valid = validateAndSendDownlink(atCmdData, port);
      if (valid) {
        cleanInput(this.inputDownlinkExtraAction, "AT");
        setTextFieldHasError(this.inputDownlinkExtraAction, false);
        this.inputExtraHistoryHandler.addCommandToHistory(atCmd);
        this.inputExtraHistoryHandler.resetPosition();
      } 
    } else {
      setTextFieldHasError(this.inputDownlinkExtraAction, true);
    } 
  }
  
  @FXML
  public void selectFirmwareFilePressed() {
    FileChooser fileChooser = new FileChooser();
    fileChooser.setTitle("Open Firmware File");
    File file = fileChooser.showOpenDialog(this.tableView.getScene().getWindow());
    if (file != null)
      if (file.length() <= 262144L) {
        this.currentFirmwareFile = file;
        this.labelFileSelected.setText(file.getName());
      } else {
        displayErrorMessage("File is too large", "Max file size allowed is 256KB");
      }  
  }
  
  @FXML
  public void sendFirmwarePressed() {
    if (this.currentFirmwareFile != null) {
      try {
        byte[] content = Files.readAllBytes(this.currentFirmwareFile.toPath());
        validateAndSendDownlink(content, 255);
      } catch (IOException e) {
        displayErrorMessage("Error opening file", "Couldn't open file");
      } 
    } else {
      displayErrorMessage("Select file", "Please select a file first");
    } 
  }
  
  public void doReloadView() {
    synchronized (this.data) {
      PSDownlink selectedDL = this.tableView.getSelectionModel().getSelectedItem();
      PSMessageStorage ms = PSMessageStorage.getInstance();
      List<PSDownlink> downlinks = ms.getDownlinks(500);
      this.data.clear();
      this.data.addAll(downlinks);
      Collections.reverse(this.data);
      this.tableView.sort();
      this.tableView.getSelectionModel().select(selectedDL);
      Collection<?> collection = null;
      boolean isMulticast = this.checkboxMulticast.isSelected();
      if (isMulticast) {
        collection = this.devicesController.getMulticastGroups().keySet();
      } else {
        collection = DevicesStorage.getInstance().getDevices();
      } 
      if (this.comboEUUID.getItems().size() != collection.size())
        setupIdInput(isMulticast); 
    } 
  }
  
  public void btnClearPressed() {
    PSMessageStorage.getInstance().clearDownlinks();
    doReloadView();
  }
  
  public void btnRefreshPressed() {
    doReloadView();
  }
  
  public void copyDownlinksToClipboard(boolean onlyFailed) {
    ClipboardContent clipboardContent = new ClipboardContent();
    synchronized (this.data) {
      String clipStr = "";
      boolean copyItem = false;
      for (PSDownlink dl : this.data) {
        copyItem = false;
        if (onlyFailed) {
          Integer ackStatus = dl.getAckStatus();
          if (ackStatus != null && 
            ackStatus.intValue() != 0)
            copyItem = true; 
        } else {
          copyItem = true;
        } 
        if (copyItem)
          clipStr = clipStr + clipStr + "\n"; 
      } 
      clipboardContent.putString(clipStr);
    } 
    Clipboard.getSystemClipboard().setContent(clipboardContent);
  }
  
  public void btnCopyFailedPressed() {
    copyDownlinksToClipboard(true);
  }
  
  public void btnCopyAllPressed() {
    copyDownlinksToClipboard(false);
  }
  
  public void comboEUUIDChanged() {
    boolean isMulticast = this.checkboxMulticast.isSelected();
    if (!isMulticast && 
      this.comboEUUID.getSelectionModel().getSelectedItem() != null) {
      Optional<UUEndDevice> optionalEd = this.comboEds.stream().filter(ed -> ed.toString().equals(this.comboEUUID.getSelectionModel().getSelectedItem().toString())).findFirst();
      if (optionalEd.isPresent()) {
        int idx = this.comboEds.indexOf(optionalEd.get());
        this.comboEUUID.getSelectionModel().select(idx);
      } 
    } 
  }
}
