package ubiik.wp.view;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.scene.Node;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonType;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.Label;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import ubiik.utils.view.TableUtils;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MCSController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.devices.DevicesStorage;
import ubiik.wp.model.devices.UUEndDevice;
import ubiik.wp.model.devices.mcs.MCSSetting;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandMCSSetting;
import ubiik.wp.model.messages.pscommands.PSCommandMCSValue;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseGetScheduleTransfer;

public class MCSViewController extends BaseViewController implements PSCommandDelegate {
  private static final double WIDTH_COLUMN_SELECT = 0.03D;
  
  private static final double WIDTH_COLUMN_STATUS = 0.072D;
  
  private static final double WIDTH_COLUMN_UUEID = 0.2D;
  
  private static final double WITDH_COLUMN_IDX = 0.05D;
  
  private static final int DL_MCS = 0;
  
  private static final int UL_MCS = 1;
  
  @FXML
  private TableView<UUEndDevice> tableView;
  
  private final ObservableList<UUEndDevice> data = FXCollections.observableArrayList();
  
  private Set<UUEndDevice> selectedDevices;
  
  private MCSController mcsController;
  
  public MCSViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
    this.selectedDevices = new HashSet<>();
    this.mcsController = new MCSController(devicesController);
  }
  
  public void initialize() {
    this.tableView.getColumns().clear();
    TableColumn<UUEndDevice, Integer> idxCol = DevicesViewController.createIdxCol(this.tableView, 0.05D);
    TableColumn<UUEndDevice, String> deviceCol = DevicesViewController.createNameCol(this.tableView, 0.2D);
    TableColumn<UUEndDevice, Byte> statusCol = DevicesViewController.createStatusCol(this.tableView, 0.072D);
    TableColumn<UUEndDevice, Boolean> dlMCSAllowedCol = new TableColumn<>("Allowed Downlink MCS [Kbps]");
    TableColumn<UUEndDevice, Boolean> ulMCSAllowedCol = new TableColumn<>("Allowed Uplink MCS [Kbps]");
    TableColumn<UUEndDevice, UUEndDevice> selectColumn = TableUtils.column("", javafx.beans.property.ReadOnlyObjectWrapper::new, 10.0D);
    selectColumn.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.03D));
    selectColumn.setSortable(false);
    selectColumn.setCellFactory(col -> {
          final CheckBox cb = new CheckBox();
          TableCell<UUEndDevice, UUEndDevice> cell = new TableCell<UUEndDevice, UUEndDevice>() {
              public void updateItem(UUEndDevice ed, boolean empty) {
                super.updateItem(ed, empty);
                if (empty) {
                  setGraphic((Node)null);
                } else {
                	cb.setSelected(MCSViewController.this.selectedDevices.contains(ed));
                	setGraphic(cb);
                } 
              }
            };
            
            cb.setOnMousePressed(e -> {
            	endDeviceItemSelected(cell.getTableRow().getItem(), cb.isPressed());
            });
          return cell;
        });
    double mcsColWidth = 0.648D / (MCSSetting.DL_MCS_NAMES_SHORT.size() + MCSSetting.UL_MCS_NAMES_SHORT.size());
    int i;
    for (i = 0; i < MCSSetting.DL_MCS_NAMES_SHORT.size(); i++)
      dlMCSAllowedCol.getColumns().add(createMCSAvailableCol(this.tableView, mcsColWidth, i, 0)); 
    for (i = 0; i < MCSSetting.UL_MCS_NAMES_SHORT.size(); i++)
      ulMCSAllowedCol.getColumns().add(createMCSAvailableCol(this.tableView, mcsColWidth, i, 1)); 
    this.tableView.getColumns().addAll(new TableColumn[] { idxCol, selectColumn, deviceCol, statusCol, dlMCSAllowedCol, ulMCSAllowedCol });
    this.tableView.setItems(this.data);
  }
  
  private synchronized void endDeviceItemSelected(UUEndDevice ed, boolean selected) {
    if (selected) {
      this.selectedDevices.add(ed);
    } else {
      this.selectedDevices.remove(ed);
    } 
  }
  
  public static TableColumn<UUEndDevice, Boolean> createMCSAvailableCol(TableView<UUEndDevice> tableView, double widthProperty, int flagIdx, int mcsType) {
    String title;
    if (mcsType == 0) {
      title = MCSSetting.DL_MCS_NAMES_SHORT.get(flagIdx);
    } else if (mcsType == 1) {
      title = MCSSetting.UL_MCS_NAMES_SHORT.get(flagIdx);
    } else {
      return null;
    } 
    TableColumn<UUEndDevice, Boolean> mcsCol = new TableColumn<>(title);
    mcsCol.prefWidthProperty().bind(tableView.widthProperty().multiply(widthProperty));
    mcsCol.setSortable(true);
    if (mcsType == 0) {
      mcsCol.setCellValueFactory(c -> (((UUEndDevice)c.getValue()).getMCSSetting() != null) ? new SimpleBooleanProperty(((UUEndDevice)c.getValue()).getMCSSetting().getIndexesDLMCSAvailable().contains(Integer.valueOf(flagIdx))) : null);
    } else if (mcsType == 1) {
      mcsCol.setCellValueFactory(c -> (((UUEndDevice)c.getValue()).getMCSSetting() != null) ? new SimpleBooleanProperty(((UUEndDevice)c.getValue()).getMCSSetting().getIndexesULMCSAvailable().contains(Integer.valueOf(flagIdx))) : null);
    } 
    return mcsCol;
  }
  
  public void btnRefreshPressed() {
    doReloadView();
  }
  
  public synchronized void doReloadView() {
    UUEndDevice selectedDevice = this.tableView.getSelectionModel().getSelectedItem();
    Collection<UUEndDevice> endDevices = DevicesStorage.getInstance().getDevices();
    this.data.clear();
    for (UUEndDevice uuEndDevice : endDevices)
      this.data.add(uuEndDevice); 
    this
      .selectedDevices = (Set<UUEndDevice>)this.selectedDevices.stream().filter(ed -> endDevices.contains(ed)).collect(Collectors.toSet());
    this.tableView.sort();
    this.tableView.refresh();
    this.tableView.getSelectionModel().select(selectedDevice);
  }
  
  public synchronized void btnReadAllMCSPressed() {
    Collection<UUEndDevice> endDevices = DevicesStorage.getInstance().getDevices();
    
    for (UUEndDevice endDevice : endDevices) {
      sendCommandAndUpdateStatusBar((PSCommand)new PSCommandMCSValue(this, (byte)0, endDevice
            .getUuid()));
    } 
    for (UUEndDevice endDevice : endDevices) {
        sendCommandAndUpdateStatusBar((PSCommand)new PSCommandMCSSetting(this, (byte)0, endDevice
              .getUuid()));
      }
  }
  
  public synchronized void btnSetMCSPressed() {
    Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
    alert.setTitle("Set Allowed MCS");
    alert.setHeaderText("Please, select the allowed MCS for uplink and downlink");
    VBox content = new VBox();
    List<CheckBox> ulMCSCheckboxes = new ArrayList<>();
    List<CheckBox> dlMCSCheckboxes = new ArrayList<>();
    FlowPane containerAllowedDLMCSValues = new FlowPane();
    FlowPane containerAllowedULMCSValues = new FlowPane();
    containerAllowedDLMCSValues.setVgap(10.0D);
    containerAllowedDLMCSValues.setHgap(10.0D);
    containerAllowedDLMCSValues.setPadding(new Insets(10.0D, 0.0D, 10.0D, 0.0D));
    containerAllowedULMCSValues.setVgap(10.0D);
    containerAllowedULMCSValues.setHgap(10.0D);
    containerAllowedULMCSValues.setPadding(new Insets(10.0D, 0.0D, 10.0D, 0.0D));
    setMCSSettingsView((UUEndDevice)null, (ChoiceBox<String>)null, (ChoiceBox<String>)null, (ChoiceBox<String>)null, ulMCSCheckboxes, dlMCSCheckboxes, containerAllowedDLMCSValues, containerAllowedULMCSValues);
    content.getChildren().add(new Label("Allowed Downlink MCS values"));
    content.getChildren().add(containerAllowedDLMCSValues);
    content.getChildren().add(new Label("Allowed uplink MCS values"));
    content.getChildren().add(containerAllowedULMCSValues);
    alert.getDialogPane().setContent(content);
    Optional<ButtonType> result = alert.showAndWait();
    if (result.get() == ButtonType.OK) {
      int flagsUl = getFlagsFromCheckboxes(ulMCSCheckboxes);
      int flagsDl = getFlagsFromCheckboxes(dlMCSCheckboxes);
      if (flagsUl != 0 && flagsDl != 0) {
        for (UUEndDevice endDevice : this.selectedDevices) {
          PSCommandMCSSetting cmd = new PSCommandMCSSetting(this, (byte)1, endDevice.getUuid(), (short)flagsUl, (short)flagsDl);
          sendCommandAndUpdateStatusBar((PSCommand)cmd);
        } 
        displayInfoMessage("MCS commands sent", "Commands sent to " + this.selectedDevices.size() + " end devices");
      } else {
        displayErrorMessage("Select values", "Please select at least one value for Uplink and Downlink MCS");
      } 
    } 
  }
  
  public static int getFlagsFromCheckboxes(List<CheckBox> checkboxes) {
    int flags = 0;
    for (int i = checkboxes.size(); i > 0; i--) {
      flags <<= 1;
      if (((CheckBox)checkboxes.get(i - 1)).isSelected())
        flags++; 
    } 
    return flags;
  }
  
  public void commandandReceivedResponse(PSCommand command) {
    if (command.getResponse().isSuccess()) {
      if (command instanceof ubiik.wp.model.messages.pscommands.PSCommandGetScheduleTransfer) {
        if (command.getResponse() instanceof PSCommandResponseGetScheduleTransfer) {
          PSCommandResponseGetScheduleTransfer cmdResp = (PSCommandResponseGetScheduleTransfer)command.getResponse();
          this.devicesController.setScheduleTransferForDevice(cmdResp.getDeviceUuid(), 
        		  cmdResp.getFlag(), 
        		  cmdResp.getWindow(), 
        		  cmdResp.getInterval());
        } 
      } else if (command instanceof PSCommandMCSValue) {
        this.mcsController.processMCSValuesCommandResponse((PSCommandMCSValue)command);
      } else if (command instanceof PSCommandMCSSetting) {
        this.mcsController.processMCSCommandResponse((PSCommandMCSSetting)command);
      } 
    } else {
      setStatusBarCommandFailed(command);
    } 
  }
  
  public static void setMCSSettingsView(UUEndDevice uuEndDevice, ChoiceBox<String> choiceboxULMCSValues, ChoiceBox<String> choiceboxULMCSValuesNB, ChoiceBox<String> choiceboxDLMCSValues, List<CheckBox> ulMCSCheckboxes, List<CheckBox> dlMCSCheckboxes, Pane containerAllowedDLMCSValues, Pane containerAllowedULMCSValues) {
    short dlMcsFlags = 0;
    short ulMcsFlags = 0;
    if (uuEndDevice != null) {
      MCSSetting mcsSettings = uuEndDevice.getMCSSetting();
      if (mcsSettings != null) {
        dlMcsFlags = mcsSettings.getDownlinkMcsFlags();
        ulMcsFlags = mcsSettings.getUplinkMcsFlags();
      } 
    } 
    setUpCheckboxesForMCSFlags(MCSSetting.DL_MCS_NAMES, dlMCSCheckboxes, dlMcsFlags, containerAllowedDLMCSValues);
    setUpCheckboxesForMCSFlags(MCSSetting.UL_MCS_NAMES, ulMCSCheckboxes, ulMcsFlags, containerAllowedULMCSValues);
    if (choiceboxDLMCSValues != null)
      choiceboxDLMCSValues.getItems().addAll(MCSSetting.DL_MCS_NAMES); 
    if (choiceboxULMCSValues != null)
      choiceboxULMCSValues.getItems().addAll(MCSSetting.UL_MCS_NAMES.subList(0, 4)); 
    if (choiceboxULMCSValuesNB != null)
      choiceboxULMCSValuesNB.getItems().addAll(MCSSetting.UL_MCS_NAMES.subList(4, MCSSetting.UL_MCS_NAMES.size())); 
  }
  
  public static void setUpCheckboxesForMCSFlags(List<String> names, List<CheckBox> checkboxes, short mcsFlags, Pane container) {
    short currVal = 1;
    checkboxes.clear();
    for (int i = 0; i < names.size(); i++) {
      CheckBox cb = new CheckBox(names.get(i));
      cb.setSelected(((short)(currVal & mcsFlags) != 0));
      cb.setMinWidth(150.0D);
      checkboxes.add(cb);
      currVal = (short)(currVal << 1);
      container.getChildren().add(cb);
    } 
  }
}
