package ubiik.wp.view;

import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandSDRSet;
import ubiik.wp.model.sdr.SDRParameter;

public class SDRParameterViewController extends BaseViewController implements PSCommandDelegate {
  private int parameterId;
  
  @FXML
  private TextField input;
  
  @FXML
  private Label labelName;
  
  @FXML
  private Label labelDefault;
  
  @FXML
  private Label labelMin;
  
  @FXML
  private Label labelMax;
  
  @FXML
  private Label labelStep;
  
  @FXML
  private Label labelDescription;
  
  public SDRParameterViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver, int parameterId) {
    super(devicesController, messageController, commandStatusReceiver);
    this.parameterId = parameterId;
  }
  
  public void initialize() {
    SDRParameter param = this.devicesController.getSDRParameterForId(this.parameterId);
    if (param != null) {
      this.labelName.setText(param.getName());
      this.labelMin.setText("Min: " + param.getMinStr());
      this.labelMax.setText("Max: " + param.getMaxStr());
      this.labelStep.setText("Step: " + param.getStepStr());
      this.labelDefault.setText("Default: " + param.getDefaultStr());
      this.labelDescription.setText(param.getDescription());
      this.input.setText(param.getValueStr());
    } 
  }
  
  public void commandandReceivedResponse(PSCommand command) {
    if (command.getResponse().isSuccess()) {
      setStatusBarCommandSuccess(command);
      if (command instanceof PSCommandSDRSet) {
        this.devicesController.setSDRParameterValue(((PSCommandSDRSet)command)
            .getSdrParameter().getId(), ((PSCommandSDRSet)command)
            .getSdrParameter().getValueStr());
        setTextFieldIsSuccessful(this.input, true);
      } 
    } else {
      displayErrorMessage("Error", "Set " + command.getTypeString() + " failed");
      setStatusBarCommandFailed(command);
    } 
  }
  
  public void buttonSetParameterPressed() {
    PSCommandSDRSet cmd = null;
    synchronized (this) {
      SDRParameter param = this.devicesController.getSDRParameterForId(this.parameterId);
      boolean valid = param.setValuFromString(this.input.getText());
      setTextFieldIsSuccessful(this.input, false);
      setTextFieldHasError(this.input, !valid);
      if (valid)
        cmd = new PSCommandSDRSet(this, param); 
    } 
    if (cmd != null)
      sendCommandAndUpdateStatusBar((PSCommand)cmd); 
  }
  
  public void updateInput() {
    SDRParameter param = this.devicesController.getSDRParameterForId(this.parameterId);
    this.input.setText(param.getValueStr());
  }
  
  public void doReloadView() {}
}
