package ubiik.wp.view;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.layout.Pane;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.sdr.SDRParameter;

public class SDRViewController extends BaseViewController {
  @FXML
  private Pane txContainer;

  @FXML
  private Pane rxIinContainer;

  @FXML
  private Pane rxOutContainer;

  private HashMap<Integer, SDRParameterViewController> parameterVCs;

  public SDRViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
    this.parameterVCs = new HashMap<>();
  }

  public void initialize() {
    synchronized (this) {
      Collection<SDRParameter> parameters = this.devicesController.getSDRParameters();
      for (SDRParameter sdrParameter : parameters) {
        SDRParameterViewController vc = new SDRParameterViewController(this.devicesController, this.messageController, this.commandStatusReceiver, sdrParameter.getId());
        this.parameterVCs.put(Integer.valueOf(sdrParameter.getId()), vc);
        FXMLLoader loader = new FXMLLoader(getClass().getResource("/main/resources/ubiik/wp/admin/SDRParameterView.fxml"));
        loader.setController(vc);
        try {
          Pane newPane = loader.<Pane>load();
          Pane parent = null;
          switch (sdrParameter.getCategory()) {
            case "tx":
              parent = this.txContainer;
              break;
            case "rx_in":
              parent = this.rxIinContainer;
              break;
            case "rx_out":
              parent = this.rxOutContainer;
              break;
            default:
              parent = this.txContainer;
              break;
          }
          parent.getChildren().add(newPane);
        } catch (IOException e) {
          e.printStackTrace();
        }
      }
    }
  }

  public void doReloadView() {
    for (SDRParameterViewController vc : this.parameterVCs.values())
      vc.updateInput();
  }
}
