package ubiik.wp.view;

import javafx.fxml.FXML;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.Pane;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.devices.UUSIB0;
import ubiik.wp.model.devices.UUSIB1;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandReadSIB;
import ubiik.wp.model.messages.pscommands.PSCommandSetSIB0and1;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponse;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseReadSIB;

public class SIBViewController extends BaseViewController implements PSCommandDelegate {
  @FXML
  private TextField input_frame_duration;
  
  @FXML
  private Label label_frame_duration;
  
  @FXML
  private TextField input_even_ra_mcs;
  
  @FXML
  private Label label_even_ra_mcs;
  
  @FXML
  private TextField input_odd_ra_mcs;
  
  @FXML
  private Label label_odd_ra_mcs;
  
  @FXML
  private CheckBox checkboxIsSIB1;
  
  @FXML
  private Label label_include_sib1;
  
  @FXML
  private TextField input_hop_first_ch;
  
  @FXML
  private Label label_hop_first_ch;
  
  @FXML
  private TextField input_hop_last_ch;
  
  @FXML
  private Label label_hop_last_ch;
  
  @FXML
  private TextField input_hop_ch_nb;
  
  @FXML
  private Label label_hop_ch_nb;
  
  @FXML
  private TextField input_nb_ncell_hop_ch;
  
  @FXML
  private Label label_nb_ncell_hop_ch;
  
  @FXML
  private TextField input_ncell_ch;
  
  @FXML
  private Pane ncellHopChContainer;
  
  public SIBViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
  }
  
  public void initialize() {}
  
  @FXML
  public void input_nb_ncell_hop_chChanged() {}
  
  @FXML
  public synchronized void btnWriteSIBPressed() {
    try {
      UUSIB0 sib0 = null;
      UUSIB1 sib1 = null;
      Integer i_frame_duration = validateAndGetIntegerTextFieldMinMax(this.input_frame_duration, 0, 3);
      Integer i_even_ra_mcs = validateAndGetIntegerTextFieldSetValues(this.input_even_ra_mcs, UUSIB0.EVEN_RA_MCS_VALID_VALUES);
      Integer i_odd_ra_mcs = validateAndGetIntegerTextFieldSetValues(this.input_odd_ra_mcs, UUSIB0.ODD_RA_MCS_VALID_VALUES);
      boolean includeSIB1 = this.checkboxIsSIB1.isSelected();
      if (includeSIB1) {
        Integer i_hop_first_ch = validateAndGetIntegerTextFieldMinMax(this.input_hop_first_ch, 0, 10000);
        Integer i_hop_last_ch = validateAndGetIntegerTextFieldMinMax(this.input_hop_last_ch, 0, 255);
        Integer i_hop_ch_nb = validateAndGetIntegerTextFieldMinMax(this.input_hop_ch_nb, 0, 63);
        short hop_first_ch = i_hop_first_ch.shortValue();
        short hop_last_ch = i_hop_last_ch.shortValue();
        short hop_ch_nb = i_hop_ch_nb.shortValue();
        sib1 = new UUSIB1(hop_first_ch, hop_last_ch, hop_ch_nb, (short)0, new short[0]);
      } 
      short frame_duration = i_frame_duration.shortValue();
      short even_ra_mcs = i_even_ra_mcs.shortValue();
      short odd_ra_mcs = i_odd_ra_mcs.shortValue();
      sib0 = new UUSIB0(frame_duration, (short)-1, even_ra_mcs, odd_ra_mcs, sib1);
      PSCommandSetSIB0and1 cmd = new PSCommandSetSIB0and1(this, sib0);
      sendCommandAndUpdateStatusBar((PSCommand)cmd);
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }
  
  @FXML
  public void btnReadSIBPressed() {
    //readSIB();
  }
  
  public synchronized void readSIB() {
    //PSCommandReadSIB cmd = new PSCommandReadSIB(this);
    //sendCommandAndUpdateStatusBar((PSCommand)cmd);
  }
  
  public void commandandReceivedResponse(PSCommand command) {
    if (command.getResponse().isSuccess()) {
      setStatusBarCommandSuccess(command);
      if (command instanceof PSCommandSetSIB0and1) {
        this.devicesController.setSIB(((PSCommandSetSIB0and1)command)
            .getSib0());
      } else if (command instanceof PSCommandReadSIB) {
        PSCommandResponse resp = command.getResponse();
        if (resp instanceof PSCommandResponseReadSIB) {
          this.devicesController.setSIB(((PSCommandResponseReadSIB)resp)
              .getSib0());
        } else {
          displayErrorMessage("Error", "Error in command response format");
        } 
      } 
    } else {
      displayErrorMessage("Error", "Command failed");
      setStatusBarCommandFailed(command);
    } 
  }
  
  public void doReloadView() {
    UUSIB0 sib0 = this.devicesController.getSIB();
    UUSIB1 sib1 = sib0.getSib1();
    this.label_frame_duration.setText(Short.toString(sib0.getFrame_duration()));
    this.label_even_ra_mcs.setText(Short.toString(sib0.getEven_ra_mcs()));
    this.label_odd_ra_mcs.setText(Short.toString(sib0.getOdd_ra_mcs()));
    this.label_include_sib1.setText((new Boolean(sib0.isSib1_present())).toString());
    if (sib0.isSib1_present()) {
      this.label_hop_first_ch.setText(Short.toString(sib1.getHop_first_ch()));
      this.label_hop_last_ch.setText(Short.toString(sib1.getHop_ch_spacing()));
      this.label_hop_ch_nb.setText(Short.toString(sib1.getHop_ch_nb()));
    } else {
      this.label_hop_first_ch.setText("-");
      this.label_hop_last_ch.setText("-");
      this.label_hop_ch_nb.setText("-");
    } 
  }
}
