package ubiik.wp.view;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.Clipboard;
import javafx.scene.input.ClipboardContent;
import javafx.util.Callback;
import ubiik.utils.view.ColumnFormatter;
import ubiik.utils.view.TableUtils;
import ubiik.wp.controller.PSMessageStorage;
import ubiik.wp.model.messages.PSUplink;

public class UplinksViewController extends RefreshableViewController {
  private static final int UPLINKS_IN_LIST = 500;
  
  @FXML
  private TableView<PSUplink> tableView;
  
  private final ObservableList<PSUplink> data = FXCollections.observableArrayList();
  
  public void initialize() {
    this.tableView.getColumns().clear();
    TableColumn<PSUplink, String> idCol = new TableColumn<>("ID");
    TableColumn<PSUplink, String> deviceCol = new TableColumn<>("Device");
    TableColumn<PSUplink, String> dataCol = new TableColumn<>("Data");
    TableColumn<PSUplink, Integer> dataSizeCol = new TableColumn<>("Bytes");
    TableColumn<PSUplink, Integer> rssiCol = new TableColumn<>("RSSI");
    TableColumn<PSUplink, Long> foiCol = new TableColumn<>("Foi");
    TableColumn<PSUplink, String> modeCol = new TableColumn<>("Mode");
    TableColumn<PSUplink, Date> datelCol = new TableColumn<>("Date");
    idCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.07D));
    deviceCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.29D));
    dataCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.29D));
    dataSizeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.05D));
    rssiCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.05D));
    foiCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.05D));
    modeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    datelCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.15D));
    idCol.setSortable(true);
    deviceCol.setSortable(true);
    dataCol.setSortable(false);
    dataSizeCol.setSortable(false);
    rssiCol.setSortable(true);
    foiCol.setSortable(true);
    modeCol.setSortable(true);
    datelCol.setSortable(true);
    idCol.setCellValueFactory(new PropertyValueFactory<>("applicationId"));
    deviceCol.setCellValueFactory(new PropertyValueFactory<>("targetName"));
    dataCol.setCellValueFactory(new PropertyValueFactory<>("applicationDataString"));
    dataSizeCol.setCellValueFactory(new PropertyValueFactory<>("applicationDataSize"));
    rssiCol.setCellValueFactory(new PropertyValueFactory<>("rssi"));
    foiCol.setCellValueFactory(new PropertyValueFactory<>("foi"));
    modeCol.setCellValueFactory(new PropertyValueFactory<>("endpointStr"));
    datelCol.setCellValueFactory(new PropertyValueFactory<>("timestamp"));
    datelCol.setCellFactory((Callback<TableColumn<PSUplink, Date>, TableCell<PSUplink, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    this.tableView.getColumns().addAll(new TableColumn[] { idCol, deviceCol, dataCol, dataSizeCol, rssiCol, foiCol, modeCol, datelCol });
    this.tableView.setItems(this.data);
    reloadView();
    this.tableView.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
    TableUtils.installCopyPasteHandler(this.tableView);
  }
  
  public void doReloadView() {
    PSUplink selectedUL = this.tableView.getSelectionModel().getSelectedItem();
    PSMessageStorage ms = PSMessageStorage.getInstance();
    List<PSUplink> uplinks = ms.getUplinks(500);
    this.data.clear();
    this.data.addAll(uplinks);
    Collections.reverse(this.data);
    this.tableView.sort();
    this.tableView.getSelectionModel().select(selectedUL);
  }
  
  public void btnClearPressed() {
    PSMessageStorage.getInstance().clearUplinks();
    doReloadView();
  }
  
  public void btnRefreshPressed() {
    doReloadView();
  }
  
  public void btnCopyAllPressed() {
    ClipboardContent clipboardContent = new ClipboardContent();
    synchronized (this.data) {
      String clipStr = "";
      for (PSUplink ul : this.data)
        clipStr = clipStr + clipStr + "\n"; 
      clipboardContent.putString(clipStr);
    } 
    Clipboard.getSystemClipboard().setContent(clipboardContent);
  }
}
